@@ -1,5 +1,4 @@
 /* Hiearchic listboxes browser dialog commons */
-/* $Id: hierbox.c,v 1.201.2.7 2005/05/01 21:05:57 jonas Exp $ */
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
@@ -17,10 +16,9 @@
 #include "bfu/msgbox.h"
 #include "bfu/text.h"
 #include "config/kbdbind.h"
-#include "dialogs/download.h"
 #include "intl/gettext/libintl.h"
 #include "protocol/uri.h"
-#include "sched/task.h"
+#include "session/task.h"
 #include "terminal/screen.h"
 #include "terminal/tab.h"
 #include "terminal/terminal.h"
@@ -84,13 +82,13 @@ add_listbox_item(struct hierbox_browser *browser, struct listbox_item *root,
 static inline struct listbox_item *
 replace_listbox_item(struct listbox_item *item, struct listbox_data *data)
 {
-	struct listbox_item *box;
+	struct listbox_item *new_item;
 
-	box = traverse_listbox_items_list(item, data, 1, 1, NULL, NULL);
-	if (item != box) return box;
+	new_item = traverse_listbox_items_list(item, data, 1, 1, NULL, NULL);
+	if (item != new_item) return new_item;
 
-	box = traverse_listbox_items_list(item, data, -1, 1, NULL, NULL);
-	return (item == box) ? NULL : box;
+	new_item = traverse_listbox_items_list(item, data, -1, 1, NULL, NULL);
+	return (item == new_item) ? NULL : new_item;
 }
 
 void
@@ -100,51 +98,45 @@ done_listbox_item(struct hierbox_browser *browser, struct listbox_item *box_item
 
 	assert(box_item && list_empty(box_item->child));
 
-	/* If we are removing the top or the selected box we have to figure out
-	 * a replacement. */
+	/* The option dialog needs this test */
+	if (box_item->next) {
+		/* If we are removing the top or the selected box
+		 * we have to figure out a replacement. */
 
-	foreach (box_data, browser->boxes) {
-		if (box_data->sel == box_item)
-			box_data->sel = replace_listbox_item(box_item, box_data);
+		foreach (box_data, browser->boxes) {
+			if (box_data->sel == box_item)
+				box_data->sel = replace_listbox_item(box_item,
+				                                     box_data);
 
-		if (box_data->top == box_item)
-			box_data->top = replace_listbox_item(box_item, box_data);
+			if (box_data->top == box_item)
+				box_data->top = replace_listbox_item(box_item,
+				                                     box_data);
+		}
+
+		del_from_list(box_item);
+
+		update_hierbox_browser(browser);
 	}
 
-	/* The option dialog needs this test */
-	if (box_item->next) del_from_list(box_item);
-
 	mem_free(box_item);
-	update_hierbox_browser(browser);
 }
 
 
 static void
-recursively_set_expanded(struct listbox_item *box, int expanded)
+recursively_set_expanded(struct listbox_item *item, int expanded)
 {
 	struct listbox_item *child;
 
-	if (box->type != BI_FOLDER)
+	if (item->type != BI_FOLDER)
 		return;
 
-	box->expanded = expanded;
+	item->expanded = expanded;
 
-	foreach (child, box->child)
+	foreach (child, item->child)
 		recursively_set_expanded(child, expanded);
 }
 
-static int
-test_search(struct listbox_item *item, void *data_, int *offset)
-{
-	struct listbox_context *listbox_context = data_;
-
-	listbox_context->offset--;
-
-	if (item == listbox_context->box->sel) *offset = 0;
-	return 0;
-}
-
-static t_handler_event_status
+static widget_handler_status_T
 hierbox_ev_kbd(struct dialog_data *dlg_data)
 {
 	struct hierbox_browser *browser = dlg_data->dlg->udata2;
@@ -152,7 +144,7 @@ hierbox_ev_kbd(struct dialog_data *dlg_data)
 	struct widget *widget = widget_data->widget;
 	struct listbox_data *box;
 	struct listbox_item *selected;
-	enum menu_action action;
+	enum menu_action action_id;
 	struct term_event *ev = dlg_data->term_event;
 
 	/* Check if listbox has something to say to this */
@@ -163,15 +155,15 @@ hierbox_ev_kbd(struct dialog_data *dlg_data)
 
 	box = get_dlg_listbox_data(dlg_data);
 	selected = box->sel;
-	action = kbd_action(KEYMAP_MENU, ev, NULL);
+	action_id = kbd_action(KEYMAP_MENU, ev, NULL);
 
-	if (action == ACT_MENU_SELECT) {
+	if (action_id == ACT_MENU_SELECT) {
 		if (!selected) return EVENT_PROCESSED;
 		if (selected->type != BI_FOLDER)
 			return EVENT_NOT_PROCESSED;
 		selected->expanded = !selected->expanded;
 
-	} else if (action == ACT_MENU_UNEXPAND) {
+	} else if (action_id == ACT_MENU_UNEXPAND) {
 		/* Recursively unexpand all folders */
 		if (!selected) return EVENT_PROCESSED;
 
@@ -184,24 +176,14 @@ hierbox_ev_kbd(struct dialog_data *dlg_data)
 			struct listbox_item *root = box->ops->get_root(selected);
 
 			if (root) {
-				struct listbox_context ctx;
-
-				memset(&ctx, 0, sizeof(ctx));
-				ctx.box = box;
-				ctx.offset = 1;
-
-				traverse_listbox_items_list(
-						root, box, 0, 1,
-						test_search, &ctx);
-				listbox_sel_move(dlg_data->widgets_data,
-					         ctx.offset);
+				listbox_sel(widget_data, root);
 			}
 
 		} else if (selected->type == BI_FOLDER) {
 			recursively_set_expanded(selected, 0);
 		}
 
-	} else if (action == ACT_MENU_EXPAND) {
+	} else if (action_id == ACT_MENU_EXPAND) {
 		/* Recursively expand all folders */
 
 		if (!selected || box->sel->type != BI_FOLDER)
@@ -209,7 +191,7 @@ hierbox_ev_kbd(struct dialog_data *dlg_data)
 
 		recursively_set_expanded(box->sel, 1);
 
-	} else if (action == ACT_MENU_SEARCH) {
+	} else if (action_id == ACT_MENU_SEARCH) {
 		if (!box->ops->match)
 			return EVENT_NOT_PROCESSED;
 
@@ -224,12 +206,12 @@ hierbox_ev_kbd(struct dialog_data *dlg_data)
 	if (browser->expansion_callback)
 		browser->expansion_callback();
 
-	display_widget(dlg_data, dlg_data->widgets_data);
+	display_widget(dlg_data, widget_data);
 
 	return EVENT_PROCESSED;
 }
 
-static t_handler_event_status
+static widget_handler_status_T
 hierbox_ev_init(struct dialog_data *dlg_data)
 {
 	struct hierbox_browser *browser = dlg_data->dlg->udata2;
@@ -251,7 +233,7 @@ hierbox_ev_init(struct dialog_data *dlg_data)
 	return EVENT_NOT_PROCESSED;	/* FIXME: is this correct ? --Zas */
 }
 
-static t_handler_event_status
+static widget_handler_status_T
 hierbox_ev_abort(struct dialog_data *dlg_data)
 {
 	struct listbox_data *box = get_dlg_listbox_data(dlg_data);
@@ -281,7 +263,7 @@ hierbox_ev_abort(struct dialog_data *dlg_data)
  * unselectable, instead one of the buttons below is always active. So, we
  * always first let the listbox catch the keypress and handle it, and if it
  * doesn't care, we pass it on to the button. */
-static t_handler_event_status
+static widget_handler_status_T
 hierbox_dialog_event_handler(struct dialog_data *dlg_data)
 {
 	struct term_event *ev = dlg_data->term_event;
@@ -420,7 +402,7 @@ done_listbox_context(void *context_)
 
 /* Info action */
 
-t_handler_event_status
+widget_handler_status_T
 push_hierbox_info_button(struct dialog_data *dlg_data, struct widget_data *button)
 {
 	struct listbox_data *box = get_dlg_listbox_data(dlg_data);
@@ -466,18 +448,18 @@ recursively_goto_listbox(struct session *ses, struct listbox_item *root,
 	struct listbox_item *item;
 
 	foreach (item, root->child) {
-		struct uri *uri;
-
 		if (item->type == BI_FOLDER) {
 			recursively_goto_listbox(ses, item, box);
 			continue;
+
+		} else if (item->type == BI_LEAF) {
+			struct uri *uri = box->ops->get_uri(item);
+
+			if (!uri) continue;
+
+			open_uri_in_new_tab(ses, uri, 1, 0);
+			done_uri(uri);
 		}
-
-		uri = box->ops->get_uri(item);
-		if (!uri) continue;
-
-		open_uri_in_new_tab(ses, uri, 1, 0);
-		done_uri(uri);
 	}
 }
 
@@ -489,24 +471,25 @@ goto_marked(struct listbox_item *item, void *data_, int *offset)
 	if (item->marked) {
 		struct session *ses = context->dlg_data->dlg->udata;
 		struct listbox_data *box = context->box;
-		struct uri *uri;
 
 		if (item->type == BI_FOLDER) {
 			recursively_goto_listbox(ses, item, box);
 			return 0;
+
+		} else if (item->type == BI_LEAF) {
+			struct uri *uri = box->ops->get_uri(item);
+
+			if (!uri) return 0;
+
+			open_uri_in_new_tab(ses, uri, 1, 0);
+			done_uri(uri);
 		}
-
-		uri = box->ops->get_uri(item);
-		if (!uri) return 0;
-
-		open_uri_in_new_tab(ses, uri, 1, 0);
-		done_uri(uri);
 	}
 
 	return 0;
 }
 
-t_handler_event_status
+widget_handler_status_T
 push_hierbox_goto_button(struct dialog_data *dlg_data,
 			 struct widget_data *button)
 {
@@ -530,13 +513,17 @@ push_hierbox_goto_button(struct dialog_data *dlg_data,
 	} else if (box->sel->type == BI_FOLDER) {
 		recursively_goto_listbox(ses, box->sel, box);
 
-	} else {
+	} else if (box->sel->type == BI_LEAF) {
 		struct uri *uri = box->ops->get_uri(box->sel);
 
 		if (uri) {
 			goto_uri(ses, uri);
 			done_uri(uri);
 		}
+
+	} else {
+		mem_free(context);
+		return EVENT_PROCESSED;
 	}
 
 	mem_free(context);
@@ -701,7 +688,7 @@ push_ok_delete_button(void *context_)
 		listbox_sel_move(context->widget_data, -1);
 }
 
-t_handler_event_status
+widget_handler_status_T
 push_hierbox_delete_button(struct dialog_data *dlg_data,
 			   struct widget_data *button)
 {
@@ -825,7 +812,7 @@ do_clear_browser(void *context_)
 				    delete_unused, context);
 }
 
-t_handler_event_status
+widget_handler_status_T
 push_hierbox_clear_button(struct dialog_data *dlg_data,
 			  struct widget_data *button)
 {
@@ -950,7 +937,7 @@ search_hierbox_browser(void *data, unsigned char *text)
 	mem_free(context);
 }
 
-t_handler_event_status
+widget_handler_status_T
 push_hierbox_search_button(struct dialog_data *dlg_data,
 			   struct widget_data *button)
 {
