@@ -1,11 +1,9 @@
-/* $Id: leds.h,v 1.17.2.1 2005/01/29 01:39:15 jonas Exp $ */
-
 #ifndef EL__BFU_LEDS_H
 #define EL__BFU_LEDS_H
 
 #ifdef CONFIG_LEDS
 
-#include "modules/module.h"
+#include "main/module.h"
 #include "util/color.h"
 
 struct session;
@@ -24,17 +22,16 @@ struct terminal;
  * assignment scheme in order to prevent conflicts. */
 
 struct led {
-	int number;
-	unsigned char value;
-
 	/* Private data. */
-	int used__;
+	/* 32 bits */
+	unsigned int used__:1;
+	unsigned int value_changed__:1;
+	unsigned int value__:8;
 };
 
 /* Per-session led panel structure. */
 struct led_panel {
 	struct led leds[LEDS_COUNT];
-	unsigned char leds_backup[LEDS_COUNT];
 };
 
 
@@ -48,6 +45,8 @@ void menu_leds_info(struct terminal *term, void *xxx, void *xxxx);
 
 struct led *register_led(struct session *ses, int number);
 void unregister_led(struct led *);
+void set_led_value(struct led *led, unsigned char value);
+void unset_led_value(struct led *led);
 
 #endif
 #endif
