@@ -1,5 +1,4 @@
 /* Menu system implementation. */
-/* $Id: menu.c,v 1.286.4.3 2005/09/14 13:24:34 jonas Exp $ */
 
 #ifndef _GNU_SOURCE
 #define _GNU_SOURCE /* XXX: we _WANT_ strcasestr() ! */
@@ -19,7 +18,7 @@
 #include "bfu/menu.h"
 #include "config/kbdbind.h"
 #include "intl/gettext/libintl.h"
-#include "sched/action.h"
+#include "session/session.h"
 #include "terminal/draw.h"
 #include "terminal/event.h"
 #include "terminal/kbd.h"
@@ -30,6 +29,7 @@
 #include "util/color.h"
 #include "util/conv.h"
 #include "util/memory.h"
+#include "viewer/action.h"
 
 /* Left and right main menu reserved spaces. */
 #define L_MAINMENU_SPACE	2
@@ -58,17 +58,18 @@ static unsigned char m_submenu[] = ">>";
 static int m_submenu_len = sizeof(m_submenu) - 1;
 
 /* Prototypes */
-static window_handler menu_handler;
-static window_handler mainmenu_handler;
+static window_handler_T menu_handler;
+static window_handler_T mainmenu_handler;
 
 
 static inline int
 count_items(struct menu_item *items)
 {
-	struct menu_item *item;
 	int i = 0;
 
 	if (items) {
+		struct menu_item *item;
+
 		foreach_menu_item (item, items) {
 			i++;
 		}
@@ -82,7 +83,7 @@ free_menu_items(struct menu_item *items)
 {
 	struct menu_item *item;
 
-	if (!items) return;
+	if (!items || !(items->flags & FREE_ANY)) return;
 
 	/* Note that flags & FREE_DATA applies only when menu is aborted;
 	 * it is zeroed when some menu field is selected. */
@@ -108,12 +109,12 @@ do_menu_selected(struct terminal *term, struct menu_item *items,
 		menu->data = data;
 		menu->size = count_items(items);
 		menu->hotkeys = hotkeys;
-#ifdef ENABLE_NLS
+#ifdef CONFIG_NLS
 		menu->lang = -1;
 #endif
 		refresh_hotkeys(term, menu);
 		add_window(term, menu_handler, menu);
-	} else if (items->flags & FREE_ANY) {
+	} else {
 		free_menu_items(items);
 	}
 }
@@ -128,9 +129,9 @@ static void
 select_menu_item(struct terminal *term, struct menu_item *it, void *data)
 {
 	/* We save these values due to delete_window() call below. */
-	menu_func func = it->func;
+	menu_func_T func = it->func;
 	void *it_data = it->data;
-	enum main_action action = it->action;
+	enum main_action action_id = it->action_id;
 
 	if (!mi_is_selectable(it)) return;
 
@@ -149,10 +150,10 @@ select_menu_item(struct terminal *term, struct menu_item *it, void *data)
 		}
 	}
 
-	if (action != ACT_MAIN_NONE && !func) {
+	if (action_id != ACT_MAIN_NONE && !func) {
 		struct session *ses = data;
 
-		do_action(ses, action, 1);
+		do_action(ses, action_id, 1);
 		return;
 	}
 
@@ -197,11 +198,11 @@ get_menuitem_rtext_width(struct terminal *term, struct menu_item *mi)
 	if (mi_is_submenu(mi)) {
 		rtext_width = L_RTEXT_SPACE + m_submenu_len + R_RTEXT_SPACE;
 
-	} else if (mi->action != ACT_MAIN_NONE) {
+	} else if (mi->action_id != ACT_MAIN_NONE) {
 		struct string keystroke;
 
 		if (init_string(&keystroke)) {
-			add_keystroke_to_string(&keystroke, mi->action, KEYMAP_MAIN);
+			add_keystroke_action_to_string(&keystroke, mi->action_id, KEYMAP_MAIN);
 			rtext_width = L_RTEXT_SPACE + keystroke.length + R_RTEXT_SPACE;
 			done_string(&keystroke);
 		}
@@ -253,80 +254,57 @@ count_menu_size(struct terminal *term, struct menu *menu)
 	int_bounds(&menu->box.y, 0, height - my);
 }
 
-static int
-search_selectable(struct menu *menu, int pos, int dir)
-{
-	assert(pos >= 0 && pos < menu->size && (dir == 1 || dir == -1));
-	if_assert_failed return -1;
-
-	while (!mi_is_selectable(&menu->items[pos])) {
-		if (dir > 0 && pos == menu->size - 1)
-			return -1;
-		else if (dir < 0 && pos == 0)
-			return -1;
-
-		pos += dir;
-	}
-
-	return pos;
-}
-
 static void
 scroll_menu(struct menu *menu, int steps, int wrap)
 {
-	int height, scr_i, pos;
+	int height, scr_i, pos, start;
 	int s = steps ? steps/abs(steps) : 1; /* Selectable item search direction. */
 
 	if (menu->size <= 0) {
+no_item:
 		/* Menu is empty. */
 		menu->selected = -1;
 		menu->first = 0;
 		return;
 	}
 
-	/* Move by required steps and handle wraparound if needed.
-	 * A step of zero can be used, indicating we want to select
-	 * item corresponding to |menu->selected| value rather than
-	 * moving by to a position relative to this value.
-	 * We override search direction for selectable items if we encounter
-	 * a limit, since it depends in which conditions this limit is
-	 * attained. */
-	menu->selected += steps;
-	if (menu->selected >= menu->size) {
-		if (wrap) {
-			menu->selected = 0;
-			s = 1;
-		} else {
-			menu->selected = int_max(0, menu->size - 1);
-			s = -1;
-		}
-	} else if (menu->selected < 0) {
-		if (wrap) {
-			menu->selected = int_max(0, menu->size - 1);
-			s = -1;
-		} else {
-			menu->selected = 0;
-			s = 1;
+	start = pos = menu->selected;
+
+	if (!steps) steps = 1, --pos;
+
+	while (steps) {
+		pos += s, steps -= s;
+
+		while (1) {
+			if (start == pos) {
+				goto select_item;
+			} else if (pos >= menu->size && s == 1) {
+				if (wrap) {
+					pos = 0;
+				} else {
+					pos = menu->size - 1;
+					goto select_item;
+				}
+			} else if (pos < 0 && s == -1) {
+				if (wrap) {
+					pos = menu->size - 1;
+				} else {
+					pos = 0;
+					goto select_item;
+				}
+			} else if (!mi_is_selectable(&menu->items[pos])) {
+				pos += s;
+			} else {
+				break;
+			}
+
+			if (start == -1) start = 0;
 		}
 	}
 
-	/* Current selected item may be an unselectable item, so we need to
-	 * find first selectable item near to it.
-	 * @s = 1 : ascending search.
-	 * @s = -1: descending search. */
-
-	/* Search first selectable item in one direction. */
-	pos = search_selectable(menu, menu->selected, s);
-	if (pos == -1) {
-		/* If not found, invert the search direction and try again. */
-		pos = search_selectable(menu, menu->selected, -s);
-	}
-
-	/* No selectable item found, just return. */
-	if (pos == -1) {
-		menu->selected = -1;
-		menu->first = 0;
-	}
+select_item:
+	if (!mi_is_selectable(&menu->items[pos]))
+		goto no_item;
 
 	menu->selected = pos;
 
@@ -513,7 +491,7 @@ display_menu(struct terminal *term, struct menu *menu)
 		if (mi_is_submenu(mi)) {
 			draw_menu_right_text(term, m_submenu, m_submenu_len,
 					     menu->box.x, box.y, box.width, color);
-		} else if (mi->action != ACT_MAIN_NONE) {
+		} else if (mi->action_id != ACT_MAIN_NONE) {
 			struct string keystroke;
 
 #ifdef CONFIG_DEBUG
@@ -527,9 +505,9 @@ display_menu(struct terminal *term, struct menu *menu)
 #endif /* CONFIG_DEBUG */
 
 			if (init_string(&keystroke)) {
-				add_keystroke_to_string(&keystroke,
-							mi->action,
-							KEYMAP_MAIN);
+				add_keystroke_action_to_string(&keystroke,
+							       mi->action_id,
+							       KEYMAP_MAIN);
 				draw_menu_right_text(term, keystroke.source,
 						     keystroke.length,
 						     menu->box.x, box.y,
@@ -686,7 +664,7 @@ search_menu_item(struct menu_item *item, unsigned char *buffer,
 }
 
 static enum input_line_code
-menu_search_handler(struct input_line *line, int action)
+menu_search_handler(struct input_line *line, int action_id)
 {
 	struct menu *menu = line->data;
 	struct terminal *term = menu->win->term;
@@ -695,7 +673,7 @@ menu_search_handler(struct input_line *line, int action)
 	int pos = menu->selected;
 	int direction;
 
-	switch (action) {
+	switch (action_id) {
 	case ACT_EDIT_REDRAW:
 		return INPUT_LINE_PROCEED;
 
@@ -760,10 +738,10 @@ static void
 menu_kbd_handler(struct menu *menu, struct term_event *ev)
 {
 	struct window *win = menu->win;
-	enum menu_action action = kbd_action(KEYMAP_MENU, ev, NULL);
+	enum menu_action action_id = kbd_action(KEYMAP_MENU, ev, NULL);
 	int s = 0;
 
-	switch (action) {
+	switch (action_id) {
 		case ACT_MENU_LEFT:
 		case ACT_MENU_RIGHT:
 			if (list_has_next(win->term->windows, win)
@@ -777,7 +755,7 @@ menu_kbd_handler(struct menu *menu, struct term_event *ev)
 				return;
 			}
 
-			if (action == ACT_MENU_RIGHT)
+			if (action_id == ACT_MENU_RIGHT)
 				goto enter;
 
 			delete_window(win);
@@ -829,7 +807,7 @@ menu_kbd_handler(struct menu *menu, struct term_event *ev)
 			int key = get_kbd_key(ev);
 
 			if ((key >= KBD_F1 && key <= KBD_F12)
-			    || check_kbd_modifier(ev, KBD_ALT)) {
+			    || check_kbd_modifier(ev, KBD_MOD_ALT)) {
 				delete_window_ev(win, ev);
 				return;
 			}
@@ -880,9 +858,7 @@ menu_handler(struct window *win, struct term_event *ev)
 			break;
 
 		case EVENT_ABORT:
-			if (menu->items->flags & FREE_ANY)
-				free_menu_items(menu->items);
-
+			free_menu_items(menu->items);
 			break;
 	}
 }
@@ -902,7 +878,7 @@ do_mainmenu(struct terminal *term, struct menu_item *items,
 	menu->size = count_items(items);
 	menu->hotkeys = 1;
 
-#ifdef ENABLE_NLS
+#ifdef CONFIG_NLS
 	clear_hotkeys_cache(menu);
 #endif
 	init_hotkeys(term, menu);
@@ -1083,9 +1059,9 @@ static void
 mainmenu_kbd_handler(struct menu *menu, struct term_event *ev)
 {
 	struct window *win = menu->win;
-	enum menu_action action = kbd_action(KEYMAP_MENU, ev, NULL);
+	enum menu_action action_id = kbd_action(KEYMAP_MENU, ev, NULL);
 
-	switch (action) {
+	switch (action_id) {
 	case ACT_MENU_ENTER:
 	case ACT_MENU_DOWN:
 	case ACT_MENU_UP:
@@ -1107,13 +1083,13 @@ mainmenu_kbd_handler(struct menu *menu, struct term_event *ev)
 	case ACT_MENU_PREVIOUS_ITEM:
 		/* This is pretty western centric since `what is next'?
 		 * Anyway we cycle clockwise by resetting the action ... */
-		action = (action == ACT_MENU_NEXT_ITEM)
+		action_id = (action_id == ACT_MENU_NEXT_ITEM)
 		       ? ACT_MENU_RIGHT : ACT_MENU_LEFT;
 		/* ... and then letting left/right handling take over. */
 
 	case ACT_MENU_LEFT:
 	case ACT_MENU_RIGHT:
-		scroll_menu(menu, action == ACT_MENU_LEFT ? -1 : 1, 1);
+		scroll_menu(menu, action_id == ACT_MENU_LEFT ? -1 : 1, 1);
 		break;
 
 	case ACT_MENU_REDRAW:
@@ -1131,7 +1107,7 @@ mainmenu_kbd_handler(struct menu *menu, struct term_event *ev)
 		}
 
 	case ACT_MENU_CANCEL:
-		delete_window_ev(win, action != ACT_MENU_CANCEL ? ev : NULL);
+		delete_window_ev(win, action_id != ACT_MENU_CANCEL ? ev : NULL);
 		return;
 	}
 
@@ -1184,7 +1160,7 @@ new_menu(enum menu_item_flags flags)
 
 void
 add_to_menu(struct menu_item **mi, unsigned char *text, unsigned char *rtext,
-	    enum main_action action, menu_func func, void *data,
+	    enum main_action action_id, menu_func_T func, void *data,
 	    enum menu_item_flags flags)
 {
 	int n = count_items(*mi);
@@ -1199,7 +1175,7 @@ add_to_menu(struct menu_item **mi, unsigned char *text, unsigned char *rtext,
 	copy_struct(item + 1, item);
 
 	/* Setup the new item. All menu items share the item_free value. */
-	SET_MENU_ITEM(item, text, rtext, action, func, data,
+	SET_MENU_ITEM(item, text, rtext, action_id, func, data,
 		      item->flags | flags, HKS_SHOW, 0);
 }
 
