@@ -1,5 +1,4 @@
 /* BFU display helpers. */
-/* $Id: style.c,v 1.13.4.1 2005/04/05 21:08:40 jonas Exp $ */
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
@@ -18,8 +17,8 @@
 
 struct bfu_color_entry {
 	/* Pointers to the options tree values. */
-	color_t *background;
-	color_t *foreground;
+	color_T *background;
+	color_T *foreground;
 
 	/* The copy of "text" and "background" colors. */
 	struct color_pair colors;
