@@ -1,5 +1,4 @@
 /* Internal bookmarks XBEL bookmarks basic support */
-/* $Id: xbel.c,v 1.47.4.4 2005/04/06 08:48:56 jonas Exp $ */
 
 /*
  * TODO: Decent XML output.
@@ -103,7 +102,7 @@ read_bookmarks_xbel(FILE *f)
 	XML_SetCharacterDataHandler(p, on_text);
 
 	while (!done && !err) {
-		int len = fread(in_buffer, 1, BUFSIZ, f);
+		size_t len = fread(in_buffer, 1, BUFSIZ, f);
 
 		if (ferror(f)) {
 			ERROR(gettext("read_bookmarks_xbel(): Error reading %s"),
@@ -218,8 +217,7 @@ write_bookmarks_list(struct secure_save_info *ssi,
 		indentation(ssi, n + 1);
 
 		if (bm->box_item->type == BI_FOLDER) {
-			int expanded = folder_state
-				     ? bm->box_item->expanded : 0;
+			int expanded = folder_state && bm->box_item->expanded;
 
 			secure_fputs(ssi, "<folder folded=\"");
 			secure_fputs(ssi, expanded ? "no" : "yes");
