@@ -1,5 +1,4 @@
 /* Bookmarks dialogs */
-/* $Id: dialogs.c,v 1.194.2.8 2005/04/06 09:02:09 jonas Exp $ */
 
 #ifndef _GNU_SOURCE
 #define _GNU_SOURCE /* XXX: we _WANT_ strcasestr() ! */
@@ -18,14 +17,14 @@
 #include "bookmarks/dialogs.h"
 #include "dialogs/edit.h"
 #include "intl/gettext/libintl.h"
+#include "main/object.h"
 #include "protocol/uri.h"
-#include "sched/session.h"
+#include "session/session.h"
 #include "terminal/terminal.h"
 #include "util/conv.h"
 #include "util/error.h"
 #include "util/memory.h"
-#include "util/object.h"
-#include "util/ttime.h"
+#include "util/time.h"
 
 
 /* Whether to save bookmarks after each modification of their list
@@ -33,10 +32,6 @@
 #define BOOKMARKS_RESAVE	1
 
 
-/* Last searched values */
-unsigned char *bm_last_searched_name = NULL;
-unsigned char *bm_last_searched_url = NULL;
-
 static void
 lock_bookmark(struct listbox_item *item)
 {
@@ -84,7 +79,7 @@ get_bookmark_uri(struct listbox_item *item)
 	struct bookmark *bookmark = item->udata;
 
 	return bookmark->url && *bookmark->url
-		? get_translated_uri(bookmark->url, 0) : NULL;
+		? get_translated_uri(bookmark->url, NULL) : NULL;
 }
 
 static struct listbox_item *
@@ -166,7 +161,7 @@ void launch_bm_add_doc_dialog(struct terminal *, struct dialog_data *,
 
 
 /* Callback for the "add" button in the bookmark manager */
-static t_handler_event_status
+static widget_handler_status_T
 push_add_button(struct dialog_data *dlg_data, struct widget_data *widget_data)
 {
 	launch_bm_add_doc_dialog(dlg_data->win->term, dlg_data,
@@ -181,7 +176,7 @@ void launch_bm_search_doc_dialog(struct terminal *, struct dialog_data *,
 
 
 /* Callback for the "search" button in the bookmark manager */
-static t_handler_event_status
+static widget_handler_status_T
 push_search_button(struct dialog_data *dlg_data, struct widget_data *widget_data)
 {
 	launch_bm_search_doc_dialog(dlg_data->win->term, dlg_data,
@@ -206,9 +201,6 @@ move_bookmark_after_selected(struct bookmark *bookmark, struct bookmark *selecte
 	add_at_pos(selected->box_item, bookmark->box_item);
 }
 
-
-/**** ADD FOLDER *****************************************************/
-
 static void
 focus_bookmark(struct widget_data *box_widget_data, struct listbox_data *box,
 		struct bookmark *bm)
@@ -223,26 +215,28 @@ focus_bookmark(struct widget_data *box_widget_data, struct listbox_data *box,
 	} while (box->sel->udata != bm && box->sel != sel2);
 }
 
-/* TODO: merge with bookmark_add_add() ? --Zas */
 static void
-do_add_special_bookmark(struct dialog_data *dlg_data, unsigned char *name, unsigned char *url)
+do_add_bookmark(struct dialog_data *dlg_data, unsigned char *title, unsigned char *url)
 {
-	struct widget_data *widget_data = dlg_data->widgets_data;
-	struct listbox_data *box = get_dlg_listbox_data(dlg_data);
 	struct bookmark *bm = NULL;
 	struct bookmark *selected = NULL;
+	struct listbox_data *box = NULL;
 
-	if (box->sel) {
-		selected = box->sel ? box->sel->udata : NULL;
+	if (dlg_data) {
+		box = get_dlg_listbox_data(dlg_data);
 
-		if (box->sel->type == BI_FOLDER && box->sel->expanded) {
-			bm = selected;
-		} else {
-			bm = selected->root;
+		if (box->sel) {
+			selected = box->sel ? box->sel->udata : NULL;
+
+			if (box->sel->type == BI_FOLDER && box->sel->expanded) {
+				bm = selected;
+			} else {
+				bm = selected->root;
+			}
 		}
 	}
 
-	bm = add_bookmark(bm, 1, name, url);
+	bm = add_bookmark(bm, 1, title, url);
 	if (!bm) return;
 
 	move_bookmark_after_selected(bm, selected);
@@ -251,26 +245,25 @@ do_add_special_bookmark(struct dialog_data *dlg_data, unsigned char *name, unsig
 	write_bookmarks();
 #endif
 
-	/* We touch only the actual bookmark dialog, not all of them;
-	 * that's right, right? ;-) --pasky */
-	focus_bookmark(widget_data, box, bm);
+	if (dlg_data) {
+		struct widget_data *widget_data = dlg_data->widgets_data;
+
+		/* We touch only the actual bookmark dialog, not all of them;
+		 * that's right, right? ;-) --pasky */
+		focus_bookmark(widget_data, box, bm);
+	}
 }
 
+
+/**** ADD FOLDER *****************************************************/
+
 static void
-do_add_folder(struct dialog_data *dlg_data, unsigned char *name)
+do_add_folder(struct dialog_data *dlg_data, unsigned char *foldername)
 {
-	do_add_special_bookmark(dlg_data, name, NULL);
+	do_add_bookmark(dlg_data, foldername, NULL);
 }
 
-static t_handler_event_status
-push_add_separator_button(struct dialog_data *dlg_data, struct widget_data *widget_data)
-{
-	do_add_special_bookmark(dlg_data, "-", "");
-	redraw_dialog(dlg_data, 1);
-	return EVENT_PROCESSED;
-}
-
-static t_handler_event_status
+static widget_handler_status_T
 push_add_folder_button(struct dialog_data *dlg_data, struct widget_data *widget_data)
 {
 	input_dialog(dlg_data->win->term, NULL,
@@ -283,6 +276,17 @@ push_add_folder_button(struct dialog_data *dlg_data, struct widget_data *widget_
 }
 
 
+/**** ADD SEPARATOR **************************************************/
+
+static widget_handler_status_T
+push_add_separator_button(struct dialog_data *dlg_data, struct widget_data *widget_data)
+{
+	do_add_bookmark(dlg_data, "-", "");
+	redraw_dialog(dlg_data, 1);
+	return EVENT_PROCESSED;
+}
+
+
 /**** EDIT ***********************************************************/
 
 /* Called when an edit is complete. */
@@ -307,7 +311,7 @@ bookmark_edit_cancel(struct dialog *dlg) {
 }
 
 /* Called when the edit button is pushed */
-static t_handler_event_status
+static widget_handler_status_T
 push_edit_button(struct dialog_data *dlg_data, struct widget_data *edit_btn)
 {
 	struct listbox_data *box = get_dlg_listbox_data(dlg_data);
@@ -315,12 +319,12 @@ push_edit_button(struct dialog_data *dlg_data, struct widget_data *edit_btn)
 	/* Follow the bookmark */
 	if (box->sel) {
 		struct bookmark *bm = (struct bookmark *) box->sel->udata;
-		const unsigned char *name = bm->title;
+		const unsigned char *title = bm->title;
 		const unsigned char *url = bm->url;
 
 		object_lock(bm);
 		do_edit_dialog(dlg_data->win->term, 1, N_("Edit bookmark"),
-			       name, url,
+			       title, url,
 			       (struct session *) dlg_data->dlg->udata, dlg_data,
 			       bookmark_edit_done, bookmark_edit_cancel,
 			       (void *) bm, EDIT_DLG_ADD);
@@ -412,7 +416,7 @@ do_move_bookmark(struct bookmark *dest, struct list_head *destb,
 	}
 }
 
-static t_handler_event_status
+static widget_handler_status_T
 push_move_button(struct dialog_data *dlg_data,
 		 struct widget_data *blah)
 {
@@ -454,8 +458,6 @@ push_move_button(struct dialog_data *dlg_data,
 
 /**** MANAGEMENT *****************************************************/
 
-#define	BOOKMARK_MANAGER_BUTTONS	8
-
 static struct hierbox_browser_button bookmark_buttons[] = {
 	{ N_("~Goto"),		push_hierbox_goto_button,	1 },
 	{ N_("~Edit"),		push_edit_button,		0 },
@@ -486,62 +488,16 @@ struct_hierbox_browser(
 void
 bookmark_manager(struct session *ses)
 {
-	/* Reset momorized search criterias */
-	mem_free_set(&bm_last_searched_name, NULL);
-	mem_free_set(&bm_last_searched_url, NULL);
+	free_last_searched_bookmark();
 	bookmark_browser.expansion_callback = bookmarks_set_dirty;
 	hierbox_browser(&bookmark_browser, ses);
 }
 
 
-
 /****************************************************************************\
-  Bookmark add dialog.
+  Bookmark search dialog.
 \****************************************************************************/
 
-/* Adds the bookmark */
-static void
-bookmark_add_add(void *data)
-{
-	struct dialog *dlg = data;
-	struct widget_data *widget_data = NULL; /* silence stupid gcc */
-	struct listbox_data *box = NULL;
-	struct bookmark *bm = NULL;
-	struct bookmark *selected = NULL;
-
-	if (dlg->udata) {
-		struct dialog_data *dlg_data = (struct dialog_data *) dlg->udata;
-
-		widget_data = dlg_data->widgets_data;
-		box = get_dlg_listbox_data(dlg_data);
-
-		if (box->sel) {
-			selected = box->sel->udata;
-
-			if (box->sel->type == BI_FOLDER) {
-				bm = selected;
-			} else {
-				bm = selected->root;
-			}
-		}
-	}
-
-	bm = add_bookmark(bm, 1, dlg->widgets[0].data, dlg->widgets[1].data);
-	if (!bm) return;
-
-	move_bookmark_after_selected(bm, selected);
-
-#ifdef BOOKMARKS_RESAVE
-	write_bookmarks();
-#endif
-
-	if (dlg->udata) {
-		/* We touch only the actual bookmark dialog, not all of them;
-		 * that's right, right? ;-) --pasky */
-		focus_bookmark(widget_data, box, bm);
-	}
-}
-
 
 /* Searchs a substring either in title or url fields (ignoring
  * case).  If search_title and search_url are not empty, it selects bookmarks
@@ -554,71 +510,92 @@ bookmark_add_add(void *data)
  * rapid search of an already existing bookmark. --Zas */
 
 struct bookmark_search_ctx {
-	unsigned char *search_url;
-	unsigned char *search_title;
+	unsigned char *url;
+	unsigned char *title;
 	int found;
-	int ofs;
+	int offset;
 };
 
 #define NULL_BOOKMARK_SEARCH_CTX {NULL, NULL, 0, 0}
 
 static int
-test_search(struct listbox_item *item, void *data_, int *offset) {
+test_search(struct listbox_item *item, void *data_, int *offset)
+{
 	struct bookmark_search_ctx *ctx = data_;
-	struct bookmark *bm = item->udata;
-	int m = ((ctx->search_title && *ctx->search_title
-		  && strcasestr(bm->title, ctx->search_title)) ||
-		 (ctx->search_url && *ctx->search_url
-		  && strcasestr(bm->url, ctx->search_url)));
 
-	if (!ctx->ofs) m = 0; /* ignore possible match on first item */
-	ctx->found = m;
-	ctx->ofs++;
+	if (!ctx->offset) {
+		ctx->found = 0; /* ignore possible match on first item */
+	} else {
+		struct bookmark *bm = item->udata;
+
+		assert(ctx->title && ctx->url);
+
+		ctx->found = (*ctx->title && strcasestr(bm->title, ctx->title))
+			     || (*ctx->url && strcasestr(bm->url, ctx->url));
+
+		if (ctx->found) *offset = 0;
+	}
+	ctx->offset++;
 
-	if (m) *offset = 0;
 	return 0;
 }
 
+/* Last searched values */
+static unsigned char *bm_last_searched_title = NULL;
+static unsigned char *bm_last_searched_url = NULL;
+
+void
+free_last_searched_bookmark(void)
+{
+	mem_free_set(&bm_last_searched_title, NULL);
+	mem_free_set(&bm_last_searched_url, NULL);
+}
+
+static int
+memorize_last_searched_bookmark(struct bookmark_search_ctx *ctx)
+{
+	/* Memorize last searched title */
+	mem_free_set(&bm_last_searched_title, stracpy(ctx->title));
+	if (!bm_last_searched_title) return 0;
+
+	/* Memorize last searched url */
+	mem_free_set(&bm_last_searched_url, stracpy(ctx->url));
+	if (!bm_last_searched_url) {
+		mem_free(bm_last_searched_title);
+		return 0;
+	}
+
+	return 1;
+}
+
 /* Search bookmarks */
 static void
 bookmark_search_do(void *data)
 {
 	struct dialog *dlg = data;
-	unsigned char *search_title = dlg->widgets[0].data;
-	unsigned char *search_url = dlg->widgets[1].data;
 	struct bookmark_search_ctx ctx = NULL_BOOKMARK_SEARCH_CTX;
-	struct widget_data *widget_data = NULL;
-	struct listbox_data *box = NULL;
+	struct listbox_data *box;
 	struct dialog_data *dlg_data;
 
-	if (!search_title || !search_url)
-		return;
-
 	assertm(dlg->udata, "Bookmark search with NULL udata in dialog");
 	if_assert_failed return;
 
-	dlg_data = (struct dialog_data *) dlg->udata;
-	widget_data = dlg_data->widgets_data;
-	box = get_dlg_listbox_data(dlg_data);
+	ctx.title = dlg->widgets[0].data;
+	ctx.url   = dlg->widgets[1].data;
 
-	/* Memorize last searched title */
-	mem_free_set(&bm_last_searched_name, stracpy(search_title));
-	if (!bm_last_searched_name) return;
-
-	/* Memorize last searched url */
-	mem_free_set(&bm_last_searched_url, stracpy(search_url));
-	if (!bm_last_searched_url) {
-		mem_free(bm_last_searched_name);
+	if (!ctx.title || !ctx.url)
 		return;
-	}
 
-	ctx.search_url = search_url;
-	ctx.search_title = search_title;
+	if (!memorize_last_searched_bookmark(&ctx))
+		return;
+
+	dlg_data = (struct dialog_data *) dlg->udata;
+	box = get_dlg_listbox_data(dlg_data);
 
 	traverse_listbox_items_list(box->sel, box, 0, 0, test_search, &ctx);
 	if (!ctx.found) return;
 
-	listbox_sel_move(widget_data, ctx.ofs - 1);
+	listbox_sel_move(dlg_data->widgets_data, ctx.offset - 1);
 }
 
 
@@ -629,12 +606,27 @@ launch_bm_search_doc_dialog(struct terminal *term,
 			    struct session *ses)
 {
 	do_edit_dialog(term, 1, N_("Search bookmarks"),
-		       bm_last_searched_name, bm_last_searched_url,
+		       bm_last_searched_title, bm_last_searched_url,
 		       ses, parent, bookmark_search_do, NULL, NULL,
 		       EDIT_DLG_SEARCH);
 }
 
 
+
+/****************************************************************************\
+  Bookmark add dialog.
+\****************************************************************************/
+
+/* Adds the bookmark */
+static void
+bookmark_add_add(void *data)
+{
+	struct dialog *dlg = data;
+	struct dialog_data *dlg_data = (struct dialog_data *) dlg->udata;
+
+	do_add_bookmark(dlg_data, dlg->widgets[0].data, dlg->widgets[1].data);
+}
+
 void
 launch_bm_add_dialog(struct terminal *term,
 		     struct dialog_data *parent,
@@ -666,6 +658,11 @@ launch_bm_add_link_dialog(struct terminal *term,
 			     get_current_link_url(ses, url, MAX_STR_LEN));
 }
 
+
+/****************************************************************************\
+  Bookmark tabs dialog.
+\****************************************************************************/
+
 void
 bookmark_terminal_tabs_dialog(struct terminal *term)
 {
