@@ -1,5 +1,4 @@
 /* Cache subsystem */
-/* $Id: cache.c,v 1.194.2.4 2005/05/02 21:14:03 jonas Exp $ */
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
@@ -13,23 +12,23 @@
 #include "cache/cache.h"
 #include "cache/dialogs.h"
 #include "config/options.h"
-#include "main.h"
+#include "main/main.h"
+#include "main/object.h"
+#include "network/connection.h"
 #include "protocol/protocol.h"
 #include "protocol/proxy.h"
 #include "protocol/uri.h"
-#include "sched/connection.h"
 #include "util/error.h"
 #include "util/memory.h"
-#include "util/object.h"
 #include "util/string.h"
-#include "util/types.h"
 
 /* The list of cache entries */
 static INIT_LIST_HEAD(cache_entries);
 
-static long cache_size;
+static unsigned longlong cache_size;
 static int id_counter = 1;
 
+static void truncate_entry(struct cache_entry *cached, off_t offset, int final);
 
 /* Change 0 to 1 to enable cache debugging features (redirect stderr to a file). */
 #if 0
@@ -40,8 +39,9 @@ static int id_counter = 1;
 
 #define dump_frag(frag, count) \
 do { \
-	DBG(" [%d] f=%p offset=%li length=%li real_length=%li", \
-	      count, frag, frag->offset, frag->length, frag->real_length); \
+	DBG(" [%d] f=%p offset=%" PRId64 " length=%" PRId64 \
+	    " real_length=%" PRId64, \
+	    count, frag, frag->offset, frag->length, frag->real_length); \
 } while (0)
 
 #define dump_frags(entry, comment) \
@@ -49,7 +49,7 @@ do { \
 	struct fragment *frag; \
         int count = 0;	\
  \
-	DBG("%s: url=%s, cache_size=%li", comment, entry->url, cache_size); \
+	DBG("%s: url=%s, cache_size=%li", comment, struri(entry->uri), cache_size); \
 	foreach (frag, entry->frag) \
 		dump_frag(frag, ++count); \
 } while (0)
@@ -58,42 +58,40 @@ do { \
 #define dump_frags(entry, comment)
 #endif /* DEBUG_CACHE */
 
-
-static int
-is_entry_used(struct cache_entry *cached)
+unsigned longlong
+get_cache_size(void)
 {
-	struct connection *conn;
-
-	foreach (conn, queue)
-		if (conn->cached == cached)
-			return 1;
-
-	return 0;
+	return cache_size;
 }
 
-
-long
-cache_info(int type)
+int
+get_cache_entry_count(void)
 {
-	int i = 0;
-	struct cache_entry *cached;
+	return list_size(&cache_entries);
+}
 
-	switch (type) {
-		case INFO_BYTES:
-			return cache_size;
-		case INFO_FILES:
-			foreach (cached, cache_entries) i++;
-			return i;
-		case INFO_LOCKED:
-			foreach (cached, cache_entries)
-				i += is_object_used(cached);
-			return i;
-		case INFO_LOADING:
-			foreach (cached, cache_entries)
-				i += is_entry_used(cached);
-			return i;
-	}
-	return 0;
+int
+get_cache_entry_used_count(void)
+{
+	struct cache_entry *cached;
+	int i = 0;
+
+	foreach (cached, cache_entries)
+		i += is_object_used(cached);
+
+	return i;
+}
+
+int
+get_cache_entry_loading_count(void)
+{
+	struct cache_entry *cached;
+	int i = 0;
+
+	foreach (cached, cache_entries)
+		i += is_entry_used(cached);
+
+	return i;
 }
 
 struct cache_entry *
@@ -111,9 +109,7 @@ find_in_cache(struct uri *uri)
 		if (!compare_uri(c_uri, uri, URI_BASE))
 			continue;
 
-		/* Move it on the top of the list. */
-		del_from_list(cached);
-		add_to_list(cache_entries, cached);
+		move_to_top_of_list(cache_entries, cached);
 
 		return cached;
 	}
@@ -154,17 +150,21 @@ get_cache_entry(struct uri *uri)
 	cached->id = id_counter++;
 	object_nolock(cached, "cache_entry"); /* Debugging purpose. */
 
-	add_to_list(cache_entries, cached);
-
 	cached->box_item = add_listbox_leaf(&cache_browser, NULL, cached);
 
+	add_to_list(cache_entries, cached);
+
 	return cached;
 }
 
 static int
 cache_entry_has_expired(struct cache_entry *cached)
 {
-	return cached->max_age <= time(NULL);
+	timeval_T now;
+
+	timeval_now(&now);
+
+	return timeval_cmp(&cached->max_age, &now) <= 0;
 }
 
 struct cache_entry *
@@ -190,6 +190,7 @@ get_validated_cache_entry(struct uri *uri, enum cache_mode cache_mode)
 	if (is_object_used(cached)) {
 #if 0
 		/* Never use expired entries. */
+		/* Disabled because it hurts usability too much. */
 		if (cached->expire && cache_entry_has_expired(cached))
 			return NULL;
 #endif
@@ -254,7 +255,7 @@ get_redirected_cache_entry(struct uri *uri)
 
 
 static inline void
-enlarge_entry(struct cache_entry *cached, int size)
+enlarge_entry(struct cache_entry *cached, off_t size)
 {
 	cached->data_size += size;
 	assertm(cached->data_size >= 0,
@@ -299,6 +300,7 @@ static struct fragment *
 frag_alloc(size_t size)
 {
 	struct fragment *f = mem_mmap_alloc(FRAGSIZE(size));
+
 	if (!f) return NULL;
 	memset(f, 0, FRAGSIZE(size));
 	return f;
@@ -317,17 +319,17 @@ frag_free(struct fragment *f)
 }
 
 
-/* Contatenate overlapping fragments. */
+/* Concatenate overlapping fragments. */
 static void
 remove_overlaps(struct cache_entry *cached, struct fragment *f, int *trunc)
 {
-	int f_end_offset = f->offset + f->length;
+	off_t f_end_offset = f->offset + f->length;
 
 	/* Iterate thru all fragments we still overlap to. */
 	while (list_has_next(cached->frag, f)
 		&& f_end_offset > f->next->offset) {
 		struct fragment *nf;
-		int end_offset = f->next->offset + f->next->length;
+		off_t end_offset = f->next->offset + f->next->length;
 
 		if (f_end_offset < end_offset) {
 			/* We end before end of the following fragment, though.
@@ -372,12 +374,12 @@ remove_overlaps(struct cache_entry *cached, struct fragment *f, int *trunc)
 /* Note that this function is maybe overcommented, but I'm certainly not
  * unhappy from that. */
 int
-add_fragment(struct cache_entry *cached, int offset,
-	     unsigned char *data, int length)
+add_fragment(struct cache_entry *cached, off_t offset,
+	     const unsigned char *data, ssize_t length)
 {
 	struct fragment *f, *nf;
 	int trunc = 0;
-	int end_offset;
+	off_t end_offset;
 
 	if (!length) return 0;
 
@@ -392,7 +394,7 @@ add_fragment(struct cache_entry *cached, int offset,
 	/* Possibly insert the new data in the middle of existing fragment. */
 	foreach (f, cached->frag) {
 		int ret = 0;
-		int f_end_offset = f->offset + f->length;
+		off_t f_end_offset = f->offset + f->length;
 
 		/* No intersection? */
 		if (f->offset > offset) break;
@@ -464,6 +466,17 @@ add_fragment(struct cache_entry *cached, int offset,
 	return 1;
 }
 
+/* Try to defragment the cache entry. Defragmentation will not be possible
+ * if there is a gap in the fragments; if we have bytes 1-100 in one fragment
+ * and bytes 201-300 in the second, we must leave those two fragments separate
+ * so that the fragment for bytes 101-200 can later be inserted. However,
+ * if we have the fragments for bytes 1-100, 101-200, and 201-300, we will
+ * catenate them into one new fragment and replace the original fragments
+ * with that new fragment.
+ *
+ * If are no fragments, return NULL. If there is no fragment with byte 1,
+ * return NULL. Otherwise, return the first fragment, whether or not it was
+ * possible to fully defragment the entry. */
 struct fragment *
 get_cache_fragment(struct cache_entry *cached)
 {
@@ -477,23 +490,26 @@ get_cache_fragment(struct cache_entry *cached)
 	if (first_frag->offset)
 		return NULL;
 
-	/* Find the first pair of fragments that overlap. It will be used to
-	 * figure out what sequence of fragments to include in the
-	 * defragmentation. */
+	/* Only one fragment so no defragmentation is needed */
+	if (list_is_singleton(cached->frag))
+		return first_frag;
+
+	/* Find the first pair of fragments with a gap in between. Only
+	 * fragments up to the first gap can be defragmented. */
 	for (adj_frag = first_frag->next; adj_frag != (void *) &cached->frag;
 	     adj_frag = adj_frag->next) {
-		long overlay = adj_frag->offset
-				- (adj_frag->prev->offset
-				   + adj_frag->prev->length);
+		long gap = adj_frag->offset
+			    - (adj_frag->prev->offset + adj_frag->prev->length);
 
-		if (overlay > 0) break;
-		if (overlay == 0) continue;
+		if (gap > 0) break;
+		if (gap == 0) continue;
 
-		INTERNAL("fragments overlay");
+		INTERNAL("fragments overlap");
 		return NULL;
 	}
 
-	/* Only one fragment so no defragmentation is needed */
+	/* There is a gap between the first two fragments, so we can't
+	 * defragment anything. */
 	if (adj_frag == first_frag->next)
 		return first_frag;
 
@@ -503,9 +519,8 @@ get_cache_fragment(struct cache_entry *cached)
 	     frag = frag->next)
 		new_frag_len += frag->length;
 
-	/* XXX: Even tho' the defragmentation fails because of allocation
-	 * failure just fall back to return the first fragment and pretend all
-	 * is well. */
+	/* XXX: If the defragmentation fails because of allocation failure,
+	 * fall back to return the first fragment and pretend all is well. */
 	/* FIXME: Is this terribly brain-dead? It corresponds to the semantic of
 	 * the code this extended version of the old defrag_entry() is supposed
 	 * to replace. --jonas */
@@ -549,8 +564,8 @@ delete_fragment(struct cache_entry *cached, struct fragment *f)
 	}
 }
 
-void
-truncate_entry(struct cache_entry *cached, int offset, int final)
+static void
+truncate_entry(struct cache_entry *cached, off_t offset, int final)
 {
 	struct fragment *f;
 
@@ -560,7 +575,7 @@ truncate_entry(struct cache_entry *cached, int offset, int final)
 	}
 
 	foreach (f, cached->frag) {
-		long size = offset - f->offset;
+		off_t size = offset - f->offset;
 
 		/* XXX: is zero length fragment really legal here ? --Zas */
 		assert(f->length >= 0);
@@ -594,7 +609,7 @@ truncate_entry(struct cache_entry *cached, int offset, int final)
 }
 
 void
-free_entry_to(struct cache_entry *cached, int offset)
+free_entry_to(struct cache_entry *cached, off_t offset)
 {
 	struct fragment *f;
 
@@ -607,7 +622,7 @@ free_entry_to(struct cache_entry *cached, int offset)
 			del_from_list(tmp);
 			frag_free(tmp);
 		} else if (f->offset < offset) {
-			long size = offset - f->offset;
+			off_t size = offset - f->offset;
 
 			enlarge_entry(cached, -size);
 			f->length -= size;
@@ -636,14 +651,13 @@ delete_entry_content(struct cache_entry *cached)
 	mem_free_set(&cached->etag, NULL);
 }
 
-void
-delete_cache_entry(struct cache_entry *cached)
+static void
+done_cache_entry(struct cache_entry *cached)
 {
 	assertm(!is_object_used(cached), "deleting locked cache entry");
 	assertm(!is_entry_used(cached), "deleting loading cache entry");
 
 	delete_entry_content(cached);
-	del_from_list(cached);
 
 	if (cached->box_item) done_listbox_item(&cache_browser, cached->box_item);
 
@@ -661,6 +675,26 @@ delete_cache_entry(struct cache_entry *cached)
 	mem_free(cached);
 }
 
+void
+delete_cache_entry(struct cache_entry *cached)
+{
+	del_from_list(cached);
+
+	done_cache_entry(cached);
+}
+
+
+void
+normalize_cache_entry(struct cache_entry *cached, off_t truncate_length)
+{
+	if (truncate_length < 0)
+		return;
+
+	truncate_entry(cached, truncate_length, 1);
+	cached->incomplete = 0;
+	cached->preformatted = 0;
+}
+
 
 struct uri *
 redirect_cache(struct cache_entry *cached, unsigned char *location,
@@ -720,20 +754,20 @@ garbage_collection(int whole)
 	struct cache_entry *cached;
 	/* We recompute cache_size when scanning cache entries, to ensure
 	 * consistency. */
-	long old_cache_size = 0;
+	unsigned longlong old_cache_size = 0;
 	/* The maximal cache size tolerated by user. Note that this is only
 	 * size of the "just stored" unused cache entries, used cache entries
 	 * are not counted to that. */
-	long opt_cache_size = get_opt_long("document.cache.memory.size");
+	unsigned longlong opt_cache_size = get_opt_long("document.cache.memory.size");
 	/* The low-treshold cache size. Basically, when the cache size is
 	 * higher than opt_cache_size, we free the cache so that there is no
 	 * more than this value in the cache anymore. This is to make sure we
 	 * aren't cleaning cache too frequently when working with a lot of
 	 * small cache entries but rather free more and then let it grow a
 	 * little more as well. */
-	long gc_cache_size = opt_cache_size * MEMORY_CACHE_GC_PERCENT / 100;
+	unsigned longlong gc_cache_size = opt_cache_size * MEMORY_CACHE_GC_PERCENT / 100;
 	/* The cache size we aim to reach. */
-	long new_cache_size = cache_size;
+	unsigned longlong new_cache_size = cache_size;
 #ifdef DEBUG_CACHE
 	/* Whether we've hit an used (unfreeable) entry when collecting
 	 * garbage. */
@@ -759,11 +793,12 @@ garbage_collection(int whole)
 		if (!is_object_used(cached) && !is_entry_used(cached))
 			continue;
 
+		assertm(new_cache_size >= cached->data_size,
+			"cache_size (%ld) underflow: subtracting %ld from %ld",
+			cache_size, cached->data_size, new_cache_size);
+
 		new_cache_size -= cached->data_size;
 
-		assertm(new_cache_size >= 0,
-				"cache_size (%ld) underflow: %ld",
-				cache_size, new_cache_size);
 		if_assert_failed { new_cache_size = 0; }
 	}
 
@@ -797,13 +832,14 @@ garbage_collection(int whole)
 		 * but that will probably complicate things too much. We'd have
 		 * to sort entries so prioritize removing the oldest entries. */
 
+		assertm(new_cache_size >= cached->data_size,
+			"cache_size (%ld) underflow: subtracting %ld from %ld",
+			cache_size, cached->data_size, new_cache_size);
+
 		/* Mark me for destruction, sir. */
 		cached->gc_target = 1;
 		new_cache_size -= cached->data_size;
 
-		assertm(new_cache_size >= 0,
-			"cache_size (%ld) underflow: %ld",
-			cache_size, new_cache_size);
 		if_assert_failed { new_cache_size = 0; }
 	}
 
@@ -838,7 +874,7 @@ shrinked_enough:
 		 * situation. */
 
 		for (entry = cached; (void *) entry != &cache_entries; entry = entry->next) {
-			long newer_cache_size = new_cache_size + entry->data_size;
+			unsigned longlong newer_cache_size = new_cache_size + entry->data_size;
 
 			if (newer_cache_size > gc_cache_size)
 				continue;
