@@ -1,5 +1,4 @@
 /* Cache-related dialogs */
-/* $Id: dialogs.c,v 1.79.4.2 2005/05/01 22:47:54 jonas Exp $ */
 
 #ifndef _GNU_SOURCE
 #define _GNU_SOURCE /* XXX: we _WANT_ strcasestr() ! */
@@ -18,13 +17,13 @@
 #include "cache/dialogs.h"
 #include "dialogs/edit.h"
 #include "intl/gettext/libintl.h"
+#include "main/object.h"
 #include "protocol/uri.h"
-#include "sched/session.h"
+#include "session/session.h"
 #include "terminal/draw.h"
 #include "terminal/terminal.h"
 #include "util/conv.h"
 #include "util/memory.h"
-#include "util/object.h"
 #include "util/string.h"
 
 
@@ -83,8 +82,9 @@ get_cache_entry_info(struct listbox_item *item, struct terminal *term)
 		}
 	}
 
-	add_format_to_string(&msg, "\n%s: %d", _("Size", term), cached->length);
-	add_format_to_string(&msg, "\n%s: %d", _("Loaded size", term),
+	add_format_to_string(&msg, "\n%s: %" PRId64, _("Size", term),
+	                     cached->length);
+	add_format_to_string(&msg, "\n%s: %" PRId64, _("Loaded size", term),
 						cached->data_size);
 	if (cached->content_type) {
 		add_format_to_string(&msg, "\n%s: %s", _("Content type", term),
@@ -120,14 +120,15 @@ get_cache_entry_info(struct listbox_item *item, struct terminal *term)
 
 #ifdef HAVE_STRFTIME
 	if (cached->expire) {
-		add_format_to_string(&msg, "\n%s: ", _("Expires", term));
-		add_date_to_string(&msg, get_opt_str("ui.date_format"), &cached->max_age);
+		time_t expires = timeval_to_seconds(&cached->max_age);
 
+		add_format_to_string(&msg, "\n%s: ", _("Expires", term));
+		add_date_to_string(&msg, get_opt_str("ui.date_format"), &expires);
 	}
 #endif
 #ifdef CONFIG_DEBUG
 	add_format_to_string(&msg, "\n%s: %d", "Refcount", get_object_refcount(cached));
-	add_format_to_string(&msg, "\n%s: %d", _("ID", term), cached->id);
+	add_format_to_string(&msg, "\n%s: %u", _("ID", term), cached->id);
 
 	if (cached->head && *cached->head) {
 		add_format_to_string(&msg, "\n%s:\n\n%s", _("Header", term),
