@@ -1,5 +1,4 @@
 /* Command line processing */
-/* $Id: cmdline.c,v 1.107.2.8 2005/09/13 22:25:11 jonas Exp $ */
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
@@ -31,9 +30,9 @@
 #include "config/options.h"
 #include "config/opttypes.h"
 #include "intl/gettext/libintl.h"
-#include "lowlevel/dns.h"
+#include "network/dns.h"
 #include "protocol/uri.h"
-#include "sched/session.h"
+#include "session/session.h"
 #include "util/error.h"
 #include "util/file.h"
 #include "util/lists.h"
@@ -91,7 +90,7 @@ parse_options_(int argc, unsigned char *argv[], struct option *opt,
 				if (err[0]) {
 					usrerror(gettext("Cannot parse option %s: %s"), argv[-1], err);
 
-					return RET_ERROR;
+					return RET_SYNTAX;
 				}
 
 				/* XXX: Empty strings means all is well and have
@@ -113,7 +112,7 @@ parse_options_(int argc, unsigned char *argv[], struct option *opt,
 
 unknown_option:
 	usrerror(gettext("Unknown option %s"), argv[-1]);
-	return RET_ERROR;
+	return RET_SYNTAX;
 }
 
 enum retval
@@ -134,7 +133,7 @@ eval_cmd(struct option *o, unsigned char ***argv, int *argc)
 
 	(*argv)++; (*argc)--;	/* Consume next argument */
 
-	parse_config_file(config_options, "-eval", *(*argv - 1), NULL);
+	parse_config_file(config_options, "-eval", *(*argv - 1), NULL, 0);
 
 	fflush(stdout);
 
@@ -158,7 +157,7 @@ lookup_cmd(struct option *o, unsigned char ***argv, int *argc)
 	if (*argc > 1) return gettext("Too many parameters");
 
 	(*argv)++; (*argc)--;
-	if (do_real_lookup(*(*argv - 1), &addrs, &addrno, 0)) {
+	if (do_real_lookup(*(*argv - 1), &addrs, &addrno, 0) == DNS_ERROR) {
 #ifdef HAVE_HERROR
 		herror(gettext("error"));
 #else
@@ -208,12 +207,14 @@ remote_cmd(struct option *o, unsigned char ***argv, int *argc)
 			REMOTE_METHOD_PING,
 			REMOTE_METHOD_XFEDOCOMMAND,
 			REMOTE_METHOD_ADDBOOKMARK,
+			REMOTE_METHOD_INFOBOX,
 			REMOTE_METHOD_NOT_SUPPORTED,
 		} type;
 	} remote_methods[] = {
 		{ "openURL",	  REMOTE_METHOD_OPENURL },
 		{ "ping",	  REMOTE_METHOD_PING },
 		{ "addBookmark",  REMOTE_METHOD_ADDBOOKMARK },
+		{ "infoBox",	  REMOTE_METHOD_INFOBOX },
 		{ "xfeDoCommand", REMOTE_METHOD_XFEDOCOMMAND },
 		{ NULL,		  REMOTE_METHOD_NOT_SUPPORTED },
 	};
@@ -310,6 +311,14 @@ remote_cmd(struct option *o, unsigned char ***argv, int *argc)
 		remote_session_flags = SES_REMOTE_ADD_BOOKMARK;
 		break;
 
+	case REMOTE_METHOD_INFOBOX:
+		if (arg == argend) break;
+		remote_url = memacpy(arg, argend - arg);
+		if (remote_url)
+			insert_in_string(&remote_url, 0, "about:", 6);
+		remote_session_flags = SES_REMOTE_INFO_BOX;
+		break;
+
 	case REMOTE_METHOD_NOT_SUPPORTED:
 		break;
 	}
@@ -415,7 +424,7 @@ print_full_help(struct option *tree, unsigned char *path)
 
 			case OPT_COLOR:
 			{
-				color_t color = option->value.color;
+				color_T color = option->value.color;
 				unsigned char hexcolor[8];
 
 				printf(gettext("(default: %s)"),
@@ -427,7 +436,7 @@ print_full_help(struct option *tree, unsigned char *path)
 				break;
 
 			case OPT_LANGUAGE:
-#ifdef ENABLE_NLS
+#ifdef CONFIG_NLS
 				printf(gettext("(default: \"%s\")"),
 				       language_to_name(option->value.number));
 #endif
@@ -482,7 +491,7 @@ print_full_help(struct option *tree, unsigned char *path)
 }
 
 static void
-print_short_help()
+print_short_help(void)
 {
 #define ALIGN_WIDTH 20
 	struct option *option;
@@ -750,13 +759,13 @@ struct option_info cmdline_options_info[] = {
 		"configuration directory (~/.elinks). It forces default configuration\n"
 		"values to be used and disables saving of runtime state files.")),
 
-	INIT_OPT_BOOL("", N_("Disable link numbering in dump output"),
-		"no-numbering", 0, 0,
+	INIT_OPT_CMDALIAS("", N_("Disable link numbering in dump output"),
+		"no-numbering", OPT_ALIAS_NEGATE, "document.dump.numbering",
 		N_("Prevents printing of link number in dump output.\n"
 		"Note that this really affects only -dump, nothing else.")),
 
-	INIT_OPT_BOOL("", N_("Disable printing of link references in dump output"),
-		"no-references", 0, 0,
+	INIT_OPT_CMDALIAS("", N_("Disable printing of link references in dump output"),
+		"no-references", OPT_ALIAS_NEGATE, "document.dump.references",
 		N_("Prevents printing of references (URIs) of document links\n"
 		"in dump output.\n"
 		"Note that this really affects only -dump, nothing else.")),
@@ -776,6 +785,7 @@ struct option_info cmdline_options_info[] = {
 		"\topenURL(URL, new-tab)     : open URL in new tab\n"
 		"\topenURL(URL, new-window)  : open URL in new window\n"
 		"\taddBookmark(URL)          : bookmark URL\n"
+		"\tinfoBox(text)             : show text in a message box\n"
 		"\txfeDoCommand(openBrowser) : open new window")),
 
 	INIT_OPT_INT("", N_("Connect to session ring with given ID"),
