@@ -1,5 +1,4 @@
 /* Options dialogs */
-/* $Id: dialogs.c,v 1.201.4.8 2005/04/06 09:11:18 jonas Exp $ */
 
 #ifndef _GNU_SOURCE
 #define _GNU_SOURCE /* XXX: we _WANT_ strcasestr() ! */
@@ -20,20 +19,22 @@
 #include "config/options.h"
 #include "config/opttypes.h"
 #include "intl/gettext/libintl.h"
-#include "sched/session.h"
+#include "main/event.h"
+#include "main/object.h"
+#include "session/session.h"
 #include "terminal/kbd.h"
 #include "terminal/terminal.h"
 #include "util/color.h"
 #include "util/error.h"
 #include "util/lists.h"
 #include "util/memory.h"
-#include "util/object.h"
 #include "util/secsave.h"
 
 
 static void
 toggle_success_msgbox(void *dummy)
 {
+	/* TODO: option_changed() */
 	get_opt_bool("ui.success_msgbox") = !get_opt_bool("ui.success_msgbox");
 	get_opt_rec(config_options, "ui.success_msgbox")->flags |= OPT_TOUCHED;
 }
@@ -164,13 +165,19 @@ get_option_info(struct listbox_item *item, struct terminal *term)
 
 	if (!init_string(&info)) return NULL;
 
+	add_format_to_string(&info, "%s: %s", _("Name", term), option->name);
+
 	type = _(option_types[option->type].name, term);
 	if (option->type == OPT_TREE) {
 		type = straconcat(type, " ",
 				_("(expand by pressing space)", term), NULL);
 	}
 
-	desc = _(option->desc  ? option->desc : (unsigned char *) "N/A", term);
+	add_format_to_string(&info, "\n%s: %s", _("Type", term), type);
+
+	if (option->type == OPT_TREE) {
+		mem_free(type);
+	}
 
 	if (option_types[option->type].write) {
 		unsigned char *range;
@@ -183,8 +190,6 @@ get_option_info(struct listbox_item *item, struct terminal *term)
 
 		option_types[option->type].write(option, &value);
 
-		add_format_to_string(&info, "%s: %s", _("Name", term), option->name);
-		add_format_to_string(&info, "\n%s: %s", _("Type", term), type);
 		range = get_range_string(option);
 		if (range) {
 			if (*range) {
@@ -194,25 +199,18 @@ get_option_info(struct listbox_item *item, struct terminal *term)
 			mem_free(range);
 		}
 		add_format_to_string(&info, "\n%s: %s", _("Value", term), value.source);
+		done_string(&value);
 
 		if (option->flags & OPT_TOUCHED)
 			add_to_string(&info, _("\n\nThis value has been changed"
 					     " since you last saved your"
 					     " configuration.", term));
 
-		if (*desc)
-			add_format_to_string(&info, "\n\n%s:\n%s", _("Description", term), desc);
-		done_string(&value);
-	} else {
-		add_format_to_string(&info, "%s: %s", _("Name", term), option->name);
-		add_format_to_string(&info, "\n%s: %s", _("Type", term), type);
-		if (*desc)
-			add_format_to_string(&info, "\n\n%s:\n%s", _("Description", term), desc);
 	}
 
-	if (option->type == OPT_TREE) {
-		mem_free(type);
-	}
+	desc = _(option->desc  ? option->desc : (unsigned char *) "N/A", term);
+	if (*desc)
+		add_format_to_string(&info, "\n\n%s:\n%s", _("Description", term), desc);
 
 	return info.source;
 }
@@ -291,7 +289,7 @@ static struct listbox_ops options_listbox_ops = {
 
 /* Button handlers */
 
-static t_handler_event_status
+static widget_handler_status_T
 check_valid_option(struct dialog_data *dlg_data, struct widget_data *widget_data)
 {
 	struct terminal *term = dlg_data->win->term;
@@ -308,21 +306,7 @@ check_valid_option(struct dialog_data *dlg_data, struct widget_data *widget_data
 		    option_types[option->type].set(option, chinon)) {
 			struct option *current = option;
 
-			option->flags |= OPT_TOUCHED;
-
-			/* Notify everyone out there! */
-
-			/* This boolean thing can look a little weird - it
-			 * basically says that we should proceed when there's
-			 * no change_hook or there's one and its return value
-			 * was zero. */
-			while (current && (!current->change_hook ||
-				!current->change_hook(ses, current, option))) {
-				if (!current->root)
-					break;
-
-				current = current->root;
-			}
+			option_changed(ses, current, option);
 
 			commandline = 0;
 			mem_free(chinon);
@@ -413,7 +397,7 @@ build_edit_dialog(struct terminal *term, struct session *ses,
 #undef EDIT_WIDGETS_COUNT
 }
 
-static t_handler_event_status
+static widget_handler_status_T
 push_edit_button(struct dialog_data *dlg_data,
 		 struct widget_data *some_useless_info_button)
 {
@@ -442,19 +426,44 @@ push_edit_button(struct dialog_data *dlg_data,
 }
 
 
+struct add_option_to_tree_ctx {
+	struct option *option;
+	struct widget_data *widget_data;
+};
+
 static void
 add_option_to_tree(void *data, unsigned char *name)
 {
-	struct option *option = data;
-	struct option *old = get_opt_rec_real(option, name);
+	struct add_option_to_tree_ctx *ctx = data;
+	struct option *old = get_opt_rec_real(ctx->option, name);
+	struct option *new;
 
 	if (old && (old->flags & OPT_DELETED)) delete_option(old);
-	/* get_opt_rec() will do all the work for ourselves... ;-) */
-	get_opt_rec(option, name);
+	/* get_opt_rec() will create the option. */
+	new = get_opt_rec(ctx->option, name);
+	if (new) listbox_sel(ctx->widget_data, new->box_item);
 	/* TODO: If the return value is NULL, we should pop up a msgbox. */
 }
 
-static t_handler_event_status
+static widget_handler_status_T
+check_option_name(struct dialog_data *dlg_data, struct widget_data *widget_data)
+{
+	unsigned char *p;
+
+	for (p = widget_data->cdata; *p; p++)
+		/* Not '*' since it is used internally. */
+		if (!isident(*p)) {
+			/* FIXME: Encode '.' into '*'? */
+			info_box(dlg_data->win->term, 0,
+				 N_("Bad string"), ALIGN_CENTER,
+				 N_("Option names may only contain alpha-numeric characters\n"
+				 "in addition to '_' and '-'."));
+			return EVENT_NOT_PROCESSED;
+		}
+
+	return EVENT_PROCESSED;
+}
+static widget_handler_status_T
 push_add_button(struct dialog_data *dlg_data,
 		struct widget_data *some_useless_info_button)
 {
@@ -462,6 +471,7 @@ push_add_button(struct dialog_data *dlg_data,
 	struct listbox_data *box = get_dlg_listbox_data(dlg_data);
 	struct listbox_item *item = box->sel;
 	struct option *option;
+	struct add_option_to_tree_ctx *ctx;
 
 	if (!item || !item->udata) {
 
@@ -486,16 +496,21 @@ invalid_option:
 			goto invalid_option;
 	}
 
-	input_dialog(term, NULL, N_("Add option"), N_("Name"),
-		     option, NULL,
-		     MAX_STR_LEN, "", 0, 0, check_nonempty,
+	ctx = mem_alloc(sizeof(*ctx));
+	if (!ctx) return EVENT_PROCESSED;
+	ctx->option = option;
+	ctx->widget_data = dlg_data->widgets_data;
+
+	input_dialog(term, getml(ctx, NULL), N_("Add option"), N_("Name"),
+		     ctx, NULL,
+		     MAX_STR_LEN, "", 0, 0, check_option_name,
 		     add_option_to_tree, NULL);
-	
+
 	return EVENT_PROCESSED;
 }
 
 
-static t_handler_event_status
+static widget_handler_status_T
 push_save_button(struct dialog_data *dlg_data,
 		struct widget_data *some_useless_info_button)
 {
@@ -506,7 +521,6 @@ push_save_button(struct dialog_data *dlg_data,
 	return EVENT_PROCESSED;
 }
 
-#define	OPTION_MANAGER_BUTTONS	5
 
 static struct hierbox_browser_button option_buttons[] = {
 	{ N_("~Info"),   push_hierbox_info_button,   1 },
@@ -548,38 +562,32 @@ static int keybinding_text_toggle;
 static struct listbox_item *action_box_items[KEYMAP_MAX][ACTION_BOX_SIZE];
 
 struct listbox_item *
-get_keybinding_action_box_item(enum keymap km, int action)
+get_keybinding_action_box_item(enum keymap_id keymap_id, action_id_T action_id)
 {
-	assert(action < ACTION_BOX_SIZE);
+	assert(action_id < ACTION_BOX_SIZE);
 	if_assert_failed return NULL;
 
-	return action_box_items[km][action];
+	return action_box_items[keymap_id][action_id];
 }
 
-struct keymap_box_item_info {
-	struct listbox_item *box_item;
-	struct strtonum *first, *last;
-};
-
-struct keymap_box_item_info keymap_box_item_info[KEYMAP_MAX];
+struct listbox_item *keymap_box_item[KEYMAP_MAX];
 
 void
-init_keybinding_listboxes(struct strtonum *keymaps, struct strtonum *actions[])
+init_keybinding_listboxes(struct keymap keymap_table[KEYMAP_MAX], struct action_list actions[])
 {
 	struct listbox_item *root = &keybinding_browser.root;
-	struct strtonum *act, *map;
+	struct action *act;
+	enum keymap_id keymap_id;
 
 	/* Do it backwards because add_listbox_item() add to front
 	 * of list. */
-	for (map = keymaps; map->str; map++) {
-		struct listbox_item *keymap;
+	for (keymap_id = 0; keymap_id < KEYMAP_MAX; keymap_id++) {
+		struct listbox_item *keymap_box;
 
-		keymap = add_listbox_item(NULL, root, BI_FOLDER, map, -1);
-		if (!keymap) continue;
+		keymap_box = add_listbox_item(NULL, root, BI_FOLDER, &keymap_table[keymap_id], -1);
+		if (!keymap_box) continue;
 
-		keymap_box_item_info[map->num].box_item = keymap;
-
-		for (act = actions[map->num]; act->str; act++) {
+		for (act = actions[keymap_id].actions; act->str; act++) {
 			struct listbox_item *item;
 
 			assert(act->num < ACTION_BOX_SIZE);
@@ -593,16 +601,15 @@ init_keybinding_listboxes(struct strtonum *keymaps, struct strtonum *actions[])
 			assert(act->desc);
 #endif
 
-			item = add_listbox_item(NULL, keymap, BI_FOLDER, act, -1);
+			item = add_listbox_item(NULL, keymap_box, BI_FOLDER, act, -1);
 			if (!item) continue;
 
 			item->expanded = 1;
 
-			action_box_items[map->num][act->num] = item;
+			action_box_items[keymap_id][act->num] = item;
 		}
 
-		keymap_box_item_info[map->num].first = actions[map->num];
-		keymap_box_item_info[map->num].last = act;
+		keymap_box_item[keymap_id] = keymap_box;
 	}
 }
 
@@ -645,7 +652,7 @@ unlock_keybinding(struct listbox_item *item)
 static int
 is_keybinding_used(struct listbox_item *item)
 {
-	if (item->depth == 2) return 0;
+	if (item->depth != 2) return 0;
 	return is_object_used((struct keybinding *) item->udata);
 }
 
@@ -653,19 +660,22 @@ static unsigned char *
 get_keybinding_text(struct listbox_item *item, struct terminal *term)
 {
 	struct keybinding *keybinding = item->udata;
-	unsigned char *keymap;
 	struct string info;
 
-	if (item->depth < 2) {
-		struct strtonum *strtonum = item->udata;
+	if (item->depth == 0) {
+		struct keymap *keymap = item->udata;
 
-		keymap = keybinding_text_toggle
-			? strtonum->str : _(strtonum->desc, term);
-		return stracpy(keymap);
+		return stracpy(keybinding_text_toggle ? keymap->str
+		                                      : _(keymap->desc, term));
+	} else if (item->depth < 2) {
+		struct action *action = item->udata;
+
+		return stracpy(keybinding_text_toggle ? action->str
+		                                      : _(action->desc, term));
 	}
 
 	if (!init_string(&info)) return NULL;
-	make_keystroke(&info, keybinding->key, keybinding->meta, 0);
+	add_keystroke_to_string(&info, &keybinding->kbd, 0);
 	return info.source;
 }
 
@@ -682,11 +692,11 @@ get_keybinding_info(struct listbox_item *item, struct terminal *term)
 	if (!init_string(&info))
 		return NULL;
 
-	action = write_action(keybinding->keymap, keybinding->action);
-	keymap = write_keymap(keybinding->keymap);
+	action = get_action_name(keybinding->keymap_id, keybinding->action_id);
+	keymap = get_keymap_name(keybinding->keymap_id);
 
 	add_format_to_string(&info, "%s: ", _("Keystroke", term));
-	make_keystroke(&info, keybinding->key, keybinding->meta, 0);
+	add_keystroke_to_string(&info, &keybinding->kbd, 0);
 	add_format_to_string(&info, "\n%s: %s", _("Action", term), action);
 	add_format_to_string(&info, "\n%s: %s", _("Keymap", term), keymap);
 
@@ -700,20 +710,13 @@ get_keybinding_root(struct listbox_item *item)
 	if (item->depth == 0) return NULL;
 
 	if (item->depth == 1) {
-		struct strtonum *action = item->udata;
-		int keymap;
+		struct action *action = item->udata;
 
-		for (keymap = 0; keymap < KEYMAP_MAX; keymap++) {
-			if (keymap_box_item_info[keymap].first <= action
-			    && keymap_box_item_info[keymap].last > action)
-				return keymap_box_item_info[keymap].box_item;
-		}
-
-		return NULL;
+		return keymap_box_item[action->keymap_id];
 	} else {
 		struct keybinding *kb = item->udata;
 
-		return get_keybinding_action_box_item(kb->keymap, kb->action);
+		return get_keybinding_action_box_item(kb->keymap_id, kb->action_id);
 	}
 }
 
@@ -721,14 +724,14 @@ static enum listbox_match
 match_keybinding(struct listbox_item *item, struct terminal *term,
 		 unsigned char *text)
 {
-	struct strtonum *strtonum = item->udata;
+	struct action *action = item->udata;
 	unsigned char *desc;
 
 	if (item->depth != 1)
 		return LISTBOX_MATCH_IMPOSSIBLE;
 
 	desc = keybinding_text_toggle
-	     ? strtonum->str : _(strtonum->desc, term);
+	     ? action->str : _(action->desc, term);
 
 	if ((desc && strcasestr(desc, text)))
 		return LISTBOX_MATCH_OK;
@@ -771,12 +774,15 @@ static struct listbox_ops keybinding_listbox_ops = {
 
 struct kbdbind_add_hop {
 	struct terminal *term;
-	int action, keymap;
-	long key, meta;
+	action_id_T action_id;
+	enum keymap_id keymap_id;
+	struct term_event_keyboard kbd;
+	struct widget_data *widget_data;
 };
 
 struct kbdbind_add_hop *
-new_hop_from(struct kbdbind_add_hop *hop) {
+new_hop_from(struct kbdbind_add_hop *hop)
+{
 	struct kbdbind_add_hop *new_hop = mem_alloc(sizeof(*new_hop));
 
 	if (new_hop)
@@ -789,24 +795,29 @@ static void
 really_really_add_keybinding(void *data)
 {
 	struct kbdbind_add_hop *hop = data;
+	struct keybinding *keybinding;
 
 	assert(hop);
 
-	add_keybinding(hop->keymap, hop->action, hop->key, hop->meta, 0);
+	keybinding = add_keybinding(hop->keymap_id, hop->action_id, &hop->kbd,
+	                            EVENT_NONE);
+
+	if (keybinding && keybinding->box_item)
+		listbox_sel(hop->widget_data, keybinding->box_item);
 }
 
 static void
 really_add_keybinding(void *data, unsigned char *keystroke)
 {
 	struct kbdbind_add_hop *hop = data;
-	int action;
+	action_id_T action_id;
 
-	if (keybinding_exists(hop->keymap, hop->key, hop->meta, &action)
-	    && action != ACT_MAIN_NONE) {
+	if (keybinding_exists(hop->keymap_id, &hop->kbd, &action_id)
+	    && action_id != ACT_MAIN_NONE) {
 		struct kbdbind_add_hop *new_hop;
 
 		/* Same keystroke for same action, just return. */
-		if (action == hop->action) return;
+		if (action_id == hop->action_id) return;
 
 		new_hop = new_hop_from(hop);
 		if (!new_hop) return; /* out of mem */
@@ -816,7 +827,7 @@ really_add_keybinding(void *data, unsigned char *keystroke)
 			msg_text(new_hop->term, N_("The keystroke \"%s\" "
 			"is currently used for \"%s\".\n"
 			"Are you sure you want to replace it?"),
-			keystroke, write_action(hop->keymap, action)),
+			keystroke, get_action_name(hop->keymap_id, action_id)),
 			new_hop, 2,
 			N_("~Yes"), really_really_add_keybinding, B_ENTER,
 			N_("~No"), NULL, B_ESC);
@@ -827,13 +838,13 @@ really_add_keybinding(void *data, unsigned char *keystroke)
 	really_really_add_keybinding((void *) hop);
 }
 
-t_handler_event_status
+widget_handler_status_T
 check_keystroke(struct dialog_data *dlg_data, struct widget_data *widget_data)
 {
 	struct kbdbind_add_hop *hop = dlg_data->dlg->udata2;
 	unsigned char *keystroke = widget_data->cdata;
 
-	if (parse_keystroke(keystroke, &hop->key, &hop->meta) >= 0)
+	if (parse_keystroke(keystroke, &hop->kbd) >= 0)
 		return EVENT_PROCESSED;
 
 	info_box(hop->term, 0, N_("Add keybinding"), ALIGN_CENTER,
@@ -842,7 +853,7 @@ check_keystroke(struct dialog_data *dlg_data, struct widget_data *widget_data)
 	return EVENT_NOT_PROCESSED;
 }
 
-static t_handler_event_status
+static widget_handler_status_T
 push_kbdbind_add_button(struct dialog_data *dlg_data,
 			struct widget_data *some_useless_info_button)
 {
@@ -861,25 +872,18 @@ push_kbdbind_add_button(struct dialog_data *dlg_data,
 	hop = mem_calloc(1, sizeof(*hop));
 	if (!hop) return EVENT_PROCESSED;
 	hop->term = term;
+	hop->widget_data = dlg_data->widgets_data;
 
 	if (item->depth == 2) {
 		struct keybinding *keybinding = item->udata;
 
-		hop->action = keybinding->action;
-		hop->keymap = keybinding->keymap;
+		hop->action_id = keybinding->action_id;
+		hop->keymap_id = keybinding->keymap_id;
 	} else {
-		struct strtonum *strtonum = item->udata;
+		struct action *action = item->udata;
 
-		hop->action = strtonum->num;
-
-		item = get_keybinding_root(item);
-		if (!item) {
-			mem_free(hop);
-			return EVENT_PROCESSED;
-		}
-
-		strtonum = item->udata;
-		hop->keymap = strtonum->num;
+		hop->action_id = action->num;
+		hop->keymap_id = action->keymap_id;
 	}
 
 	text = msg_text(term,
@@ -893,8 +897,8 @@ push_kbdbind_add_button(struct dialog_data *dlg_data,
 			"Tab,Enter,Insert,F5,..."
 			"\n\n"
 			"Keystroke",
-			write_action(hop->keymap, hop->action),
-			write_keymap(hop->keymap));
+			get_action_name(hop->keymap_id, hop->action_id),
+			get_keymap_name(hop->keymap_id));
 
 	input_dialog(term, getml(hop, text, NULL),
 		     N_("Add keybinding"), text,
@@ -906,7 +910,7 @@ push_kbdbind_add_button(struct dialog_data *dlg_data,
 }
 
 
-static t_handler_event_status
+static widget_handler_status_T
 push_kbdbind_toggle_display_button(struct dialog_data *dlg_data,
 		struct widget_data *some_useless_info_button)
 {
@@ -920,7 +924,7 @@ push_kbdbind_toggle_display_button(struct dialog_data *dlg_data,
 
 /* FIXME: Races here, we need to lock the entry..? --pasky */
 
-static t_handler_event_status
+static widget_handler_status_T
 push_kbdbind_save_button(struct dialog_data *dlg_data,
 		struct widget_data *some_useless_info_button)
 {
@@ -928,7 +932,6 @@ push_kbdbind_save_button(struct dialog_data *dlg_data,
 	return EVENT_PROCESSED;
 }
 
-#define	KEYBINDING_MANAGER_BUTTONS	4
 
 static INIT_LIST_HEAD(keybinding_dialog_list);
 
