@@ -1,5 +1,4 @@
 /* Internal cookies implementation */
-/* $Id: cookies.c,v 1.192.2.3 2005/04/06 09:28:22 jonas Exp $ */
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
@@ -24,16 +23,17 @@
 #include "cookies/cookies.h"
 #include "cookies/dialogs.h"
 #include "cookies/parser.h"
+#include "config/home.h"
 #include "config/kbdbind.h"
 #include "config/options.h"
 #include "intl/gettext/libintl.h"
-#include "lowlevel/home.h"
-#include "modules/module.h"
+#include "main/module.h"
+#include "main/object.h"
 #include "protocol/date.h"
 #include "protocol/header.h"
 #include "protocol/protocol.h"
 #include "protocol/uri.h"
-#include "sched/session.h"
+#include "session/session.h"
 #include "terminal/terminal.h"
 #include "util/conv.h"
 #ifdef DEBUG_COOKIES
@@ -41,10 +41,9 @@
 #endif
 #include "util/file.h"
 #include "util/memory.h"
-#include "util/object.h"
 #include "util/secsave.h"
 #include "util/string.h"
-#include "util/ttime.h"
+#include "util/time.h"
 
 #define COOKIES_FILENAME		"cookies"
 
@@ -190,7 +189,7 @@ done_cookie_server(struct cookie_server *cs)
 }
 
 void
-free_cookie(struct cookie *c)
+done_cookie(struct cookie *c)
 {
 	if (c->box_item) done_listbox_item(&cookie_browser, c->box_item);
 	if (c->server) done_cookie_server(c->server);
@@ -201,6 +200,13 @@ free_cookie(struct cookie *c)
 	mem_free(c);
 }
 
+void
+delete_cookie(struct cookie *c)
+{
+	del_from_list(c);
+	done_cookie(c);
+}
+
 
 /* Check whether cookie's domain matches server.
  * It returns 1 if ok, 0 else. */
@@ -309,7 +315,7 @@ set_cookie(struct uri *uri, unsigned char *str)
 	    || !cookie->name
 	    || !cookie->value
 	    || !cookie->server) {
-		free_cookie(cookie);
+		done_cookie(cookie);
 		return;
 	}
 
@@ -321,7 +327,7 @@ set_cookie(struct uri *uri, unsigned char *str)
 #ifdef DEBUG_COOKIES
 		DBG("Dropped.");
 #endif
-		free_cookie(cookie);
+		done_cookie(cookie);
 		return;
 	}
 #endif
@@ -343,14 +349,14 @@ set_cookie(struct uri *uri, unsigned char *str)
 		unsigned char *date = parse_header_param(str, "expires");
 
 		if (date) {
-			ttime expires = parse_date(&date, NULL, 0, 1); /* Convert date to seconds. */
+			time_t expires = parse_date(&date, NULL, 0, 1); /* Convert date to seconds. */
 
 			mem_free(date);
 
 			if (expires) {
 				if (max_age > 0) {
 					int seconds = max_age*24*3600;
-					ttime deadline = time(NULL) + seconds;
+					time_t deadline = time(NULL) + seconds;
 
 					if (expires > deadline) /* Over-aged cookie ? */
 						expires = deadline;
@@ -367,7 +373,7 @@ set_cookie(struct uri *uri, unsigned char *str)
 
 		path = get_uri_string(uri, URI_PATH);
 		if (!path) {
-			free_cookie(cookie);
+			done_cookie(cookie);
 			return;
 		}
 
@@ -431,13 +437,6 @@ set_cookie(struct uri *uri, unsigned char *str)
 	accept_cookie(cookie);
 }
 
-static void
-del_from_cookie_list(struct cookie *cookie)
-{
-	del_from_list(cookie);
-	free_cookie(cookie);
-}
-
 void
 accept_cookie(struct cookie *cookie)
 {
@@ -460,7 +459,7 @@ accept_cookie(struct cookie *cookie)
 			    || strcasecmp(c->domain, cookie->domain))
 				continue;
 
-			del_from_cookie_list(c);
+			delete_cookie(c);
 		}
 	}
 
@@ -507,7 +506,7 @@ delete_cookie(struct cookie *c)
 
 end:
 	del_from_list(c);
-	free_cookie(c);
+	done_cookie(c);
 
 	if (get_cookies_save() && get_cookies_resave())
 		save_cookies();
@@ -603,9 +602,6 @@ is_path_prefix(unsigned char *d, unsigned char *s)
 }
 
 
-#define is_expired(t) ((t) && (t) <= time(NULL))
-#define is_dead(t) (!(t) || (t) <= time(NULL))
-
 struct string *
 send_cookies(struct uri *uri)
 {
@@ -613,6 +609,7 @@ send_cookies(struct uri *uri)
 	struct cookie *c, *next;
 	unsigned char *path = NULL;
 	static struct string header;
+	time_t now;
 
 	if (!uri->host || !uri->data)
 		return NULL;
@@ -627,17 +624,18 @@ send_cookies(struct uri *uri)
 
 	init_string(&header);
 
+	now = time(NULL);
 	foreachsafe (c, next, cookies) {
 		if (!is_in_domain(c->domain, uri->host, uri->hostlen)
 		    || !is_path_prefix(c->path, path))
 			continue;
 
-		if (is_expired(c->expires)) {
+		if (c->expires && c->expires <= now) {
 #ifdef DEBUG_COOKIES
 			DBG("Cookie %s=%s (exp %d) expired.",
 			    c->name, c->value, c->expires);
 #endif
-			del_from_cookie_list(c);
+			delete_cookie(c);
 
 			cookies_dirty = 1;
 			continue;
@@ -682,6 +680,7 @@ load_cookies(void) {
 	unsigned char in_buffer[6 * MAX_STR_LEN];
 	unsigned char *cookfile = COOKIES_FILENAME;
 	FILE *fp;
+	time_t now;
 
 	if (elinks_home) {
 		cookfile = straconcat(elinks_home, cookfile, NULL);
@@ -694,7 +693,7 @@ load_cookies(void) {
 	done_cookies(&cookies_module);
 	cookies_nosave = 0;
 
-	fp = fopen(cookfile, "r");
+	fp = fopen(cookfile, "rb");
 	if (elinks_home) mem_free(cookfile);
 	if (!fp) return;
 
@@ -702,6 +701,7 @@ load_cookies(void) {
 	 * periodically to our death. */
 	cookies_nosave = 1;
 
+	now = time(NULL);
 	while (fgets(in_buffer, 6 * MAX_STR_LEN, fp)) {
 		struct cookie *cookie;
 		unsigned char *p, *q = in_buffer;
@@ -710,7 +710,7 @@ load_cookies(void) {
 			unsigned char *pos;
 			int len;
 		} members[MEMBERS];
-		ttime expires;
+		time_t expires;
 
 		/* First find all members. */
 		for (member = NAME; member < MEMBERS; member++, q = ++p) {
@@ -728,8 +728,8 @@ load_cookies(void) {
 		if (member != MEMBERS) continue;	/* Invalid line. */
 
 		/* Skip expired cookies if any. */
-		expires = str_to_ttime(members[EXPIRES].pos);
-		if (is_dead(expires)) {
+		expires = str_to_time_t(members[EXPIRES].pos);
+		if (!expires || expires <= now) {
 			cookies_dirty = 1;
 			continue;
 		}
@@ -747,7 +747,7 @@ load_cookies(void) {
 		/* Check whether all fields were correctly allocated. */
 		if (!cookie->server || !cookie->name || !cookie->value
 		    || !cookie->path || !cookie->domain) {
-			free_cookie(cookie);
+			done_cookie(cookie);
 			continue;
 		}
 
@@ -766,6 +766,7 @@ save_cookies(void) {
 	struct cookie *c;
 	unsigned char *cookfile;
 	struct secure_save_info *ssi;
+	time_t now;
 
 	if (cookies_nosave || !elinks_home || !cookies_dirty
 	    || get_cmd_opt_bool("anonymous"))
@@ -774,12 +775,13 @@ save_cookies(void) {
 	cookfile = straconcat(elinks_home, COOKIES_FILENAME, NULL);
 	if (!cookfile) return;
 
-	ssi = secure_open(cookfile, 0177); /* rw for user only */
+	ssi = secure_open(cookfile);
 	mem_free(cookfile);
 	if (!ssi) return;
 
+	now = time(NULL);
 	foreach (c, cookies) {
-		if (is_dead(c->expires)) continue;
+		if (!c->expires || c->expires <= now) continue;
 		if (secure_fprintf(ssi, "%s\t%s\t%s\t%s\t%s\t%ld\t%d\n",
 				   c->name, c->value,
 				   c->server->host,
@@ -805,8 +807,7 @@ free_cookies_list(struct list_head *list)
 	while (!list_empty(*list)) {
 		struct cookie *cookie = list->next;
 
-		del_from_list(cookie);
-		free_cookie(cookie);
+		delete_cookie(cookie);
 	}
 }
 
