@@ -1,5 +1,4 @@
 /* Menu system */
-/* $Id: menu.c,v 1.381.2.8 2005/05/01 21:32:11 jonas Exp $ */
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
@@ -22,21 +21,21 @@
 #include "dialogs/menu.h"
 #include "dialogs/options.h"
 #include "intl/gettext/libintl.h"
-#include "lowlevel/select.h"
-#include "main.h"
+#include "main/event.h"
+#include "main/main.h"
+#include "main/select.h"
 #include "mime/dialogs.h"
 #include "mime/mime.h"
+#include "network/connection.h"
 #include "osdep/osdep.h"
 #include "osdep/newwin.h"
 #include "protocol/protocol.h"
 #include "protocol/uri.h"
-#include "sched/connection.h"
-#include "sched/download.h"
-#include "sched/event.h"
-#include "sched/history.h"
-#include "sched/location.h"
-#include "sched/session.h"
-#include "sched/task.h"
+#include "session/download.h"
+#include "session/history.h"
+#include "session/location.h"
+#include "session/session.h"
+#include "session/task.h"
 #include "terminal/tab.h"
 #include "terminal/terminal.h"
 #include "util/conv.h"
@@ -44,6 +43,7 @@
 #include "util/memlist.h"
 #include "util/memory.h"
 #include "util/string.h"
+#include "viewer/action.h"
 #include "viewer/text/link.h"
 #include "viewer/text/view.h"
 
@@ -74,7 +74,7 @@ save_url(struct session *ses, unsigned char *url)
 
 	uri = get_translated_uri(url, ses->tab->term->cwd);
 	if (!uri) {
-		print_error_dialog(ses, S_BAD_URL, NULL, PRI_CANCEL);
+		print_error_dialog(ses, S_BAD_URL, uri, PRI_CANCEL);
 		return;
 	}
 
@@ -103,7 +103,7 @@ save_url_as(struct session *ses)
 void
 really_exit_prog(struct session *ses)
 {
-	register_bottom_half((void (*)(void *)) destroy_terminal, ses->tab->term);
+	register_bottom_half(destroy_terminal, ses->tab->term);
 }
 
 static inline void
@@ -359,15 +359,15 @@ do_file_menu(struct terminal *term, void *xxx, void *ses_)
 	}
 
 	memcpy(e, file_menu11, sizeof(file_menu11));
-	e += sizeof(file_menu11) / sizeof(struct menu_item);
+	e += sizeof_array(file_menu11);
 
 	if (!anonymous) {
 		memcpy(e, file_menu21, sizeof(file_menu21));
-		e += sizeof(file_menu21) / sizeof(struct menu_item);
+		e += sizeof_array(file_menu21);
 	}
 
 	memcpy(e, file_menu22, sizeof(file_menu22));
-	e += sizeof(file_menu22) / sizeof(struct menu_item);
+	e += sizeof_array(file_menu22);
 
 	x = 1;
 	if (!anonymous && can_open_os_shell(term->environment)) {
@@ -385,7 +385,7 @@ do_file_menu(struct terminal *term, void *xxx, void *ses_)
 	}
 
 	memcpy(e, file_menu3 + x, sizeof(file_menu3) - x * sizeof(struct menu_item));
-	e += sizeof(file_menu3) / sizeof(struct menu_item);
+	e += sizeof_array(file_menu3);
 
 	for (f = file_menu; f < e; f++)
 		f->flags |= FREE_LIST;
@@ -429,9 +429,7 @@ static struct menu_item help_menu[] = {
 	BAR_MENU_ITEM,
 	INIT_MENU_ITEM(N_("~Bugs information"), NULL, ACT_MAIN_NONE, menu_url_shortcut, ELINKS_BUGS_URL, 0),
 #ifdef CONFIG_DEBUG
-	INIT_MENU_ITEM(N_("ELinks C~vsWeb"), NULL, ACT_MAIN_NONE, menu_url_shortcut, ELINKS_CVSWEB_URL, 0),
-	INIT_MENU_ITEM(N_("ELinks Cvs ~History"), NULL, ACT_MAIN_NONE, menu_url_shortcut, ELINKS_CVSHIST_URL, 0),
-	INIT_MENU_ITEM(N_("ELinks ~LXR"), NULL, ACT_MAIN_NONE, menu_url_shortcut, ELINKS_LXR_URL, 0),
+	INIT_MENU_ITEM(N_("ELinks ~GITWeb"), NULL, ACT_MAIN_NONE, menu_url_shortcut, ELINKS_GITWEB_URL, 0),
 #endif
 	BAR_MENU_ITEM,
 	INIT_MENU_ITEM(N_("~Copying"), NULL, ACT_MAIN_NONE, menu_copying, NULL, 0),
@@ -448,7 +446,7 @@ static struct menu_item ext_menu[] = {
 };
 
 static struct menu_item setup_menu[] = {
-#ifdef ENABLE_NLS
+#ifdef CONFIG_NLS
 	INIT_MENU_ITEM(N_("~Language"), NULL, ACT_MAIN_NONE, menu_language_list, NULL, SUBMENU),
 #endif
 	INIT_MENU_ITEM(N_("C~haracter set"), NULL, ACT_MAIN_NONE, charset_list, NULL, SUBMENU),
@@ -552,7 +550,7 @@ query_file(struct session *ses, struct uri *uri, void *data,
 		return;
 	}
 
-	if (get_protocol_external_handler(uri->protocol)) {
+	if (get_protocol_external_handler(ses->tab->term, uri)) {
 		print_error_dialog(ses, S_EXTERNAL_PROTOCOL, uri, PRI_CANCEL);
 		return;
 	}
@@ -601,8 +599,9 @@ add_cmdline_bool_option(struct string *string, unsigned char *name)
 }
 
 void
-open_uri_in_new_window(struct session *ses, struct uri *uri,
-		       enum term_env_type env)
+open_uri_in_new_window(struct session *ses, struct uri *uri, struct uri *referrer,
+		       enum term_env_type env, enum cache_mode cache_mode,
+		       enum task_type task)
 {
 	int ring = get_cmd_opt_int("session-ring");
 	struct string parameters;
@@ -611,7 +610,7 @@ open_uri_in_new_window(struct session *ses, struct uri *uri,
 	assert(env && ses);
 	if_assert_failed return;
 
-	id = add_session_info(ses, uri);
+	id = add_session_info(ses, uri, referrer, cache_mode, task);
 	if (id < 1) return;
 
 	if (!init_string(&parameters)) return;
@@ -629,7 +628,7 @@ open_uri_in_new_window(struct session *ses, struct uri *uri,
 		add_cmdline_bool_option(&parameters, "no-connect");
 	}
 
-	open_new_window(ses->tab->term, path_to_exe, env, parameters.source);
+	open_new_window(ses->tab->term, program.path, env, parameters.source);
 	done_string(&parameters);
 }
 
@@ -654,7 +653,8 @@ send_open_in_new_window(struct terminal *term, const struct open_in_new *open,
 	uri = get_link_uri(ses, doc_view, link);
 	if (!uri) return;
 
-	open_uri_in_new_window(ses, uri, open->env);
+	open_uri_in_new_window(ses, uri, NULL, open->env,
+			       CACHE_MODE_NORMAL, TASK_NONE);
 	done_uri(uri);
 }
 
@@ -662,14 +662,15 @@ void
 send_open_new_window(struct terminal *term, const struct open_in_new *open,
 		     struct session *ses)
 {
-	open_uri_in_new_window(ses, NULL, open->env);
+	open_uri_in_new_window(ses, NULL, NULL, open->env,
+			       CACHE_MODE_NORMAL, TASK_NONE);
 }
 
 
 void
 open_in_new_window(struct terminal *term, void *func_, void *ses_)
 {
-	menu_func func = func_;
+	menu_func_T func = func_;
 	struct session *ses = ses_;
 	struct menu_item *mi;
 	int posibilities;
@@ -861,23 +862,23 @@ add_uri_command_to_menu(struct menu_item **mi, enum pass_uri_type type)
 	struct option *option;
 	int commands = 0;
 	enum menu_item_flags flags = NO_FLAG;
-	int action;
+	action_id_T action_id;
 	unsigned char *text;
 
 	switch (type) {
 	case PASS_URI_FRAME:
-		action = ACT_MAIN_FRAME_EXTERNAL_COMMAND;
+		action_id = ACT_MAIN_FRAME_EXTERNAL_COMMAND;
 		text = N_("~Pass frame URI to external command");
 		break;
 
 	case PASS_URI_LINK:
-		action = ACT_MAIN_LINK_EXTERNAL_COMMAND;
+		action_id = ACT_MAIN_LINK_EXTERNAL_COMMAND;
 		text = N_("Pass link URI to e~xternal command");
 		break;
 
 	default:
 	case PASS_URI_TAB:
-		action = ACT_MAIN_TAB_EXTERNAL_COMMAND;
+		action_id = ACT_MAIN_TAB_EXTERNAL_COMMAND;
 		text = N_("Pass tab URI to e~xternal command");
 	};
 
@@ -894,7 +895,7 @@ add_uri_command_to_menu(struct menu_item **mi, enum pass_uri_type type)
 
 	if (commands == 0) return;
 
-	add_to_menu(mi, text, NULL, action, NULL, NULL, flags);
+	add_to_menu(mi, text, NULL, action_id, NULL, NULL, flags);
 }
 
 
@@ -911,7 +912,7 @@ static struct menu_item empty_directory_menu[] = {
  * else the menu is launched. */
 static void
 complete_file_menu(struct terminal *term, int no_elevator, void *data,
-		   menu_func file_func, menu_func dir_func,
+		   menu_func_T file_func, menu_func_T dir_func,
 		   unsigned char *dirname, unsigned char *filename)
 {
 	struct menu_item *menu = new_menu(FREE_LIST | NO_INTL);
@@ -1011,7 +1012,7 @@ complete_file_menu(struct terminal *term, int no_elevator, void *data,
  * and splitting it in directory and file name part. */
 void
 auto_complete_file(struct terminal *term, int no_elevator, unsigned char *path,
-		   menu_func file_func, menu_func dir_func, void *data)
+		   menu_func_T file_func, menu_func_T dir_func, void *data)
 {
 	struct uri *uri;
 	unsigned char *dirname;
