@@ -1,10 +1,13 @@
 /* DOM document renderer */
-/* $Id: renderer.c,v 1.17.2.5 2005/09/14 13:30:26 jonas Exp $ */
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
 
+#include <sys/types.h> /* FreeBSD needs this before regex.h */
+#ifdef HAVE_REGEX_H
+#include <regex.h>
+#endif
 #include <string.h>
 
 #include "elinks.h"
@@ -17,11 +20,13 @@
 #include "document/css/stylesheet.h"
 #include "document/docdata.h"
 #include "document/document.h"
-#include "document/dom/navigator.h"
-#include "document/dom/node.h"
 #include "document/dom/renderer.h"
 #include "document/renderer.h"
-#include "document/sgml/parser.h"
+#include "dom/scanner.h"
+#include "dom/sgml/parser.h"
+#include "dom/sgml/html/html.h"
+#include "dom/node.h"
+#include "dom/stack.h"
 #include "intl/charsets.h"
 #include "globhist/globhist.h"		/* get_global_history_item() */
 #include "protocol/uri.h"
@@ -34,12 +39,13 @@
 
 
 struct dom_renderer {
+	enum sgml_document_type doctype;
 	struct document *document;
 
 	struct conv_table *convert_table;
 	enum convert_string_mode convert_mode;
 
-	struct dom_node *root;
+	struct uri *base_uri;
 
 	unsigned char *source;
 	unsigned char *end;
@@ -47,18 +53,26 @@ struct dom_renderer {
 	unsigned char *position;
 	int canvas_x, canvas_y;
 
+#ifdef HAVE_REGEX_H
+	regex_t url_regex;
+	unsigned int find_url:1;
+#endif
 	struct screen_char styles[DOM_NODES];
 };
 
+#define URL_REGEX "(file://|((f|ht|nt)tp(s)?|smb)://[[:alnum:]]+([-@:.]?[[:alnum:]])*\\.[[:alpha:]]{2,4}(:[[:digit:]]+)?)(/(%[[:xdigit:]]{2}|[-_~&=;?.a-z0-9])*)*"
+#define URL_REGFLAGS (REG_ICASE | REG_EXTENDED)
 
 static void
-init_template(struct screen_char *template, color_t background, color_t foreground)
+init_template(struct screen_char *template, struct document_options *options,
+	      color_T background, color_T foreground)
 {
 	struct color_pair colors = INIT_COLOR_PAIR(background, foreground);
 
 	template->attr = 0;
 	template->data = ' ';
-	set_term_color(template, &colors, global_doc_opts->color_flags, global_doc_opts->color_mode);
+	set_term_color(template, &colors,
+		       options->color_flags, options->color_mode);
 }
 
 static inline struct css_property *
@@ -77,8 +91,7 @@ get_css_property(struct list_head *list, enum css_property_type type)
 /* Checks the user CSS for properties for each DOM node type name */
 static inline void
 init_dom_renderer(struct dom_renderer *renderer, struct document *document,
-		  struct string *buffer, struct dom_node *root,
-		  struct conv_table *convert_table)
+		  struct string *buffer, struct conv_table *convert_table)
 {
 	enum dom_node_type type;
 	struct css_stylesheet *css = &default_stylesheet;
@@ -88,19 +101,28 @@ init_dom_renderer(struct dom_renderer *renderer, struct document *document,
 	renderer->document	= document;
 	renderer->convert_table = convert_table;
 	renderer->convert_mode	= document->options.plain ? CSM_NONE : CSM_DEFAULT;
-	renderer->root		= root;
 	renderer->source	= buffer->source;
 	renderer->end		= buffer->source + buffer->length;
 	renderer->position	= renderer->source;
+	renderer->base_uri	= get_uri_reference(document->uri);
+
+#ifdef HAVE_REGEX_H
+	if (renderer->document->options.plain_display_links) {
+		if (regcomp(&renderer->url_regex, URL_REGEX, URL_REGFLAGS)) {
+			regfree(&renderer->url_regex);
+		} else {
+			renderer->find_url = 1;
+		}
+	}
+#endif
 
 	for (type = 0; type < DOM_NODES; type++) {
 		struct screen_char *template = &renderer->styles[type];
-		color_t background = global_doc_opts->default_bg;
-		color_t foreground = global_doc_opts->default_fg;
+		color_T background = document->options.default_bg;
+		color_T foreground = document->options.default_fg;
 		static int i_want_struct_module_for_dom;
 
-		unsigned char *name = get_dom_node_type_name(type);
-		int namelen = name ? strlen(name) : 0;
+		struct dom_string *name = get_dom_node_type_name(type);
 		struct css_selector *selector = NULL;
 
 		if (!i_want_struct_module_for_dom) {
@@ -110,7 +132,8 @@ init_dom_renderer(struct dom_renderer *renderer, struct document *document,
 				"entity-reference { color: red } "
 				"proc-instruction { color: red } "
 				"attribute	{ color: magenta } "
-				"comment	{ color: aqua } ";
+				"comment	{ color: aqua } "
+				"cdata-section	{ color: orange2 } ";
 			unsigned char *styles = (unsigned char *) default_colors;
 
 			i_want_struct_module_for_dom = 1;
@@ -121,9 +144,10 @@ init_dom_renderer(struct dom_renderer *renderer, struct document *document,
 		}
 
 		if (name)
+		if (is_dom_string_set(name))
 			selector = find_css_selector(&css->selectors,
 						     CST_ELEMENT, CSR_ROOT,
-						     name, namelen);
+						     name->string, name->length);
 
 		if (selector) {
 			struct list_head *properties = &selector->properties;
@@ -140,7 +164,7 @@ init_dom_renderer(struct dom_renderer *renderer, struct document *document,
 			if (property) foreground = property->value.color;
 		}
 
-		init_template(template, background, foreground);
+		init_template(template, &document->options, background, foreground);
 	}
 }
 
@@ -198,7 +222,8 @@ render_dom_line(struct dom_renderer *renderer, struct screen_char *template,
 
 	assert(renderer && template && string && length);
 
-	string = convert_string(convert, string, length, mode, &length, NULL, NULL);
+	string = convert_string(convert, string, length, document->options.cp,
+	                        mode, &length, NULL, NULL);
 	if (!string) return;
 
 	if (!realloc_line(document, WIDTH(renderer, length), Y(renderer))) {
@@ -287,33 +312,6 @@ render_dom_text(struct dom_renderer *renderer, struct screen_char *template,
 	}
 }
 
-#ifdef DOM_TREE_RENDERER
-static void
-render_dom_printf(struct dom_renderer *renderer, struct screen_char *template,
-		  unsigned char *format, ...)
-{
-	unsigned char *text;
-	int textlen;
-	va_list ap, ap2;
-
-	va_start(ap, format);
-	VA_COPY(ap2, ap);
-
-	textlen = vsnprintf(NULL, 0, format, ap2);
-
-	text = mem_alloc(textlen + 1);
-	if (!text) goto free_va_args;
-
-	if (vsnprintf((char *) text, textlen + 1, format, ap) == textlen)
-		render_dom_text(renderer, template, text, textlen);
-
-	mem_free(text);
-
-free_va_args:
-	va_end(ap);
-}
-#endif /* DOM_TREE_RENDERER */
-
 #define realloc_document_links(doc, size) \
 	ALIGN_LINK(&(doc)->links, (doc)->nlinks, size)
 
@@ -328,7 +326,7 @@ add_dom_link(struct dom_renderer *renderer, unsigned char *string, int length)
 	struct point *point;
 	struct screen_char template;
 	unsigned char *uristring;
-	color_t fgcolor;
+	color_T fgcolor;
 
 	if (!realloc_document_links(document, document->nlinks + 1))
 		return NULL;
@@ -339,11 +337,11 @@ add_dom_link(struct dom_renderer *renderer, unsigned char *string, int length)
 		return NULL;
 
 	uristring = convert_string(renderer->convert_table,
-				   string, length, CSM_DEFAULT,
-				   NULL, NULL, NULL);
+				   string, length, document->options.cp,
+	                           CSM_DEFAULT, NULL, NULL, NULL);
 	if (!uristring) return NULL;
 
-	where = join_urls(document->uri, uristring);
+	where = join_urls(renderer->base_uri, uristring);
 
 	mem_free(uristring);
 
@@ -367,7 +365,8 @@ add_dom_link(struct dom_renderer *renderer, unsigned char *string, int length)
 	link->color.foreground = fgcolor;
 	link->number = document->nlinks;
 
-	init_template(&template, link->color.background, link->color.foreground);
+	init_template(&template, &document->options,
+		      link->color.background, link->color.foreground);
 
 	render_dom_text(renderer, &template, string, length);
 
@@ -382,112 +381,6 @@ add_dom_link(struct dom_renderer *renderer, unsigned char *string, int length)
 }
 
 
-/* DOM Tree Renderer */
-
-#ifdef DOM_TREE_RENDERER
-static struct dom_node *
-render_dom_tree(struct dom_navigator *navigator, struct dom_node *node, void *data)
-{
-	struct dom_renderer *renderer = navigator->data;
-	struct screen_char *template = &renderer->styles[node->type];
-	unsigned char *name, *value;
-
-	assert(node && renderer);
-
-	name  = get_dom_node_name(node);
-	value = memacpy(node->string, node->length);
-
-	render_dom_printf(renderer, template, "%-16s: %s\n", name, value);
-
-	if (name) mem_free(name);
-	if (value) mem_free(value);
-
-	return node;
-}
-
-static struct dom_node *
-render_dom_tree_id_leaf(struct dom_navigator *navigator, struct dom_node *node, void *data)
-{
-	struct dom_renderer *renderer = navigator->data;
-	struct document *document = renderer->document;
-	struct screen_char *template = &renderer->styles[node->type];
-	unsigned char *name, *value, *id;
-
-	assert(node && document);
-
-	name	= get_dom_node_name(node);
-	value	= get_dom_node_value(node);
-	id	= get_dom_node_type_name(node->type);
-
-	renderer->canvas_x += navigator->depth;
-	render_dom_printf(renderer, template, "%-16s: %s -> %s\n", id, name, value);
-
-	if (name) mem_free(name);
-	if (value) mem_free(value);
-
-	return node;
-}
-
-static struct dom_node *
-render_dom_tree_leaf(struct dom_navigator *navigator, struct dom_node *node, void *data)
-{
-	struct dom_renderer *renderer = navigator->data;
-	struct document *document = renderer->document;
-	struct screen_char *template = &renderer->styles[node->type];
-	unsigned char *name, *value;
-
-	assert(node && document);
-
-	name	= get_dom_node_name(node);
-	value	= get_dom_node_value(node);
-
-	renderer->canvas_x += navigator->depth;
-	render_dom_printf(renderer, template, "%-16s: %s\n", name, value);
-
-	if (name) mem_free(name);
-	if (value) mem_free(value);
-
-	return node;
-}
-
-static struct dom_node *
-render_dom_tree_branch(struct dom_navigator *navigator, struct dom_node *node, void *data)
-{
-	struct dom_renderer *renderer = navigator->data;
-	struct document *document = renderer->document;
-	struct screen_char *template = &renderer->styles[node->type];
-	unsigned char *name, *id;
-
-	assert(node && document);
-
-	name	= get_dom_node_name(node);
-	id	= get_dom_node_type_name(node->type);
-
-	renderer->canvas_x += navigator->depth;
-	render_dom_printf(renderer, template, "%-16s: %s\n", id, name);
-
-	if (name) mem_free(name);
-
-	return node;
-}
-
-static dom_navigator_callback_T dom_tree_renderer_callbacks[DOM_NODES] = {
-	/*				*/ NULL,
-	/* DOM_NODE_ELEMENT		*/ render_dom_tree_branch,
-	/* DOM_NODE_ATTRIBUTE		*/ render_dom_tree_id_leaf,
-	/* DOM_NODE_TEXT		*/ render_dom_tree_leaf,
-	/* DOM_NODE_CDATA_SECTION	*/ render_dom_tree_id_leaf,
-	/* DOM_NODE_ENTITY_REFERENCE	*/ render_dom_tree_id_leaf,
-	/* DOM_NODE_ENTITY		*/ render_dom_tree_id_leaf,
-	/* DOM_NODE_PROC_INSTRUCTION	*/ render_dom_tree_id_leaf,
-	/* DOM_NODE_COMMENT		*/ render_dom_tree_leaf,
-	/* DOM_NODE_DOCUMENT		*/ render_dom_tree,
-	/* DOM_NODE_DOCUMENT_TYPE	*/ render_dom_tree_id_leaf,
-	/* DOM_NODE_DOCUMENT_FRAGMENT	*/ render_dom_tree_id_leaf,
-	/* DOM_NODE_NOTATION		*/ render_dom_tree_id_leaf,
-};
-#endif /* DOM_TREE_RENDERER */
-
 /* DOM Source Renderer */
 
 #define check_dom_node_source(renderer, str, len)	\
@@ -517,8 +410,8 @@ static inline void
 render_dom_node_text(struct dom_renderer *renderer, struct screen_char *template,
 		     struct dom_node *node)
 {
-	unsigned char *string = node->string;
-	int length = node->length;
+	unsigned char *string = node->string.string;
+	int length = node->string.length;
 
 	if (node->type == DOM_NODE_ENTITY_REFERENCE) {
 		string -= 1;
@@ -534,87 +427,143 @@ render_dom_node_text(struct dom_renderer *renderer, struct screen_char *template
 	render_dom_text(renderer, template, string, length);
 }
 
-static struct dom_node *
-render_dom_node_source(struct dom_navigator *navigator, struct dom_node *node, void *data)
+#ifdef HAVE_REGEX_H
+static inline void
+render_dom_node_enhanced_text(struct dom_renderer *renderer, struct dom_node *node)
 {
-	struct dom_renderer *renderer = navigator->data;
+	regex_t *regex = &renderer->url_regex;
+	regmatch_t regmatch;
+	unsigned char *string = node->string.string;
+	int length = node->string.length;
+	struct screen_char *template = &renderer->styles[node->type];
+	unsigned char *alloc_string;
 
-	assert(node && renderer && renderer->document);
-
-	/* TODO: For (atleast) text, CDATA section and comment nodes check
-	 * for URIs ala document->options.plain_display_links */
-	render_dom_node_text(renderer, &renderer->styles[node->type], node);
-
-	return node;
-}
-
-static struct dom_node *
-render_dom_proc_instr_source(struct dom_navigator *navigator, struct dom_node *node, void *data)
-{
-	struct dom_renderer *renderer = navigator->data;
-	unsigned char *value;
-	int valuelen;
-
-	assert(node && renderer && renderer->document);
-
-	render_dom_node_text(renderer, &renderer->styles[node->type], node);
-
-	value	 = node->data.proc_instruction.instruction;
-	valuelen = node->data.proc_instruction.instructionlen;
-
-	if (!value || node->data.proc_instruction.map)
-		return node;
-
-	if (check_dom_node_source(renderer, node->string, node->length)) {
-		render_dom_flush(renderer, value);
-		renderer->position = value + valuelen;
+	if (check_dom_node_source(renderer, string, length)) {
+		render_dom_flush(renderer, string);
+		renderer->position = string + length;
+		assert_source(renderer, renderer->position, 0);
 	}
 
-	render_dom_text(renderer, &renderer->styles[DOM_NODE_ATTRIBUTE], value, valuelen);
+	alloc_string = memacpy(string, length);
+	if (alloc_string)
+		string = alloc_string;
 
-	return node;
+	while (length > 0 && !regexec(regex, string, 1, &regmatch, 0)) {
+		int matchlen = regmatch.rm_eo - regmatch.rm_so;
+		int offset = regmatch.rm_so;
+
+		if (!matchlen || offset < 0 || regmatch.rm_eo > length)
+			break;
+
+		if (offset > 0)
+			render_dom_text(renderer, template, string, offset);
+
+		string += offset;
+		length -= offset;
+
+		add_dom_link(renderer, string, matchlen);
+
+		length -= matchlen;
+		string += matchlen;
+	}
+
+	if (length > 0)
+		render_dom_text(renderer, template, string, length);
+
+	mem_free_if(alloc_string);
+}
+#endif
+
+static void
+render_dom_node_source(struct dom_stack *stack, struct dom_node *node, void *data)
+{
+	struct dom_renderer *renderer = stack->current->data;
+
+	assert(node && renderer && renderer->document);
+
+#ifdef HAVE_REGEX_H
+	if (renderer->find_url
+	    && (node->type == DOM_NODE_TEXT
+		|| node->type == DOM_NODE_CDATA_SECTION
+		|| node->type == DOM_NODE_COMMENT)) {
+		render_dom_node_enhanced_text(renderer, node);
+		return;
+	}
+#endif
+
+	render_dom_node_text(renderer, &renderer->styles[node->type], node);
 }
 
-static struct dom_node *
-render_dom_element_source(struct dom_navigator *navigator, struct dom_node *node, void *data)
+/* This callback is also used for rendering processing instruction nodes.  */
+static void
+render_dom_element_source(struct dom_stack *stack, struct dom_node *node, void *data)
 {
-	struct dom_renderer *renderer = navigator->data;
+	struct dom_renderer *renderer = stack->current->data;
 
 	assert(node && renderer && renderer->document);
 
 	render_dom_node_text(renderer, &renderer->styles[node->type], node);
-
-	return node;
 }
 
-static struct dom_node *
-render_dom_attribute_source(struct dom_navigator *navigator, struct dom_node *node, void *data)
+static void
+render_dom_element_end_source(struct dom_stack *stack, struct dom_node *node, void *data)
 {
-	struct dom_navigator_state *state = get_dom_navigator_parent(navigator);
-	struct dom_renderer *renderer = navigator->data;
+	struct dom_renderer *renderer = stack->current->data;
+	struct dom_stack_state *state = get_dom_stack_top(stack);
+	struct sgml_parser_state *pstate = get_dom_stack_state_data(stack->contexts[0], state);
+	struct dom_scanner_token *token = &pstate->end_token;
+	unsigned char *string = token->string.string;
+	int length = token->string.length;
+
+	assert(node && renderer && renderer->document);
+
+	if (!string || !length)
+		return;
+
+	if (check_dom_node_source(renderer, string, length)) {
+		render_dom_flush(renderer, string);
+		renderer->position = string + length;
+		assert_source(renderer, renderer->position, 0);
+	}
+
+	render_dom_text(renderer, &renderer->styles[node->type], string, length);
+}
+
+static void
+set_base_uri(struct dom_renderer *renderer, unsigned char *value, size_t valuelen)
+{
+	unsigned char *href = memacpy(value, valuelen);
+	unsigned char *uristring;
+	struct uri *uri;
+
+	if (!href) return;
+	uristring = join_urls(renderer->base_uri, href);
+	mem_free(href);
+
+	if (!uristring) return;
+	uri = get_uri(uristring, 0);
+	mem_free(uristring);
+
+	if (!uri) return;
+
+	done_uri(renderer->base_uri);
+	renderer->base_uri = uri;
+}
+
+static void
+render_dom_attribute_source(struct dom_stack *stack, struct dom_node *node, void *data)
+{
+	struct dom_renderer *renderer = stack->current->data;
 	struct screen_char *template = &renderer->styles[node->type];
-	struct dom_node *attribute = NULL;
-	int i;
 
 	assert(node && renderer->document);
-	assert(state && state->list);
-
-	/* The attributes are sorted but we want them in the original order */
-	foreach_dom_node(i, node, state->list) {
-		if (node->string >= renderer->position
-		    && (!attribute || node->string < attribute->string))
-			attribute = node;
-	}
-
-	assert(attribute);
-	node = attribute;
 
 	render_dom_node_text(renderer, template, node);
 
-	if (node->data.attribute.value) {
+	if (is_dom_string_set(&node->data.attribute.value)) {
 		int quoted = node->data.attribute.quoted == 1;
-		unsigned char *value = node->data.attribute.value - quoted;
-		int valuelen = node->data.attribute.valuelen + quoted * 2;
+		unsigned char *value = node->data.attribute.value.string - quoted;
+		int valuelen = node->data.attribute.value.length + quoted * 2;
 
 		if (check_dom_node_source(renderer, value, 0)) {
 			render_dom_flush(renderer, value);
@@ -655,6 +604,12 @@ render_dom_attribute_source(struct dom_navigator *navigator, struct dom_node *no
 				break;
 			}
 
+			if (renderer->doctype == SGML_DOCTYPE_HTML
+			    && node->data.attribute.type == HTML_ATTRIBUTE_HREF
+			    && node->parent->data.element.type == HTML_ELEMENT_BASE) {
+				set_base_uri(renderer, value, valuelen - skips);
+			}
+
 			add_dom_link(renderer, value, valuelen - skips);
 
 			if (skips > 0) {
@@ -665,24 +620,74 @@ render_dom_attribute_source(struct dom_navigator *navigator, struct dom_node *no
 			render_dom_text(renderer, template, value, valuelen);
 		}
 	}
-
-	return node;
 }
 
-static dom_navigator_callback_T dom_source_renderer_callbacks[DOM_NODES] = {
-	/*				*/ NULL,
-	/* DOM_NODE_ELEMENT		*/ render_dom_element_source,
-	/* DOM_NODE_ATTRIBUTE		*/ render_dom_attribute_source,
-	/* DOM_NODE_TEXT		*/ render_dom_node_source,
-	/* DOM_NODE_CDATA_SECTION	*/ render_dom_node_source,
-	/* DOM_NODE_ENTITY_REFERENCE	*/ render_dom_node_source,
-	/* DOM_NODE_ENTITY		*/ render_dom_node_source,
-	/* DOM_NODE_PROC_INSTRUCTION	*/ render_dom_proc_instr_source,
-	/* DOM_NODE_COMMENT		*/ render_dom_node_source,
-	/* DOM_NODE_DOCUMENT		*/ NULL,
-	/* DOM_NODE_DOCUMENT_TYPE	*/ render_dom_node_source,
-	/* DOM_NODE_DOCUMENT_FRAGMENT	*/ render_dom_node_source,
-	/* DOM_NODE_NOTATION		*/ render_dom_node_source,
+static void
+render_dom_cdata_source(struct dom_stack *stack, struct dom_node *node, void *data)
+{
+	struct dom_renderer *renderer = stack->current->data;
+	unsigned char *string = node->string.string;
+
+	assert(node && renderer && renderer->document);
+
+	/* Highlight the 'CDATA' part of <![CDATA[ if it is there. */
+	if (check_dom_node_source(renderer, string - 6, 6)) {
+		render_dom_flush(renderer, string - 6);
+		render_dom_text(renderer, &renderer->styles[DOM_NODE_ATTRIBUTE], string - 6, 5);
+		renderer->position = string - 1;
+		assert_source(renderer, renderer->position, 0);
+	}
+
+	render_dom_node_text(renderer, &renderer->styles[node->type], node);
+}
+
+static void
+render_dom_document_end(struct dom_stack *stack, struct dom_node *node, void *data)
+{
+	struct dom_renderer *renderer = stack->current->data;
+
+	/* If there are no non-element nodes after the last element node make
+	 * sure that we flush to the end of the cache entry source including
+	 * the '>' of the last element tag if it has one. (bug 519) */
+	if (check_dom_node_source(renderer, renderer->position, 0)) {
+		render_dom_flush(renderer, renderer->end);
+	}
+}
+
+static struct dom_stack_context_info dom_source_renderer_context_info = {
+	/* Object size: */			0,
+	/* Push: */
+	{
+		/*				*/ NULL,
+		/* DOM_NODE_ELEMENT		*/ render_dom_element_source,
+		/* DOM_NODE_ATTRIBUTE		*/ render_dom_attribute_source,
+		/* DOM_NODE_TEXT		*/ render_dom_node_source,
+		/* DOM_NODE_CDATA_SECTION	*/ render_dom_cdata_source,
+		/* DOM_NODE_ENTITY_REFERENCE	*/ render_dom_node_source,
+		/* DOM_NODE_ENTITY		*/ render_dom_node_source,
+		/* DOM_NODE_PROC_INSTRUCTION	*/ render_dom_element_source,
+		/* DOM_NODE_COMMENT		*/ render_dom_node_source,
+		/* DOM_NODE_DOCUMENT		*/ NULL,
+		/* DOM_NODE_DOCUMENT_TYPE	*/ render_dom_node_source,
+		/* DOM_NODE_DOCUMENT_FRAGMENT	*/ render_dom_node_source,
+		/* DOM_NODE_NOTATION		*/ render_dom_node_source,
+	},
+	/* Pop: */
+	{
+		/*				*/ NULL,
+		/* DOM_NODE_ELEMENT		*/ render_dom_element_end_source,
+		/* DOM_NODE_ATTRIBUTE		*/ NULL,
+		/* DOM_NODE_TEXT		*/ NULL,
+		/* DOM_NODE_CDATA_SECTION	*/ NULL,
+		/* DOM_NODE_ENTITY_REFERENCE	*/ NULL,
+		/* DOM_NODE_ENTITY		*/ NULL,
+		/* DOM_NODE_PROC_INSTRUCTION	*/ NULL,
+		/* DOM_NODE_COMMENT		*/ NULL,
+		/* DOM_NODE_DOCUMENT		*/ render_dom_document_end,
+		/* DOM_NODE_DOCUMENT_TYPE	*/ NULL,
+		/* DOM_NODE_DOCUMENT_FRAGMENT	*/ NULL,
+		/* DOM_NODE_NOTATION		*/ NULL,
+	}
 };
 
 
@@ -692,14 +697,16 @@ render_dom_document(struct cache_entry *cached, struct document *document,
 		    struct string *buffer)
 {
 	unsigned char *head = empty_string_or_(cached->head);
-	struct dom_node *root = parse_sgml(cached, document, buffer);
+	struct dom_node *root;
 	struct dom_renderer renderer;
 	struct conv_table *convert_table;
-	dom_navigator_callback_T *callbacks = dom_source_renderer_callbacks;
-	struct dom_navigator navigator;
+	struct sgml_parser *parser;
+	unsigned char *string = struri(cached->uri);
+	size_t length = strlen(string);
+	struct dom_string uri = INIT_DOM_STRING(string, length);
+	struct dom_string source = INIT_DOM_STRING(buffer->source, buffer->length);
 
 	assert(document->options.plain);
-	if (!root) return;
 
 	convert_table = get_convert_table(head, document->options.cp,
 					  document->options.assume_cp,
@@ -707,19 +714,46 @@ render_dom_document(struct cache_entry *cached, struct document *document,
 					  &document->cp_status,
 					  document->options.hard_assume);
 
-	init_dom_renderer(&renderer, document, buffer, root, convert_table);
-	init_dom_navigator(&navigator, &renderer, callbacks, 0);
+	init_dom_renderer(&renderer, document, buffer, convert_table);
 
-	document->bgcolor = global_doc_opts->default_bg;
+	document->bgcolor = document->options.default_bg;
 
-	walk_dom_nodes(&navigator, root);
-	/* If there are no non-element nodes after the last element node make
-	 * sure that we flush to the end of the cache entry source including
-	 * the '>' of the last element tag if it has one. (bug 519) */
-	if (check_dom_node_source(&renderer, renderer.position, 0)) {
-		render_dom_flush(&renderer, renderer.end);
+	if (!strcasecmp("application/rss+xml", cached->content_type)) {
+		renderer.doctype = SGML_DOCTYPE_RSS;
+
+	} else if (!strcasecmp("application/xbel+xml", cached->content_type)
+		   || !strcasecmp("application/x-xbel", cached->content_type)
+		   || !strcasecmp("application/xbel", cached->content_type)) {
+		renderer.doctype = SGML_DOCTYPE_XBEL;
+
+	} else {
+		assertm(!strcasecmp("text/html", cached->content_type)
+			|| !strcasecmp("application/xhtml+xml", cached->content_type),
+			"Couldn't resolve doctype '%s'", cached->content_type);
+
+		renderer.doctype = SGML_DOCTYPE_HTML;
 	}
 
-	done_dom_node(root);
-	done_dom_navigator(&navigator);
+	parser = init_sgml_parser(SGML_PARSER_STREAM, renderer.doctype, &uri);
+	if (!parser) return;
+
+	add_dom_stack_context(&parser->stack, &renderer,
+			      &dom_source_renderer_context_info);
+
+	root = parse_sgml(parser, &source);
+	if (root) {
+		assert(parser->stack.depth == 1);
+
+		get_dom_stack_top(&parser->stack)->immutable = 0;
+		/* For SGML_PARSER_STREAM this will free the DOM
+		 * root node. */
+		pop_dom_node(&parser->stack);
+	}
+
+#ifdef HAVE_REGEX_H
+	if (renderer.find_url)
+		regfree(&renderer.url_regex);
+#endif
+	done_uri(renderer.base_uri);
+	done_sgml_parser(parser);
 }
