@@ -1,5 +1,4 @@
 /* Global history dialogs */
-/* $Id: dialogs.c,v 1.126.4.1 2005/04/06 09:11:19 jonas Exp $ */
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
@@ -15,11 +14,11 @@
 #include "globhist/dialogs.h"
 #include "globhist/globhist.h"
 #include "intl/gettext/libintl.h"
+#include "main/object.h"
 #include "protocol/uri.h"
 #include "terminal/terminal.h"
 #include "util/memory.h"
 #include "util/string.h"
-#include "util/object.h"
 
 
 /* Implementation of the listbox operations */
@@ -172,7 +171,7 @@ launch_search_dialog(struct terminal *term, struct dialog_data *parent,
 		       NULL, NULL, EDIT_DLG_SEARCH);
 }
 
-static t_handler_event_status
+static widget_handler_status_T
 push_search_button(struct dialog_data *dlg_data, struct widget_data *widget_data)
 {
 	launch_search_dialog(dlg_data->win->term, dlg_data,
@@ -182,7 +181,7 @@ push_search_button(struct dialog_data *dlg_data, struct widget_data *widget_data
 
 /* Toggling: */
 
-static t_handler_event_status
+static widget_handler_status_T
 push_toggle_display_button(struct dialog_data *dlg_data, struct widget_data *widget_data)
 {
 	int *display_type;
@@ -198,7 +197,7 @@ push_toggle_display_button(struct dialog_data *dlg_data, struct widget_data *wid
 /* Bookmarking: */
 
 #ifdef CONFIG_BOOKMARKS
-static t_handler_event_status
+static widget_handler_status_T
 push_bookmark_button(struct dialog_data *dlg_data,
 		     struct widget_data *some_useless_info_button)
 {
