@@ -1,5 +1,4 @@
 /* Charsets convertor */
-/* $Id: charsets.c,v 1.108.4.2 2005/09/14 12:07:14 jonas Exp $ */
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
@@ -21,7 +20,6 @@
 #include "util/fastfind.h"
 #include "util/memory.h"
 #include "util/string.h"
-#include "util/types.h"
 
 
 /* Fix namespace clash on MacOS. */
@@ -29,7 +27,7 @@
 
 struct table_entry {
 	unsigned char c;
-	unicode_val u;
+	unicode_val_T u;
 };
 
 struct codepage_desc {
@@ -38,9 +36,9 @@ struct codepage_desc {
 	struct table_entry *table;
 };
 
-#include "codepage.inc"
-#include "uni_7b.inc"
-#include "entity.inc"
+#include "intl/codepage.inc"
+#include "intl/uni_7b.inc"
+#include "intl/entity.inc"
 
 
 static char strings[256][2] = {
@@ -126,7 +124,7 @@ new_translation_table(struct conv_table *p)
 	}										\
 }											\
 
-static const unicode_val strange_chars[32] = {
+static const unicode_val_T strange_chars[32] = {
 0x20ac, 0x0000, 0x002a, 0x0000, 0x201e, 0x2026, 0x2020, 0x2021,
 0x005e, 0x2030, 0x0160, 0x003c, 0x0152, 0x0000, 0x0000, 0x0000,
 0x0000, 0x0060, 0x0027, 0x0022, 0x0022, 0x002a, 0x2013, 0x2014,
@@ -136,7 +134,7 @@ static const unicode_val strange_chars[32] = {
 #define SYSTEM_CHARSET_FLAG 128
 
 unsigned char *
-u2cp_(unicode_val u, int to, int no_nbsp_hack)
+u2cp_(unicode_val_T u, int to, int no_nbsp_hack)
 {
 	int j;
 	int s;
@@ -147,7 +145,7 @@ u2cp_(unicode_val u, int to, int no_nbsp_hack)
 	if (u == 0xad) return "";
 
 	if (u < 0xa0) {
-		unicode_val strange = strange_chars[u - 0x80];
+		unicode_val_T strange = strange_chars[u - 0x80];
 
 		if (!strange) return NULL;
 		return u2cp_(strange, to, no_nbsp_hack);
@@ -168,7 +166,7 @@ u2cp_(unicode_val u, int to, int no_nbsp_hack)
 static unsigned char utf_buffer[7];
 
 static unsigned char *
-encode_utf_8(unicode_val u)
+encode_utf_8(unicode_val_T u)
 {
 	memset(utf_buffer, 0, 7);
 
@@ -221,7 +219,7 @@ cp2utf_8(int from, int c)
 }
 
 static void
-add_utf_8(struct conv_table *ct, unicode_val u, unsigned char *str)
+add_utf_8(struct conv_table *ct, unicode_val_T u, unsigned char *str)
 {
 	unsigned char *p = encode_utf_8(u);
 
@@ -290,7 +288,7 @@ get_translation_table_to_utf_8(int from)
 		utf_table[i].u.str = NULL;
 
 	for (i = 0; codepages[from].table[i].c; i++) {
-		unicode_val u = codepages[from].table[i].u;
+		unicode_val_T u = codepages[from].table[i].u;
 
 		if (!utf_table[codepages[from].table[i].c].u.str)
 			utf_table[codepages[from].table[i].c].u.str =
@@ -494,7 +492,7 @@ get_entity_string(const unsigned char *str, const int strlen, int encoding)
 	if (*str == '#') { /* Numeric entity. */
 		int l = (int) strlen;
 		unsigned char *st = (unsigned char *) str;
-		unicode_val n = 0;
+		unicode_val_T n = 0;
 
 		if (l == 1) goto end; /* &#; ? */
 		st++, l--;
@@ -522,7 +520,7 @@ get_entity_string(const unsigned char *str, const int strlen, int encoding)
 				else
 					goto end; /* Bad char. */
 				/* Limit to 0xFFFFFFFF. */
-				if (n == (unicode_val) 0xFFFFFFFFu)
+				if (n >= (unicode_val_T) 0xFFFFFFFFu)
 					goto end;
 			} while (--l);
 		}
@@ -559,7 +557,7 @@ end:
 		if (nb_entity_cache[slen] < ENTITY_CACHE_SIZE) nb_entity_cache[slen]++;
 
 #ifdef DEBUG_ENTITY_CACHE
-		fprintf(stderr, "Added in [%d]: l=%d st='%s'\n", slen,
+		fprintf(stderr, "Added in [%u]: l=%d st='%s'\n", slen,
 				entity_cache[slen][0].strlen, entity_cache[slen][0].str);
 
 #endif
@@ -573,9 +571,9 @@ end:
 	{
 		unsigned int i;
 
-		fprintf(stderr, "- Cache entries [%d] -\n", slen);
+		fprintf(stderr, "- Cache entries [%u] -\n", slen);
 		for (i = 0; i < nb_entity_cache[slen] ; i++)
-			fprintf(stderr, "%d: hits=%d l=%d st='%s'\n", i,
+			fprintf(stderr, "%d: hits=%u l=%d st='%s'\n", i,
 				entity_cache[slen][i].hits, entity_cache[slen][i].strlen,
 				entity_cache[slen][i].str);
 		fprintf(stderr, "-----------------\n");
@@ -587,7 +585,7 @@ end:
 
 unsigned char *
 convert_string(struct conv_table *convert_table,
-	       unsigned char *chars, int charslen,
+	       unsigned char *chars, int charslen, int cp,
 	       enum convert_string_mode mode, int *length,
 	       void (*callback)(void *data, unsigned char *buf, int buflen),
 	       void *callback_data)
@@ -659,7 +657,7 @@ convert_string(struct conv_table *convert_table,
 			    && i > start
 			    && !isasciialpha(chars[i]) && !isdigit(chars[i])) {
 				translit = get_entity_string(&chars[start], i - start,
-						      global_doc_opts->cp);
+						      cp);
 				if (chars[i] != ';') {
 					/* Eat &nbsp &nbsp<foo> happily, but
 					 * pull back from the character after
