@@ -1,5 +1,4 @@
 /* MIME handling backends multiplexing */
-/* $Id: common.c,v 1.21.4.1 2005/04/05 21:08:42 jonas Exp $ */
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
@@ -11,9 +10,9 @@
 
 #include "elinks.h"
 
+#include "main/module.h"
 #include "mime/backend/common.h"
 #include "mime/mime.h"
-#include "modules/module.h"
 #include "util/file.h"
 #include "util/memory.h"
 #include "util/string.h"
