@@ -1,10 +1,9 @@
-/* $Id: mailcap.h,v 1.5 2004/07/15 00:43:06 jonas Exp $ */
 
 #ifndef EL__MIME_BACKEND_MAILCAP_H
 #define EL__MIME_BACKEND_MAILCAP_H
 
+#include "main/module.h"
 #include "mime/backend/common.h"
-#include "modules/module.h"
 
 extern struct mime_backend mailcap_mime_backend;
 extern struct module mailcap_mime_module;
