@@ -1,5 +1,3 @@
-/* $Id: generic.h,v 1.19.6.7 2005/09/14 11:36:12 jonas Exp $ */
-
 /* This is... er, the OS-independent part of osdep/ ;-). */
 
 #ifndef EL__OSDEP_GENERIC_H
@@ -17,24 +15,6 @@
 #include <stddef.h> /* may contain offsetof() */
 #endif
 
-#ifndef INT_MAX
-#ifdef MAXINT
-#define INT_MAX MAXINT
-#else
-/* XXX: We could use util/types.h to determine something useful? --pasky */
-#define INT_MAX 0x7fffffff
-#endif
-#endif
-
-#ifndef LONG_MAX
-#ifdef MAXLONG
-#define LONG_MAX MAXLONG
-#else
-/* XXX: We could use util/types.h to determine something useful? --pasky */
-#define LONG_MAX 0x7fffffff
-#endif
-#endif
-
 #ifndef SA_RESTART
 #define SA_RESTART	0
 #endif
@@ -73,7 +53,7 @@
 static inline ssize_t
 safe_read(int fd, void *buf, size_t count) {
 	do {
-		int r = read(fd, buf, count);
+		ssize_t r = read(fd, buf, count);
 
 		if (r == -1 && errno == EINTR) continue;
 		return r;
@@ -83,7 +63,7 @@ safe_read(int fd, void *buf, size_t count) {
 static inline ssize_t
 safe_write(int fd, const void *buf, size_t count) {
 	do {
-		int w = write(fd, buf, count);
+		ssize_t w = write(fd, buf, count);
 
 		if (w == -1 && errno == EINTR) continue;
 		return w;
@@ -98,6 +78,13 @@ safe_write(int fd, const void *buf, size_t count) {
 
 #endif /* EINTR && !CONFIG_WIN32 */
 
+#ifndef HAVE_FTELLO
+#define ftello(stream) ftell(stream)
+#endif
+
+#ifndef HAVE_FSEEKO
+#define fseeko(stream, offset, whence) fseek(stream, offset, whence)
+#endif
 
 /* Compiler area: */
 
@@ -116,4 +103,6 @@ safe_write(int fd, const void *buf, size_t count) {
 #define copy_struct(destination, source) \
 	do { (*(destination) = *(source)); } while (0)
 
+#define sizeof_array(array) (sizeof(array)/sizeof(*(array)))
+
 #endif
