@@ -1,4 +1,3 @@
-/* $Id: overrides.c,v 1.3.2.1 2005/02/28 01:20:29 jonas Exp $ */
 
 /* Elinks overrides for Win32 (MSVC + MingW)
  * Loosely based on osdep/beos/overrides.c by Mikulas Patocka.
@@ -119,10 +118,10 @@ console_mouse_read(const MOUSE_EVENT_RECORD *mer, char *buf, int max)
 	if (mer->dwEventFlags || !mer->dwButtonState)  /* not mouse down, or shifts */
 		return 0;
 
-	ev.ev = EVENT_MOUSE;
-	ev.info.mouse.x = 1 + mer->dwMousePosition.X;
-	ev.info.mouse.y = 1 + mer->dwMousePosition.Y;
-	ev.info.mouse.button = B_DOWN;
+	set_mouse_term_event(&ev,
+			     1 + mer->dwMousePosition.X,
+			     1 + mer->dwMousePosition.Y,
+			     B_DOWN);
 
 	len = int_min(max, sizeof(ev));
 	memcpy(buf, &ev, len);
@@ -348,9 +347,9 @@ win32_ioctl(int fd, long option, int *flag)
 }
 
 int
-win32_socket(int family, int type, int protocol)
+win32_socket(int pf, int type, int protocol)
 {
-	SOCKET s = socket(family, type, protocol);
+	SOCKET s = socket(pf, type, protocol);
 	int    rc;
 
 	if (s == INVALID_SOCKET) {
@@ -360,7 +359,7 @@ win32_socket(int family, int type, int protocol)
 		rc = s + SOCK_SHIFT;
 	}
 
-	TRACE("family %d, type %d, proto %d -> rc %u", family, type, protocol, rc);
+	TRACE("family %d, type %d, proto %d -> rc %d", pf, type, protocol, rc);
 
 	return rc;
 }
