@@ -1,5 +1,4 @@
 /* Internal "ftp" protocol implementation */
-/* $Id: ftp.c,v 1.182.2.8 2005/09/14 11:36:12 jonas Exp $ */
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
@@ -35,16 +34,17 @@
 #include "cache/cache.h"
 #include "config/options.h"
 #include "intl/gettext/libintl.h"
-#include "lowlevel/connect.h"
-#include "lowlevel/select.h"
-#include "modules/module.h"
+#include "main/select.h"
+#include "main/module.h"
+#include "network/connection.h"
+#include "network/progress.h"
+#include "network/socket.h"
 #include "osdep/osdep.h"
 #include "osdep/stat.h"
 #include "protocol/auth/auth.h"
 #include "protocol/ftp/ftp.h"
 #include "protocol/ftp/parse.h"
 #include "protocol/uri.h"
-#include "sched/connection.h"
 #include "util/conv.h"
 #include "util/error.h"
 #include "util/memory.h"
@@ -107,7 +107,6 @@ struct ftp_connection_info {
 
 	unsigned int dir:1;          /* Directory listing in progress */
 	unsigned int rest_sent:1;    /* Sent RESTore command */
-	unsigned int has_data:1;     /* Do we have data socket? */
 	unsigned int use_pasv:1;     /* Use PASV (yes or no) */
 #ifdef CONFIG_IPV6
 	unsigned int use_epsv:1;     /* Use EPSV */
@@ -118,17 +117,18 @@ struct ftp_connection_info {
 
 
 /* Prototypes */
-static void ftp_login(struct connection *);
+static void ftp_login(struct socket *);
 static void ftp_send_retr_req(struct connection *, int);
-static void ftp_got_info(struct connection *, struct read_buffer *);
-static void ftp_got_user_info(struct connection *, struct read_buffer *);
+static void ftp_got_info(struct socket *, struct read_buffer *);
+static void ftp_got_user_info(struct socket *, struct read_buffer *);
 static void ftp_pass(struct connection *);
-static void ftp_pass_info(struct connection *, struct read_buffer *);
-static void ftp_retr_file(struct connection *, struct read_buffer *);
-static void ftp_got_final_response(struct connection *, struct read_buffer *);
+static void ftp_pass_info(struct socket *, struct read_buffer *);
+static void ftp_retr_file(struct socket *, struct read_buffer *);
+static void ftp_got_final_response(struct socket *, struct read_buffer *);
 static void got_something_from_data_connection(struct connection *);
-static void ftp_end_request(struct connection *, int);
+static void ftp_end_request(struct connection *, enum connection_state);
 static struct ftp_connection_info *add_file_cmd_to_str(struct connection *);
+static void ftp_data_accept(struct connection *conn);
 
 /* Parse EPSV or PASV response for address and/or port.
  * int *n should point to a sizeof(int) * 6 space.
@@ -188,86 +188,81 @@ static int
 get_ftp_response(struct connection *conn, struct read_buffer *rb, int part,
 		 struct sockaddr_storage *sa)
 {
+	unsigned char *eol;
+	unsigned char *num_end;
+	int response;
 	int pos;
 
-	set_connection_timeout(conn);
-
 again:
-	for (pos = 0; pos < rb->len; pos++) {
-		unsigned char *num_end;
-		int response;
+	eol = memchr(rb->data, ASCII_LF, rb->length);
+	if (!eol) return 0;
 
-		if (rb->data[pos] != ASCII_LF) continue;
+	pos = eol - rb->data;
 
-		errno = 0;
-		response = strtoul(rb->data, (char **) &num_end, 10);
+	errno = 0;
+	response = strtoul(rb->data, (char **) &num_end, 10);
 
-		if (errno || num_end != rb->data + 3 || response < 100)
+	if (errno || num_end != rb->data + 3 || response < 100)
+		return -1;
+
+	if (sa && response == 227) { /* PASV response parsing. */
+		struct sockaddr_in *s = (struct sockaddr_in *) sa;
+		int n[6];
+
+		if (parse_psv_resp(num_end, (int *) &n, 255) != 6)
 			return -1;
 
-		if (sa && response == 227) { /* PASV response parsing. */
-			struct sockaddr_in *s = (struct sockaddr_in *) sa;
-			int n[6];
-
-			if (parse_psv_resp(num_end, (int *) &n, 255) != 6)
-				return -1;
-
-			memset(s, 0, sizeof(*s));
-			s->sin_family = AF_INET;
-			s->sin_addr.s_addr = htonl((n[0] << 24) + (n[1] << 16) + (n[2] << 8) + n[3]);
-			s->sin_port = htons((n[4] << 8) + n[5]);
-		}
-
-#ifdef CONFIG_IPV6
-		if (sa && response == 229) { /* EPSV response parsing. */
-			/* See RFC 2428 */
-			struct sockaddr_in6 *s = (struct sockaddr_in6 *) sa;
-			int sal = sizeof(*s);
-			int n[6];
-
-			if (parse_psv_resp(num_end, (int *) &n, 65535) != 1) {
-				return -1;
-			}
-
-			memset(s, 0, sizeof(*s));
-			if (getpeername(conn->socket.fd, (struct sockaddr *) sa, &sal)) {
-				return -1;
-			}
-			s->sin6_family = AF_INET6;
-			s->sin6_port = htons(n[5]);
-		}
-#endif
-
-		if (*num_end == '-') {
-			int i;
-
-			for (i = 0; i < rb->len - 5; i++)
-				if (rb->data[i] == ASCII_LF
-				    && !memcmp(rb->data+i+1, rb->data, 3)
-				    && rb->data[i+4] == ' ') {
-					for (i++; i < rb->len; i++)
-						if (rb->data[i] == ASCII_LF)
-							goto ok;
-					return 0;
-				}
-			return 0;
-ok:
-			pos = i;
-		}
-
-		if (!part && response >= 100 && response < 200) {
-			kill_buffer_data(rb, pos + 1);
-			goto again;
-		}
-
-		if (part == 2)
-			return response;
-
-		kill_buffer_data(rb, pos + 1);
-		return response;
+		memset(s, 0, sizeof(*s));
+		s->sin_family = AF_INET;
+		s->sin_addr.s_addr = htonl((n[0] << 24) + (n[1] << 16) + (n[2] << 8) + n[3]);
+		s->sin_port = htons((n[4] << 8) + n[5]);
 	}
 
-	return 0;
+#ifdef CONFIG_IPV6
+	if (sa && response == 229) { /* EPSV response parsing. */
+		/* See RFC 2428 */
+		struct sockaddr_in6 *s = (struct sockaddr_in6 *) sa;
+		int sal = sizeof(*s);
+		int n[6];
+
+		if (parse_psv_resp(num_end, (int *) &n, 65535) != 1) {
+			return -1;
+		}
+
+		memset(s, 0, sizeof(*s));
+		if (getpeername(conn->socket->fd, (struct sockaddr *) sa, &sal)) {
+			return -1;
+		}
+		s->sin6_family = AF_INET6;
+		s->sin6_port = htons(n[5]);
+	}
+#endif
+
+	if (*num_end == '-') {
+		int i;
+
+		for (i = 0; i < rb->length - 5; i++)
+			if (rb->data[i] == ASCII_LF
+			    && !memcmp(rb->data+i+1, rb->data, 3)
+			    && rb->data[i+4] == ' ') {
+				for (i++; i < rb->length; i++)
+					if (rb->data[i] == ASCII_LF)
+						goto ok;
+				return 0;
+			}
+		return 0;
+ok:
+		pos = i;
+	}
+
+	if (part != 2)
+		kill_buffer_data(rb, pos + 1);
+
+	if (!part && response >= 100 && response < 200) {
+		goto again;
+	}
+
+	return response;
 }
 
 
@@ -275,35 +270,23 @@ ok:
 void
 ftp_protocol_handler(struct connection *conn)
 {
-	set_connection_timeout(conn);
-
 	if (!has_keepalive_connection(conn)) {
-		make_connection(conn, &conn->socket, ftp_login);
+		make_connection(conn->socket, conn->uri, ftp_login,
+				conn->cache_mode >= CACHE_MODE_FORCE_RELOAD);
 
 	} else {
 		ftp_send_retr_req(conn, S_SENT);
 	}
 }
 
-/* Get connection response. */
-static void
-get_resp(struct connection *conn)
-{
-	struct read_buffer *rb = alloc_read_buffer(conn);
-
-	if (!rb) return;
-	read_from_socket(conn, &conn->socket, rb, conn->read_func);
-}
-
 /* Send command, set connection state and free cmd string. */
 static void
 send_cmd(struct connection *conn, struct string *cmd, void *callback, int state)
 {
-	conn->read_func = callback;
-	write_to_socket(conn, &conn->socket, cmd->source, cmd->length, get_resp);
+	request_from_socket(conn->socket, cmd->source, cmd->length, state,
+			    SOCKET_RETRY_ONCLOSE, callback);
 
 	done_string(cmd);
-	set_connection_state(conn, state);
 }
 
 /* Check if this auth token really belongs to this URI. */
@@ -324,33 +307,34 @@ prompt_username_pw(struct connection *conn)
 	if (!conn->cached) {
 		conn->cached = get_cache_entry(conn->uri);
 		if (!conn->cached) {
-			abort_conn_with_state(conn, S_OUT_OF_MEM);
+			abort_connection(conn, S_OUT_OF_MEM);
 			return;
 		}
 	}
 
 	mem_free_set(&conn->cached->content_type, stracpy("text/html"));
 	if (!conn->cached->content_type) {
-		abort_conn_with_state(conn, S_OUT_OF_MEM);
+		abort_connection(conn, S_OUT_OF_MEM);
 		return;
 	}
 
 	add_auth_entry(conn->uri, "FTP Login", NULL, NULL, 0);
 
-	abort_conn_with_state(conn, S_OK);
+	abort_connection(conn, S_OK);
 }
 
 /* Send USER command. */
 static void
-ftp_login(struct connection *conn)
+ftp_login(struct socket *socket)
 {
+	struct connection *conn = socket->conn;
 	struct string cmd;
 	struct auth_entry* auth;
 
 	auth = find_auth(conn->uri);
 
 	if (!init_string(&cmd)) {
-		abort_conn_with_state(conn, S_OUT_OF_MEM);
+		abort_connection(conn, S_OUT_OF_MEM);
 		return;
 	}
 
@@ -373,17 +357,18 @@ ftp_login(struct connection *conn)
 
 /* Parse connection response. */
 static void
-ftp_got_info(struct connection *conn, struct read_buffer *rb)
+ftp_got_info(struct socket *socket, struct read_buffer *rb)
 {
+	struct connection *conn = socket->conn;
 	int response = get_ftp_response(conn, rb, 0, NULL);
 
 	if (response == -1) {
-		abort_conn_with_state(conn, S_FTP_ERROR);
+		abort_connection(conn, S_FTP_ERROR);
 		return;
 	}
 
 	if (!response) {
-		read_from_socket(conn, &conn->socket, rb, ftp_got_info);
+		read_from_socket(conn->socket, rb, conn->state, ftp_got_info);
 		return;
 	}
 
@@ -394,27 +379,28 @@ ftp_got_info(struct connection *conn, struct read_buffer *rb)
 
 	if (response != 220) {
 		/* TODO? Retry in case of ... ?? */
-		retry_conn_with_state(conn, S_FTP_UNAVAIL);
+		retry_connection(conn, S_FTP_UNAVAIL);
 		return;
 	}
 
-	ftp_got_user_info(conn, rb);
+	ftp_got_user_info(socket, rb);
 }
 
 
 /* Parse USER response and send PASS command if needed. */
 static void
-ftp_got_user_info(struct connection *conn, struct read_buffer *rb)
+ftp_got_user_info(struct socket *socket, struct read_buffer *rb)
 {
+	struct connection *conn = socket->conn;
 	int response = get_ftp_response(conn, rb, 0, NULL);
 
 	if (response == -1) {
-		abort_conn_with_state(conn, S_FTP_ERROR);
+		abort_connection(conn, S_FTP_ERROR);
 		return;
 	}
 
 	if (!response) {
-		read_from_socket(conn, &conn->socket, rb, ftp_got_user_info);
+		read_from_socket(conn->socket, rb, conn->state, ftp_got_user_info);
 		return;
 	}
 
@@ -439,7 +425,7 @@ ftp_got_user_info(struct connection *conn, struct read_buffer *rb)
 	 * non-RFC compliant servers may return even something other than 421.
 	 * --Zas */
 	if (response >= 400) {
-		abort_conn_with_state(conn, S_FTP_UNAVAIL);
+		abort_connection(conn, S_FTP_UNAVAIL);
 		return;
 	}
 
@@ -461,7 +447,7 @@ ftp_pass(struct connection *conn)
 	auth = find_auth(conn->uri);
 
 	if (!init_string(&cmd)) {
-		abort_conn_with_state(conn, S_OUT_OF_MEM);
+		abort_connection(conn, S_OUT_OF_MEM);
 		return;
 	}
 
@@ -488,18 +474,18 @@ ftp_pass(struct connection *conn)
 
 /* Parse PASS command response. */
 static void
-ftp_pass_info(struct connection *conn, struct read_buffer *rb)
+ftp_pass_info(struct socket *socket, struct read_buffer *rb)
 {
+	struct connection *conn = socket->conn;
 	int response = get_ftp_response(conn, rb, 0, NULL);
 
 	if (response == -1) {
-		abort_conn_with_state(conn, S_FTP_ERROR);
+		abort_connection(conn, S_FTP_ERROR);
 		return;
 	}
 
 	if (!response) {
-		read_from_socket(conn, &conn->socket, rb, ftp_pass_info);
-		set_connection_state(conn, S_LOGIN);
+		read_from_socket(conn->socket, rb, S_LOGIN, ftp_pass_info);
 		return;
 	}
 
@@ -521,7 +507,7 @@ ftp_pass_info(struct connection *conn, struct read_buffer *rb)
 	}
 
 	if (response >= 400) {
-		abort_conn_with_state(conn, S_FTP_UNAVAIL);
+		abort_connection(conn, S_FTP_UNAVAIL);
 		return;
 	}
 
@@ -591,64 +577,106 @@ add_eprtcmd_to_string(struct string *string, struct sockaddr_in6 *addr)
 }
 #endif
 
+/* Depending on options, get proper ftp data socket and command.
+ * It appends ftp command (either PASV,PORT,EPSV or EPRT) to @command
+ * string.
+ * When PORT or EPRT are used, related sockets are created.
+ * It returns 0 on error (data socket creation failure). */
+static int
+get_ftp_data_socket(struct connection *conn, struct string *command)
+{
+	struct ftp_connection_info *ftp = conn->info;
+
+	ftp->use_pasv = get_opt_bool("protocol.ftp.use_pasv");
+
+#ifdef CONFIG_IPV6
+	ftp->use_epsv = get_opt_bool("protocol.ftp.use_epsv");
+
+	if (conn->socket->protocol_family == EL_PF_INET6) {
+		if (ftp->use_epsv) {
+			add_to_string(command, "EPSV");
+
+		} else {
+			struct sockaddr_storage data_addr;
+			int data_sock;
+
+			memset(&data_addr, 0, sizeof(data_addr));
+			data_sock = get_pasv_socket(conn->socket, &data_addr);
+			if (data_sock < 0) return 0;
+
+			conn->data_socket->fd = data_sock;
+			add_eprtcmd_to_string(command,
+					      (struct sockaddr_in6 *) &data_addr);
+		}
+
+	} else
+#endif
+	{
+		if (ftp->use_pasv) {
+			add_to_string(command, "PASV");
+
+		} else {
+			struct sockaddr_in sa;
+			unsigned char pc[6];
+			int data_sock;
+
+			memset(pc, 0, sizeof(pc));
+			data_sock = get_pasv_socket(conn->socket,
+			 	    (struct sockaddr_storage *) &sa);
+			if (data_sock < 0) return 0;
+
+			memcpy(pc, &sa.sin_addr.s_addr, 4);
+			memcpy(pc + 4, &sa.sin_port, 2);
+			conn->data_socket->fd = data_sock;
+			add_portcmd_to_string(command, pc);
+		}
+	}
+
+	add_crlf_to_string(command);
+
+	return 1;
+}
+
+
 /* Create passive socket and add appropriate announcing commands to str. Then
  * go and retrieve appropriate object from server.
  * Returns NULL if error. */
 static struct ftp_connection_info *
 add_file_cmd_to_str(struct connection *conn)
 {
-#ifdef CONFIG_IPV6
-	struct sockaddr_storage data_addr;
-#endif
-	struct ftp_connection_info *c_i;
-	struct string command;
-	int data_sock;
-	unsigned char pc[6];
-
-	c_i = mem_calloc(1, sizeof(*c_i));
-	if (!c_i) {
-		abort_conn_with_state(conn, S_OUT_OF_MEM);
-		return NULL;
-	}
-
-	if (!init_string(&command)) {
-		mem_free(c_i);
-		abort_conn_with_state(conn, S_OUT_OF_MEM);
-		return NULL;
-	}
-
-	conn->info = c_i;
-
-#ifdef CONFIG_IPV6
-	memset(&data_addr, 0, sizeof(data_addr));
-#endif
-	memset(pc, 0, 6);
-
-	if (get_opt_bool("protocol.ftp.use_pasv"))
-		c_i->use_pasv = 1;
-
-#ifdef CONFIG_IPV6
-	if (get_opt_bool("protocol.ftp.use_epsv"))
-		c_i->use_epsv = 1;
-
-	if (!c_i->use_epsv && conn->protocol_family == 1) {
-		data_sock = get_pasv6_socket(conn, conn->socket.fd, &data_addr);
-		if (data_sock < 0)
-			return NULL;
-		conn->data_socket.fd = data_sock;
-	}
-#endif
-
-	if (!c_i->use_pasv && conn->protocol_family != 1) {
-		data_sock = get_pasv_socket(conn, conn->socket.fd, pc);
-		if (data_sock < 0)
-			return NULL;
-		conn->data_socket.fd = data_sock;
-	}
+	struct ftp_connection_info *ftp;
+	struct string command, ftp_data_command;
 
 	if (!conn->uri->data) {
 		INTERNAL("conn->uri->data empty");
-		abort_conn_with_state(conn, S_INTERNAL);
+		abort_connection(conn, S_INTERNAL);
+		return NULL;
+	}
+
+	ftp = mem_calloc(1, sizeof(*ftp));
+	if (!ftp) {
+		abort_connection(conn, S_OUT_OF_MEM);
+		return NULL;
+	}
+
+	conn->info = ftp;	/* Freed when connection is destroyed. */
+
+	if (!init_string(&command)) {
+		abort_connection(conn, S_OUT_OF_MEM);
+		return NULL;
+	}
+
+	if (!init_string(&ftp_data_command)) {
+		done_string(&command);
+		abort_connection(conn, S_OUT_OF_MEM);
+		return NULL;
+	}
+
+	if (!get_ftp_data_socket(conn, &ftp_data_command)) {
+		done_string(&command);
+		done_string(&ftp_data_command);
+		INTERNAL("Ftp data socket failure");
+		abort_connection(conn, S_INTERNAL);
 		return NULL;
 	}
 
@@ -656,28 +684,14 @@ add_file_cmd_to_str(struct connection *conn)
 	    || conn->uri->data[conn->uri->datalen - 1] == '/') {
 		/* Commands to get directory listing. */
 
-		c_i->dir = 1;
-		c_i->pending_commands = 4;
+		ftp->dir = 1;
+		ftp->pending_commands = 4;
 
 		/* ASCII */
 		add_to_string(&command, "TYPE A");
 		add_crlf_to_string(&command);
 
-#ifdef CONFIG_IPV6
-		if (conn->protocol_family == 1)
-			if (c_i->use_epsv)
-				add_to_string(&command, "EPSV");
-			else
-				add_eprtcmd_to_string(&command,
-					(struct sockaddr_in6 *) &data_addr);
-		else
-#endif
-			if (c_i->use_pasv)
-				add_to_string(&command, "PASV");
-			else
-				add_portcmd_to_string(&command, pc);
-
-		add_crlf_to_string(&command);
+		add_string_to_string(&command, &ftp_data_command);
 
 		add_to_string(&command, "CWD ");
 		add_uri_to_string(&command, conn->uri, URI_PATH);
@@ -691,38 +705,24 @@ add_file_cmd_to_str(struct connection *conn)
 	} else {
 		/* Commands to get a file. */
 
-		c_i->dir = 0;
-		c_i->pending_commands = 3;
+		ftp->dir = 0;
+		ftp->pending_commands = 3;
 
 		/* BINARY */
 		add_to_string(&command, "TYPE I");
 		add_crlf_to_string(&command);
 
-#ifdef CONFIG_IPV6
-		if (conn->protocol_family == 1)
-			if (c_i->use_epsv)
-				add_to_string(&command, "EPSV");
-			else
-				add_eprtcmd_to_string(&command,
-					(struct sockaddr_in6 *) &data_addr);
-		else
-#endif
-			if (c_i->use_pasv)
-				add_to_string(&command, "PASV");
-			else
-				add_portcmd_to_string(&command, pc);
+		add_string_to_string(&command, &ftp_data_command);
 
-		add_crlf_to_string(&command);
-
-		if (conn->from || (conn->progress.start > 0)) {
+		if (conn->from || conn->progress->start > 0) {
 			add_to_string(&command, "REST ");
 			add_long_to_string(&command, conn->from
 							? conn->from
-							: conn->progress.start);
+							: conn->progress->start);
 			add_crlf_to_string(&command);
 
-			c_i->rest_sent = 1;
-			c_i->pending_commands++;
+			ftp->rest_sent = 1;
+			ftp->pending_commands++;
 		}
 
 		add_to_string(&command, "RETR ");
@@ -730,37 +730,39 @@ add_file_cmd_to_str(struct connection *conn)
 		add_crlf_to_string(&command);
 	}
 
-	c_i->opc = c_i->pending_commands;
+	done_string(&ftp_data_command);
+
+	ftp->opc = ftp->pending_commands;
 
 	/* 1 byte is already reserved for cmd_buffer in struct ftp_connection_info. */
-	c_i = mem_realloc(c_i, sizeof(*c_i) + command.length);
-	if (!c_i) {
+	ftp = mem_realloc(ftp, sizeof(*ftp) + command.length);
+	if (!ftp) {
 		done_string(&command);
-		abort_conn_with_state(conn, S_OUT_OF_MEM);
+		abort_connection(conn, S_OUT_OF_MEM);
 		return NULL;
 	}
 
-	memcpy(c_i->cmd_buffer, command.source, command.length + 1);
+	memcpy(ftp->cmd_buffer, command.source, command.length + 1);
 	done_string(&command);
-	conn->info = c_i;
+	conn->info = ftp;
 
-	return c_i;
+	return ftp;
 }
 
 static void
 send_it_line_by_line(struct connection *conn, struct string *cmd)
 {
-	struct ftp_connection_info *c_i = conn->info;
-	unsigned char *nl = strchr(c_i->cmd_buffer, '\n');
+	struct ftp_connection_info *ftp = conn->info;
+	unsigned char *nl = strchr(ftp->cmd_buffer, '\n');
 
 	if (!nl) {
-		add_to_string(cmd, c_i->cmd_buffer);
+		add_to_string(cmd, ftp->cmd_buffer);
 		return;
 	}
 
 	nl++;
-	add_bytes_to_string(cmd, c_i->cmd_buffer, nl - c_i->cmd_buffer);
-	memmove(c_i->cmd_buffer, nl, strlen(nl) + 1);
+	add_bytes_to_string(cmd, ftp->cmd_buffer, nl - ftp->cmd_buffer);
+	memmove(ftp->cmd_buffer, nl, strlen(nl) + 1);
 }
 
 /* Send commands to retrieve file or directory. */
@@ -770,7 +772,7 @@ ftp_send_retr_req(struct connection *conn, int state)
 	struct string cmd;
 
 	if (!init_string(&cmd)) {
-		abort_conn_with_state(conn, S_OUT_OF_MEM);
+		abort_connection(conn, S_OUT_OF_MEM);
 		return;
 	}
 
@@ -788,10 +790,10 @@ ftp_send_retr_req(struct connection *conn, int state)
 }
 
 /* Parse RETR response and return file size or -1 on error. */
-static long int
+static off_t
 get_filesize_from_RETR(unsigned char *data, int data_len)
 {
-	long int file_len;
+	off_t file_len;
 	int pos;
 	int pos_file_len = 0;
 
@@ -827,49 +829,50 @@ next:
 		return -1;
 
 	errno = 0;
-	file_len = strtol(&data[pos_file_len], NULL, 10);
+	file_len = (off_t) strtol(&data[pos_file_len], NULL, 10);
 	if (errno) return -1;
 
 	return file_len;
 }
 
 static int
-ftp_data_connect(struct connection *conn, int family, struct sockaddr_storage *sa,
+ftp_data_connect(struct connection *conn, int pf, struct sockaddr_storage *sa,
 		 int size_of_sockaddr)
 {
-	int fd = socket(family, SOCK_STREAM, 0);
+	int fd = socket(pf, SOCK_STREAM, 0);
 
 	if (fd < 0 || set_nonblocking_fd(fd) < 0) {
-		abort_conn_with_state(conn, S_FTP_ERROR);
+		abort_connection(conn, S_FTP_ERROR);
 		return -1;
 	}
 
 	set_ip_tos_throughput(fd);
 
-	conn->data_socket.fd = fd;
+	conn->data_socket->fd = fd;
 	/* XXX: We ignore connect() errors here. */
 	connect(fd, (struct sockaddr *) sa, size_of_sockaddr);
 	return 0;
 }
 
 static void
-ftp_retr_file(struct connection *conn, struct read_buffer *rb)
+ftp_retr_file(struct socket *socket, struct read_buffer *rb)
 {
-	struct ftp_connection_info *c_i = conn->info;
-	struct sockaddr_storage sa;
+	struct connection *conn = socket->conn;
+	struct ftp_connection_info *ftp = conn->info;
 	int response;
 
-	if (c_i->pending_commands > 1) {
+	if (ftp->pending_commands > 1) {
+		struct sockaddr_storage sa;
+
 		response = get_ftp_response(conn, rb, 0, &sa);
 
 		if (response == -1) {
-			abort_conn_with_state(conn, S_FTP_ERROR);
+			abort_connection(conn, S_FTP_ERROR);
 			return;
 		}
 
 		if (!response) {
-			read_from_socket(conn, &conn->socket, rb, ftp_retr_file);
-			set_connection_state(conn, S_GETH);
+			read_from_socket(conn->socket, rb, S_GETH, ftp_retr_file);
 			return;
 		}
 
@@ -885,41 +888,41 @@ ftp_retr_file(struct connection *conn, struct read_buffer *rb)
 		}
 #endif
 
-		c_i->pending_commands--;
+		ftp->pending_commands--;
 
 		/* XXX: The case values are order numbers of commands. */
-		switch (c_i->opc - c_i->pending_commands) {
+		switch (ftp->opc - ftp->pending_commands) {
 			case 1:	/* TYPE */
 				break;
 
 			case 2:	/* PORT */
 				if (response >= 400) {
-					abort_conn_with_state(conn, S_FTP_PORT);
+					abort_connection(conn, S_FTP_PORT);
 					return;
 				}
 				break;
 
 			case 3:	/* REST / CWD */
 				if (response >= 400) {
-					if (c_i->dir) {
-						abort_conn_with_state(conn,
+					if (ftp->dir) {
+						abort_connection(conn,
 								S_FTP_NO_FILE);
 						return;
 					}
 					conn->from = 0;
-				} else if (c_i->rest_sent) {
+				} else if (ftp->rest_sent) {
 					/* Following code is related to resume
 					 * feature. */
 					if (response == 350)
-						conn->from = conn->progress.start;
+						conn->from = conn->progress->start;
 					/* Come on, don't be nervous ;-). */
-					if (conn->progress.start >= 0) {
+					if (conn->progress->start >= 0) {
 						/* Update to the real value
 						 * which we've got from
 						 * Content-Range. */
-						conn->progress.seek = conn->from;
+						conn->progress->seek = conn->from;
 					}
-					conn->progress.start = conn->from;
+					conn->progress->start = conn->from;
 				}
 				break;
 
@@ -934,54 +937,54 @@ ftp_retr_file(struct connection *conn, struct read_buffer *rb)
 	response = get_ftp_response(conn, rb, 2, NULL);
 
 	if (response == -1) {
-		abort_conn_with_state(conn, S_FTP_ERROR);
+		abort_connection(conn, S_FTP_ERROR);
 		return;
 	}
 
 	if (!response) {
-		read_from_socket(conn, &conn->socket, rb, ftp_retr_file);
-		set_connection_state(conn, S_GETH);
+		read_from_socket(conn->socket, rb, S_GETH, ftp_retr_file);
 		return;
 	}
 
 	if (response >= 100 && response < 200) {
 		/* We only need to parse response after RETR to
 		 * get filesize if needed. */
-		if (!c_i->dir && conn->est_length == -1) {
-			long int file_len =
-				get_filesize_from_RETR(rb->data, rb->len);
+		if (!ftp->dir && conn->est_length == -1) {
+			off_t file_len;
 
+			file_len = get_filesize_from_RETR(rb->data, rb->length);
 			if (file_len > 0) {
 				/* FIXME: ..when downloads resuming
 				 * implemented.. */
-				conn->est_length = file_len + conn->progress.start;
+				conn->est_length = file_len + conn->progress->start;
 			}
 		}
 	}
 
-	set_handlers(conn->data_socket.fd,
-		     (void (*)(void *)) got_something_from_data_connection,
+	set_handlers(conn->data_socket->fd, (select_handler_T) ftp_data_accept,
 		     NULL, NULL, conn);
 
-	/* read_from_socket(conn, &conn->socket, rb, ftp_got_final_response); */
-	ftp_got_final_response(conn, rb);
+	/* read_from_socket(conn->socket, rb, ftp_got_final_response); */
+	ftp_got_final_response(socket, rb);
 }
 
 static void
-ftp_got_final_response(struct connection *conn, struct read_buffer *rb)
+ftp_got_final_response(struct socket *socket, struct read_buffer *rb)
 {
-	struct ftp_connection_info *c_i = conn->info;
+	struct connection *conn = socket->conn;
+	struct ftp_connection_info *ftp = conn->info;
 	int response = get_ftp_response(conn, rb, 0, NULL);
 
 	if (response == -1) {
-		abort_conn_with_state(conn, S_FTP_ERROR);
+		abort_connection(conn, S_FTP_ERROR);
 		return;
 	}
 
 	if (!response) {
-		read_from_socket(conn, &conn->socket, rb, ftp_got_final_response);
-		if (conn->state != S_TRANS)
-			set_connection_state(conn, S_GETH);
+		enum connection_state state = conn->state != S_TRANS
+					    ? S_GETH : conn->state;
+
+		read_from_socket(conn->socket, rb, state, ftp_got_final_response);
 		return;
 	}
 
@@ -994,23 +997,23 @@ ftp_got_final_response(struct connection *conn, struct read_buffer *rb)
 
 		if (!conn->cached
 		    || !redirect_cache(conn->cached, "/", 1, 0)) {
-			abort_conn_with_state(conn, S_OUT_OF_MEM);
+			abort_connection(conn, S_OUT_OF_MEM);
 			return;
 		}
 
-		abort_conn_with_state(conn, S_OK);
+		abort_connection(conn, S_OK);
 		return;
 	}
 
 	if (response >= 400) {
-		abort_conn_with_state(conn, S_FTP_FILE_ERROR);
+		abort_connection(conn, S_FTP_FILE_ERROR);
 		return;
 	}
 
-	if (c_i->conn_state == 2) {
+	if (ftp->conn_state == 2) {
 		ftp_end_request(conn, S_OK);
 	} else {
-		c_i->conn_state = 1;
+		ftp->conn_state = 1;
 		if (conn->state != S_TRANS)
 			set_connection_state(conn, S_GETH);
 	}
@@ -1019,7 +1022,7 @@ ftp_got_final_response(struct connection *conn, struct read_buffer *rb)
 
 /* Display directory entry formatted in HTML. */
 static int
-display_dir_entry(struct cache_entry *cached, int *pos, int *tries,
+display_dir_entry(struct cache_entry *cached, off_t *pos, int *tries,
 		  int colorize_dir, unsigned char *dircolor,
 		  struct ftp_file_info *ftp_info)
 {
@@ -1031,7 +1034,7 @@ display_dir_entry(struct cache_entry *cached, int *pos, int *tries,
 	add_char_to_string(&string, ftp_info->type);
 
 	if (ftp_info->permissions) {
-		int p = ftp_info->permissions;
+		mode_t p = ftp_info->permissions;
 
 #define FTP_PERM(perms, buffer, flag, index, id) \
 	if ((perms) & (flag)) (buffer)[(index)] = (id);
@@ -1132,29 +1135,11 @@ display_dir_entry(struct cache_entry *cached, int *pos, int *tries,
 
 /* List a directory in html format. */
 static int
-ftp_process_dirlist(struct cache_entry *cached, int *pos,
+ftp_process_dirlist(struct cache_entry *cached, off_t *pos,
 		    unsigned char *buffer, int buflen, int last,
 		    int *tries, int colorize_dir, unsigned char *dircolor)
 {
 	int ret = 0;
-#ifdef DEBUG_FTP_PARSER
- 	static int debug_ftp_parser = 1;
-	int buflen_orig = buflen;
-	unsigned char *response_orig = NULL;
-
-	if (debug_ftp_parser) {
-		buffer = get_ftp_debug_parse_responses(buffer, buflen);
-		buflen = strlen(buffer);
-		response_orig = buffer;
-		debug_ftp_parser = 0;
-	}
-
-#define	end_ftp_dirlist_processing()	do { mem_free_if(response_orig); } while (0)
-#define get_ftp_dirlist_offset(retval)	int_min(retval, buflen_orig)
-#else
-#define	end_ftp_dirlist_processing()	/* Nothing to free */
-#define	get_ftp_dirlist_offset(retval)	(retval)
-#endif
 
 	while (1) {
 		struct ftp_file_info ftp_info = INIT_FTP_FILE_INFO;
@@ -1177,8 +1162,7 @@ ftp_process_dirlist(struct cache_entry *cached, int *pos,
 			if (bufp && buf[bufp - 1] == ASCII_CR) bufp--;
 		} else {
 			if (!bufp || (!last && bufl < FTP_BUF_SIZE)) {
-				end_ftp_dirlist_processing();
-				return get_ftp_dirlist_offset(ret);
+				return ret;
 			}
 
 			ret += bufp;
@@ -1197,8 +1181,7 @@ ftp_process_dirlist(struct cache_entry *cached, int *pos,
 			retv = display_dir_entry(cached, pos, tries, colorize_dir,
 						dircolor, &ftp_info);
 			if (retv < 0) {
-				end_ftp_dirlist_processing();
-				return get_ftp_dirlist_offset(ret);
+				return ret;
 			}
 		}
 #ifdef DEBUG_FTP_PARSER
@@ -1209,63 +1192,57 @@ ftp_process_dirlist(struct cache_entry *cached, int *pos,
 	}
 }
 
-static int
+static void
 ftp_data_accept(struct connection *conn)
 {
-	struct ftp_connection_info *c_i = conn->info;
+	struct ftp_connection_info *ftp = conn->info;
 	int newsock;
 
-	if (c_i->has_data) return 0;
+	set_connection_timeout(conn);
+	clear_handlers(conn->data_socket->fd);
 
-	c_i->has_data = 1;
-
-	clear_handlers(conn->data_socket.fd);
-
-	if ((conn->protocol_family != 1 && c_i->use_pasv)
+	if ((conn->socket->protocol_family != EL_PF_INET6 && ftp->use_pasv)
 #ifdef CONFIG_IPV6
-	    || (conn->protocol_family == 1 && c_i->use_epsv)
+	    || (conn->socket->protocol_family == EL_PF_INET6 && ftp->use_epsv)
 #endif
 	   ) {
-		newsock = conn->data_socket.fd;
+		newsock = conn->data_socket->fd;
 	} else {
-		newsock = accept(conn->data_socket.fd, NULL, NULL);
+		newsock = accept(conn->data_socket->fd, NULL, NULL);
 		if (newsock < 0) {
-			retry_conn_with_state(conn, -errno);
-			return -1;
+			retry_connection(conn, -errno);
+			return;
 		}
-		close(conn->data_socket.fd);
+		close(conn->data_socket->fd);
 	}
 
-	conn->data_socket.fd = newsock;
+	conn->data_socket->fd = newsock;
 
 	set_handlers(newsock,
-		     (void (*)(void *)) got_something_from_data_connection,
+		     (select_handler_T) got_something_from_data_connection,
 		     NULL, NULL, conn);
-	return 0;
 }
 
 static void
 got_something_from_data_connection(struct connection *conn)
 {
-	struct ftp_connection_info *c_i = conn->info;
+	struct ftp_connection_info *ftp = conn->info;
 	unsigned char dircolor[8];
 	int colorize_dir = 0;
-	int len;
+	ssize_t len;
 
 	/* XXX: This probably belongs rather to connect.c ? */
 
 	set_connection_timeout(conn);
 
-	if (ftp_data_accept(conn)) return;
-
 	if (!conn->cached) conn->cached = get_cache_entry(conn->uri);
 	if (!conn->cached) {
 out_of_mem:
-		abort_conn_with_state(conn, S_OUT_OF_MEM);
+		abort_connection(conn, S_OUT_OF_MEM);
 		return;
 	}
 
-	if (c_i->dir) {
+	if (ftp->dir) {
 		colorize_dir = get_opt_bool("document.browse.links.color_dirs");
 
 		if (colorize_dir) {
@@ -1274,12 +1251,12 @@ out_of_mem:
 		}
 	}
 
-	if (c_i->dir && !conn->from) {
+	if (ftp->dir && !conn->from) {
 		struct string string;
 		unsigned char *uristring;
 
 		if (!conn->uri->data) {
-			abort_conn_with_state(conn, S_FTP_ERROR);
+			abort_connection(conn, S_FTP_ERROR);
 			return;
 		}
 
@@ -1320,39 +1297,39 @@ out_of_mem:
 		mem_free_set(&conn->cached->content_type, stracpy("text/html"));
 	}
 
-	len = safe_read(conn->data_socket.fd, c_i->ftp_buffer + c_i->buf_pos,
-		        FTP_BUF_SIZE - c_i->buf_pos);
+	len = safe_read(conn->data_socket->fd, ftp->ftp_buffer + ftp->buf_pos,
+		        FTP_BUF_SIZE - ftp->buf_pos);
 	if (len < 0) {
-		retry_conn_with_state(conn, -errno);
+		retry_connection(conn, -errno);
 		return;
 	}
 
 	if (len > 0) {
-		if (!c_i->dir) {
-			conn->received += len;
+		conn->received += len;
+
+		if (!ftp->dir) {
 			if (add_fragment(conn->cached, conn->from,
-					 c_i->ftp_buffer, len) == 1)
+					 ftp->ftp_buffer, len) == 1)
 				conn->tries = 0;
 			conn->from += len;
 
 		} else {
 			int proceeded;
 
-			conn->received += len;
 			proceeded = ftp_process_dirlist(conn->cached,
 							&conn->from,
-							c_i->ftp_buffer,
-							len + c_i->buf_pos,
+							ftp->ftp_buffer,
+							len + ftp->buf_pos,
 							0, &conn->tries,
 							colorize_dir,
 							(unsigned char *) dircolor);
 
 			if (proceeded == -1) goto out_of_mem;
 
-			c_i->buf_pos += len - proceeded;
+			ftp->buf_pos += len - proceeded;
 
-			memmove(c_i->ftp_buffer, c_i->ftp_buffer + proceeded,
-				c_i->buf_pos);
+			memmove(ftp->ftp_buffer, ftp->ftp_buffer + proceeded,
+				ftp->buf_pos);
 
 		}
 
@@ -1361,20 +1338,19 @@ out_of_mem:
 	}
 
 	if (ftp_process_dirlist(conn->cached, &conn->from,
-				c_i->ftp_buffer, c_i->buf_pos, 1,
+				ftp->ftp_buffer, ftp->buf_pos, 1,
 				&conn->tries, colorize_dir,
 				(unsigned char *) dircolor) == -1)
 		goto out_of_mem;
 
-	if (c_i->dir) ADD_CONST("</pre>\n<hr>\n</body>\n</html>");
+	if (ftp->dir) ADD_CONST("</pre>\n<hr>\n</body>\n</html>");
 
-	clear_handlers(conn->data_socket.fd);
-	close_socket(NULL, &conn->data_socket);
+	close_socket(conn->data_socket);
 
-	if (c_i->conn_state == 1) {
+	if (ftp->conn_state == 1) {
 		ftp_end_request(conn, S_OK);
 	} else {
-		c_i->conn_state = 2;
+		ftp->conn_state = 2;
 		set_connection_state(conn, S_TRANS);
 	}
 }
@@ -1385,8 +1361,7 @@ ftp_end_request(struct connection *conn, enum connection_state state)
 	set_connection_state(conn, state);
 
 	if (conn->state == S_OK && conn->cached) {
-		truncate_entry(conn->cached, conn->from, 1);
-		conn->cached->incomplete = 0;
+		normalize_cache_entry(conn->cached, conn->from);
 	}
 
 	add_keepalive_connection(conn, FTP_KEEPALIVE_TIMEOUT, NULL);
