@@ -1,5 +1,4 @@
 /* URI rewriting module */
-/* $Id: rewrite.c,v 1.37.6.3 2005/01/04 00:42:57 jonas Exp $ */
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
@@ -9,12 +8,12 @@
 
 #include "config/options.h"
 #include "intl/gettext/libintl.h"
-#include "modules/module.h"
+#include "main/event.h"
+#include "main/module.h"
 #include "protocol/rewrite/rewrite.h"
 #include "protocol/uri.h"
-#include "sched/event.h"
-#include "sched/location.h"
-#include "sched/session.h"
+#include "session/location.h"
+#include "session/session.h"
 #include "util/string.h"
 
 
@@ -56,7 +55,7 @@ static struct option_info uri_rewrite_options[] = {
 		N_("Enable dumb prefixes - simple URI abbreviations which can\n"
 		"be written to the Goto URL dialog instead of actual URIs - i.e.\n"
 		"if you write 'elinks' there, you are directed to\n"
-		"http://elinks.or.cz/.")),
+		"http://elinks.cz/.")),
 
 	INIT_OPT_BOOL("protocol.rewrite", N_("Enable smart prefixes"),
 		"enable-smart", 0, 1,
@@ -91,6 +90,14 @@ static struct option_info uri_rewrite_options[] = {
 		"%0,%1,...,%9 means argument 0, 1, ..., 9\n"
 		"%% in the string means '%'")),
 
+	INIT_OPT_STRING("protocol.rewrite", N_("Default template"),
+		"default_template", 0, "",
+		N_("Default URI template used when the string entered in\n"
+		"the goto dialog does not appear to be a URI or a filename\n"
+		"(i.e. contains no '.', ':' or '/' characters), and does\n"
+		"not match any defined prefixes. Set the value to \"\" to\n"
+		"disable use of the default template rewrite rule.")),
+
 #define INIT_OPT_DUMB_PREFIX(prefix, uri) \
 	INIT_OPT_STRING("protocol.rewrite.dumb", NULL, prefix, 0, uri, NULL)
 
@@ -213,16 +220,6 @@ get_prefix_tree(enum uri_rewrite_option tree)
 	return &get_opt_rewrite(tree);
 }
 
-static inline void
-encode_uri_string_len(struct string *s, unsigned char *a, int alen)
-{
-	unsigned char c = a[alen];
-
-	a[alen] = 0;
-	encode_uri_string(s, a, 1);
-	a[alen] = c;
-}
-
 #define MAX_URI_ARGS 10
 
 static unsigned char *
@@ -271,7 +268,7 @@ rewrite_uri(unsigned char *url, struct uri *current_uri, unsigned char *arg)
 				add_uri_to_string(&n, current_uri, URI_ORIGINAL);
 				break;
 			case 's':
-				if (arg) encode_uri_string(&n, arg, 1);
+				if (arg) encode_uri_string(&n, arg, -1, 1);
 				break;
 			case '%':
 				add_char_to_string(&n, '%');
@@ -287,9 +284,9 @@ rewrite_uri(unsigned char *url, struct uri *current_uri, unsigned char *arg)
 			case '8':
 			case '9':
 				value = *url - '0';
-				if (value < argc)
-					encode_uri_string_len(&n, args[value],
-						argslen[value]);
+				if (value > argc) break;
+				encode_uri_string(&n, args[value],
+						  argslen[value], 1);
 				break;
 			default:
 				add_bytes_to_string(&n, url - 1, 2);
@@ -334,6 +331,18 @@ goto_url_hook(va_list ap, void *data)
 	if (get_dumb_enable() && !uu && !*argstart)
 		uu = get_uri_rewrite_prefix(URI_REWRITE_DUMB, *url);
 
+	if (!uu
+	    && !strchr(*url, ':')
+	    && !strchr(*url, '.')
+	    && !strchr(*url, '/')) {
+		uu = get_opt_str("protocol.rewrite.default_template");
+		if (uu && *uu) {
+			arg = *url;
+		} else {
+			uu = NULL;
+		}
+	}
+
 	if (uu) {
 		struct uri *uri = ses && have_location(ses)
 				? cur_loc(ses)->vs.uri : NULL;
@@ -349,7 +358,7 @@ goto_url_hook(va_list ap, void *data)
 }
 
 struct event_hook_info uri_rewrite_hooks[] = {
-	{ "goto-url",	goto_url_hook },
+	{ "goto-url", -1, goto_url_hook },
 
 	NULL_EVENT_HOOK_INFO
 };
