@@ -1,5 +1,4 @@
 /* URL parser and translator; implementation of RFC 2396. */
-/* $Id: uri.c,v 1.304.2.4 2005/05/01 20:57:46 jonas Exp $ */
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
@@ -30,6 +29,7 @@
 
 #include "elinks.h"
 
+#include "main/object.h"
 #include "protocol/protocol.h"
 #include "protocol/uri.h"
 #include "util/conv.h"
@@ -37,7 +37,6 @@
 #include "util/file.h"
 #include "util/hash.h"
 #include "util/memory.h"
-#include "util/object.h"
 #include "util/string.h"
 
 
@@ -116,50 +115,52 @@ end_with_known_tld(unsigned char *s, int slen)
 	return -1;
 }
 
+/* XXX: this function writes to @name. */
+static int
+check_whether_file_exists(unsigned char *name)
+{
+	/* Check POST_CHAR etc ... */
+	static const unsigned char chars[] = POST_CHAR_S "#?";
+	int i;
+	int namelen = strlen(name);
+
+	if (file_exists(name))
+		return namelen;
+
+	for (i = 0; i < sizeof(chars) - 1; i++) {
+		unsigned char *pos = memchr(name, chars[i], namelen);
+		int exists;
+		
+		if (!pos) continue;
+
+		*pos = 0;
+		exists = file_exists(name);
+		*pos = chars[i];
+		
+		if (exists) {
+			return pos - name;
+		}
+	}
+
+	return -1;
+}
+
 static int
 check_uri_file(unsigned char *name)
 {
 	/* Check POST_CHAR etc ... */
 	static const unsigned char chars[] = POST_CHAR_S "#?";
-	int i;
 
-	if (file_exists(name))
-		return strlen(name);
-
-	for (i = 0; i < sizeof(chars) - 1; i++) {
-		unsigned char *pos = strchr(name, chars[i]);
-		int namelen = -1;
-
-		if (!pos) continue;
-
-		*pos = 0;
-		if (file_exists(name))
-			namelen = strlen(name);
-		*pos = chars[i];
-
-		if (namelen >= 0) return namelen;
-	}
-
-	return -1;
+	return strcspn(name, chars);
 }
 
 /* Encodes URIs without encoding stuff like fragments and query separators. */
 static void
 encode_file_uri_string(struct string *string, unsigned char *uristring)
 {
-	int filenamelen = check_uri_file(uristring);
-	unsigned char saved = 0;
+	int filenamelen = check_whether_file_exists(uristring);
 
-	if (filenamelen >= 0) {
-		saved = uristring[filenamelen];
-		uristring[filenamelen] = 0;
-	}
-
-	encode_uri_string(string, uristring, 0);
-	if (filenamelen >= 0) {
-		uristring[filenamelen] = saved;
-		add_to_string(string, &uristring[filenamelen]);
-	}
+	encode_uri_string(string, uristring, filenamelen, 0);
 }
 
 
@@ -650,7 +651,7 @@ add_string_uri_to_string(struct string *string, unsigned char *uristring,
 #define normalize_uri_reparse(str)	normalize_uri(NULL, str)
 #define normalize_uri_noparse(uri)	normalize_uri(uri, struri(uri))
 
-static unsigned char *
+unsigned char *
 normalize_uri(struct uri *uri, unsigned char *uristring)
 {
 	unsigned char *parse_string = uristring;
@@ -793,7 +794,7 @@ transform_file_url(struct uri *uri, unsigned char *cwd)
 		if (!init_string(&dir))
 			return NULL;
 
-		encode_uri_string(&dir, cwd, 0);
+		encode_uri_string(&dir, cwd, -1, 0);
 
 		/* Either we will end up with '//' and translate_directories()
 		 * will shorten it or the '/' will mark the inserted cwd as a
@@ -958,14 +959,14 @@ join_urls(struct uri *base, unsigned char *rel)
 
 /* Tries to figure out what protocol @newurl might be specifying by checking if
  * it exists as a file locally or by checking parts of the host name. */
-static inline enum protocol
+static enum protocol
 find_uri_protocol(unsigned char *newurl)
 {
 	unsigned char *ch;
 
 	/* First see if it is a file so filenames that look like hostnames
 	 * won't confuse us below. */
-	if (check_uri_file(newurl) >= 0) return PROTOCOL_FILE;
+	if (check_whether_file_exists(newurl) >= 0) return PROTOCOL_FILE;
 
 	/* Yes, it would be simpler to make test for IPv6 address first,
 	 * but it would result in confusing mix of ifdefs ;-). */
@@ -978,8 +979,9 @@ find_uri_protocol(unsigned char *newurl)
 	 */
 
 	ch = newurl + strcspn(newurl, ".:/@");
-	if (*ch == '@' || (*ch == ':' && *newurl != '[')
-		|| !strncasecmp(newurl, "ftp.", 4)) {
+	if (*ch == '@'
+	    || (*ch == ':' && *newurl != '[' && strchr(newurl, '@'))
+	    || !strncasecmp(newurl, "ftp.", 4)) {
 		/* Contains user/password/ftp-hostname */
 		return PROTOCOL_FTP;
 
@@ -1018,11 +1020,7 @@ find_uri_protocol(unsigned char *newurl)
 			return PROTOCOL_HTTP;
 	}
 
-	/* We default to file:// even though we already tested if the file
-	 * existed since it will give a "No such file or directory" error.
-	 * which might better hint the user that there was problem figuring out
-	 * the URI. */
-	return PROTOCOL_FILE;
+	return PROTOCOL_UNKNOWN;
 }
 
 
@@ -1068,23 +1066,37 @@ parse_uri:
 		/* Fix translation of 1.2.3.4:5 so IP address part won't be
 		 * interpreted as the protocol name. */
 		if (uri.protocol == PROTOCOL_UNKNOWN) {
-			unsigned char *ipscan = newurl;
+			enum protocol protocol = find_uri_protocol(newurl);
 
-			/* FIXME: This is looking more and more like the code
-			 * for the URI_ERRNO_INVALID_PROTOCOL label and
-			 * find_uri_protocol(). Unite? --jonas */
-
-			/* Check if it could be just some local file with ':'
-			 * in its name. */
-			if (check_uri_file(newurl) >= 0) {
+			/* Code duplication with the URI_ERRNO_INVALID_PROTOCOL
+			 * case. */
+			if (protocol != PROTOCOL_UNKNOWN) {
 				struct string str;
 
 				if (!init_string(&str)) return NULL;
 
-				add_to_string(&str, "file://");
-				if (!dir_sep(*newurl))
-					add_to_string(&str, "./");
-				encode_file_uri_string(&str, newurl);
+				switch (protocol) {
+				case PROTOCOL_FTP:
+					add_to_string(&str, "ftp://");
+					encode_uri_string(&str, newurl, -1, 0);
+					break;
+
+				case PROTOCOL_HTTP:
+					add_to_string(&str, "http://");
+					add_to_string(&str, newurl);
+					break;
+
+				case PROTOCOL_UNKNOWN:
+					break;
+
+				case PROTOCOL_FILE:
+				default:
+					add_to_string(&str, "file://");
+					if (!dir_sep(*newurl))
+						add_to_string(&str, "./");
+
+					add_to_string(&str, newurl);
+				}
 
 				mem_free(newurl);
 				newurl = str.source;
@@ -1093,17 +1105,6 @@ parse_uri:
 				prev_errno = URI_ERRNO_EMPTY;
 				goto parse_uri;
 			}
-
-			/* Check if it's <IP-address>:<port> */
-			while (isdigit(*ipscan) || *ipscan == '.')
-				ipscan++;
-
-			if (!*ipscan || *ipscan == ':' || *ipscan == '/') {
-				insert_in_string(&newurl, 0, "http://", 7);
-				/* Work around the infinite loop prevention */
-				prev_errno = URI_ERRNO_EMPTY;
-				goto parse_uri;
-			}
 		}
 
 		/* If file:// URI is transformed we need to reparse. */
@@ -1195,7 +1196,7 @@ parse_uri:
 		switch (protocol) {
 			case PROTOCOL_FTP:
 				add_to_string(&str, "ftp://");
-				encode_uri_string(&str, newurl, 0);
+				encode_uri_string(&str, newurl, -1, 0);
 				break;
 
 			case PROTOCOL_HTTP:
@@ -1203,6 +1204,12 @@ parse_uri:
 				add_to_string(&str, newurl);
 				break;
 
+			case PROTOCOL_UNKNOWN:
+				/* We default to file:// even though we already
+				 * tested if the file existed since it will give
+				 * a "No such file or directory" error.  which
+				 * might better hint the user that there was
+				 * problem figuring out the URI. */
 			case PROTOCOL_FILE:
 			default:
 				add_to_string(&str, "file://");
@@ -1299,15 +1306,18 @@ safe_char(unsigned char c)
 }
 
 void
-encode_uri_string(struct string *string, unsigned char *name,
+encode_uri_string(struct string *string, unsigned char *name, int namelen,
 		  int convert_slashes)
 {
 	unsigned char n[4];
+	unsigned char *end;
 
 	n[0] = '%';
 	n[3] = '\0';
 
-	for (; *name; name++) {
+	if (namelen < 0) namelen = strlen(name);
+
+	for (end = name + namelen; name < end; name++) {
 #if 0
 		/* This is probably correct only for query part of URI..? */
 		if (*name == ' ') add_char_to_string(data, len, '+');
@@ -1421,7 +1431,7 @@ free_uri_list(struct uri_list *list)
 		done_uri(uri);
 	}
 
-	mem_free(list->uris);
+	mem_free_set(&list->uris, NULL);
 	list->size = 0;
 }
 
