@@ -1,9 +1,7 @@
-/* $Id: uri.h,v 1.154.2.1 2005/01/04 00:26:00 jonas Exp $ */
-
 #ifndef EL__PROTOCOL_URI_H
 #define EL__PROTOCOL_URI_H
 
-#include "util/object.h"
+#include "main/object.h"
 
 struct string;
 
@@ -239,6 +237,11 @@ struct uri *get_composed_uri(struct uri *uri, enum uri_component components);
  * @cwd		if non NULL @uristring will be translated using this CWD. */
 struct uri *get_translated_uri(unsigned char *uristring, unsigned char *cwd);
 
+/* Normalizes the directory structure given in uristring. XXX: The function
+ * modifies the uristring and returns it. The uri argument should be NULL
+ * if the uri is not the parsed uristring. */
+unsigned char *normalize_uri(struct uri *uri, unsigned char *uristring);
+
 /* Check if two URIs are equal. If @components are 0 simply compare the whole
  * URI else only compare the specific parts. */
 int compare_uri(struct uri *uri1, struct uri *uri2, enum uri_component components);
@@ -269,7 +272,12 @@ int get_uri_port(struct uri *uri);
 	(LOWEST_PORT <= (port) && (port) <= HIGHEST_PORT)
 
 
-void encode_uri_string(struct string *string, unsigned char *src, int len);
+/* Encode and add @namelen bytes from @name to @string. If @namelen is -1 it is
+ * set to strlen(@name). If the boolean convert_slashes is zero '/'-chars will
+ * not be encoded. */
+void encode_uri_string(struct string *string, unsigned char *name, int namelen,
+		       int convert_slashes);
+
 void decode_uri_string(struct string *string);
 void decode_uri(unsigned char *uristring);
 
@@ -283,8 +291,8 @@ unsigned char *get_extension_from_uri(struct uri *uri);
 
 
 /* Resolves a @relative URI to absolute form using @base URI.
- * Example: if @base is http://elinks.or.cz/ and @relative is #news
- *	    the outcome would be http://elinks.or.cz/#news */
+ * Example: if @base is http://elinks.cz/ and @relative is #news
+ *	    the outcome would be http://elinks.cz/#news */
 unsigned char *join_urls(struct uri *base, unsigned char *relative);
 
 /* Return position if end of string @s matches a known tld or -1 if not.
