@@ -1,5 +1,4 @@
 /* Guile interface (scripting engine) */
-/* $Id: core.c,v 1.17 2004/05/20 12:49:45 jonas Exp $ */
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
@@ -9,11 +8,10 @@
 
 #include "elinks.h"
 
-#include "lowlevel/home.h"
-#include "modules/module.h"
+#include "config/home.h"
+#include "main/module.h"
 #include "scripting/guile/core.h"
 #include "scripting/guile/hooks.h"
-#include "scripting/scripting.h"
 #include "util/error.h"
 #include "util/file.h"
 #include "util/string.h"
@@ -40,7 +38,7 @@
 /* c_xdialog */
 
 
-static void
+void
 init_guile(struct module *module)
 {
 	SCM user_module;
@@ -82,14 +80,3 @@ init_guile(struct module *module)
 		scm_c_primitive_load_path(path);
 	mem_free(path);
 }
-
-
-struct module guile_scripting_module = struct_module(
-	/* name: */		"Guile",
-	/* options: */		NULL,
-	/* events: */		guile_scripting_hooks,
-	/* submodules: */	NULL,
-	/* data: */		NULL,
-	/* init: */		init_guile,
-	/* done: */		NULL
-);
