@@ -1,5 +1,4 @@
 /* Perl scripting engine */
-/* $Id: core.c,v 1.11.6.1 2005/09/14 12:09:45 jonas Exp $ */
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
@@ -10,11 +9,9 @@
 
 #include "elinks.h"
 
-#include "main.h"
-#include "lowlevel/home.h"
-#include "modules/module.h"
+#include "config/home.h"
+#include "main/module.h"
 #include "scripting/perl/core.h"
-#include "scripting/perl/hooks.h"
 #include "util/file.h"
 
 #define PERL_HOOKS_FILENAME	"hooks.pl"
@@ -39,7 +36,7 @@ get_global_hook_file(void)
 static char *
 get_local_hook_file(void)
 {
-	static char buf[256];	/* TODO: MAX_PATH ??? --Zas */
+	static char buf[1024];	/* TODO: MAX_PATH ??? --Zas */
 
 	if (!elinks_home) return NULL;
 	snprintf(buf, sizeof(buf), "%s%s", elinks_home, PERL_HOOKS_FILENAME);
@@ -58,7 +55,7 @@ precleanup_perl(struct module *module)
 	my_perl = NULL;
 }
 
-static void
+void
 cleanup_perl(struct module *module)
 {
 	precleanup_perl(module);
@@ -68,7 +65,7 @@ cleanup_perl(struct module *module)
 }
 
 
-static void
+void
 init_perl(struct module *module)
 {
 	/* FIXME: it seems that some systems like OS/2 requires PERL_SYS_INIT3
@@ -108,14 +105,3 @@ init_perl(struct module *module)
 		if (err) precleanup_perl(module);
 	}
 }
-
-
-struct module perl_scripting_module = struct_module(
-	/* name: */		"Perl",
-	/* options: */		NULL,
-	/* hooks: */		perl_scripting_hooks,
-	/* submodules: */	NULL,
-	/* data: */		NULL,
-	/* init: */		init_perl,
-	/* done: */		cleanup_perl
-);
