@@ -1,10 +1,8 @@
-/* $Id: conv.h,v 1.41 2004/11/25 23:50:15 miciah Exp $ */
-
 #ifndef EL__UTIL_CONV_H
 #define EL__UTIL_CONV_H
 
 #include "util/string.h"
-#include "util/ttime.h" /* ttime type */
+#include "util/time.h" /* timeval_T types */
 
 static inline int
 is_safe_in_shell(unsigned char c)
@@ -48,13 +46,14 @@ unhx(register unsigned char a)
 /* These use granular allocation stuff. */
 struct string *add_long_to_string(struct string *string, long number);
 struct string *add_knum_to_string(struct string *string, long number);
-struct string *add_xnum_to_string(struct string *string, int number);
-struct string *add_time_to_string(struct string *string, ttime time);
+struct string *add_xnum_to_string(struct string *string, off_t number);
+struct string *add_duration_to_string(struct string *string, long seconds);
+struct string *add_timeval_to_string(struct string *string, timeval_T *timeval);
 
 #ifdef HAVE_STRFTIME
 /* Uses strftime() to add @fmt time format to @string. If @time is NULL
  * time(NULL) will be used. */
-struct string *add_date_to_string(struct string *string, unsigned char *format, ttime *time);
+struct string *add_date_to_string(struct string *string, unsigned char *format, time_t *time);
 #endif
 
 
