@@ -1,5 +1,3 @@
-/* $Id: error.h,v 1.45 2004/08/12 08:40:30 miciah Exp $ */
-
 #ifndef EL__UTIL_ERROR_H
 #define EL__UTIL_ERROR_H
 
@@ -55,6 +53,43 @@ void elinks_internal(unsigned char *fmt, ...);
 void usrerror(unsigned char *fmt, ...);
 
 
+#ifdef HAVE_VARIADIC_MACROS
+#ifdef CONFIG_DEBUG
+/* The LOG_*() macros can be used to log to a file, however, by default log
+ * messages are written to stderr. Set the following environment variables
+ * to configure the log behavior:
+ *
+ *	ELINKS_LOG	- The path to the log file, it is opened for appending
+ *	ELINKS_MSG	- A comma separated list containing "error", "warn",
+ *			  "info" and/or "debug" which can be used to limit
+ *			  what messages to emit to the log.
+ *	ELINKS_FILES	- A comma separated list of which files names to
+ *			  emit log messages from.
+ */
+void
+elinks_log(unsigned char *msg, unsigned char *file, int line,
+	   unsigned char *fmt, ...);
+
+#undef LOG_ERR
+#define LOG_ERR(args...) \
+	elinks_log("error", __FILE__, __LINE__, args)
+
+#undef LOG_WARN
+#define LOG_WARN(args...) \
+	elinks_log("warn", __FILE__, __LINE__, args)
+
+#undef LOG_INFO
+#define LOG_INFO(args...) \
+	elinks_log("info", __FILE__, __LINE__, args)
+
+#undef LOG_DBG
+#define LOG_DBG(args...) \
+	elinks_log("debug", __FILE__, __LINE__, args)
+
+#endif
+#endif
+
+
 
 /* This is our smart assert(). It is basically equivalent to if (x) INTERNAL(),
  * but it generates a uniform message and mainly does not do the test if we are
@@ -135,7 +170,7 @@ void elinks_assertm(int x, unsigned char *fmt, ...)
  * paths will get hit (and since developers usually don't test CONFIG_FASTMEM mode
  * extensively...). So better don't mess with it, even if you would do that
  * with awareness of this fact. We don't want to iterate over tens of spots all
- * over the code when we chane one detail regarding CONFIG_FASTMEM operation.
+ * over the code when we change one detail regarding CONFIG_FASTMEM operation.
  *
  * This is not that actual after introduction of if_assert_failed, but it's
  * a safe recommendation anyway, so... ;-) */
