@@ -1,5 +1,3 @@
-/* $Id: lists.h,v 1.44 2004/12/24 02:33:14 jonas Exp $ */
-
 #ifndef EL__UTIL_LISTS_H
 #define EL__UTIL_LISTS_H
 
@@ -9,6 +7,13 @@
  * because they are likely to take & of the parameter. Worst of all, it will
  * work with gcc. But nowhere else (at least not w/ tcc). */
 
+/* Note that this whole lists implementation code is severely broken. All of
+ * it is a single huge violation of C aliasing rules, just accessing things
+ * like we do here is totally prohibited and bound to generate segfaults in
+ * proportion with rising optimization level and gcc version.
+ *
+ * Fixing this would be a nice and needed janitorial project. */
+
 /* Lists debugging
  * Two unsigned int magic number will be put before and after the next and
  * prev pointers, these will be check on list operations.
@@ -48,7 +53,6 @@ struct xlist_head {
 
 #define NULL_LIST_HEAD NULL, NULL
 #define D_LIST_HEAD(x) &x, &x
-#define INIT_LIST_HEAD(x) struct list_head x = { D_LIST_HEAD(x) }
 #define LIST_HEAD(x) x *next; x *prev
 #define LIST_SET_MAGIC(x) list_magic_set(*(x))
 
@@ -107,12 +111,12 @@ struct xlist_head {
 
 #define NULL_LIST_HEAD LISTMAGIC1, NULL, NULL, LISTMAGIC2
 #define D_LIST_HEAD(x) LISTMAGIC1, &x, &x, LISTMAGIC2
-#define INIT_LIST_HEAD(x) struct list_head x = { D_LIST_HEAD(x) }
 #define LIST_HEAD(x) void *magic1; x *next; x *prev; void *magic2
 #define LIST_SET_MAGIC(x) list_magic_set(*(x))
 
 #endif /* LISTDEBUG */
 
+#define INIT_LIST_HEAD(x) struct list_head x = { D_LIST_HEAD(x) }
 
 #ifdef HAVE_TYPEOF
 #define list_typeof(x) typeof(x)
@@ -135,6 +139,9 @@ do { \
 #define list_is_singleton(x) \
 	(list_magic_chkbool(x, "list_is_singleton") && (x).next == (x).prev)
 
+#define list_has_prev(l,p) \
+	(list_magic_chkbool(l, "list_has_prev") && (p)->prev !=  (void *) &(l))
+
 #define list_has_next(l,p) \
 	(list_magic_chkbool(l, "list_has_next") && (p)->next !=  (void *) &(l))
 
@@ -182,6 +189,11 @@ do { \
 	     (list_typeof(e)) (e) != (list_typeof(e)) &(l); \
 	     (e) = (n), (n) = (e)->next)
 
+#define foreachbacksafe(e, p, l) \
+	for ((e) = (l).prev, (p) = (e)->prev; \
+	     (list_typeof(e)) (e) != (list_typeof(e)) &(l); \
+	     (e) = (p), (p) = (e)->prev)
+
 #define free_list(l) \
 do { \
 	struct xlist_head *head, *next; \
@@ -197,5 +209,24 @@ do { \
 	do_not_optimize_here_gcc_2_7(&l); \
 } while (0)
 
+static inline int
+list_size(struct list_head *list)
+{
+	struct list_head *item;
+	int size = 0;
+
+	foreach (item, *list)
+		size++;
+
+	return size;
+}
+
+#define move_to_top_of_list(list, item) do { \
+	if ((item) != (list).next) { \
+		del_from_list(item); \
+		add_to_list(list, item); \
+	} \
+} while (0)
+
 
 #endif /* EL__UTIL_LISTS_H */
