@@ -1,5 +1,4 @@
 /* Memory debugging (leaks, overflows & co) */
-/* $Id: memdebug.c,v 1.30.6.2 2005/09/14 15:09:30 jonas Exp $ */
 
 /* Wrappers for libc memory managment providing protection against common
  * pointers manipulation mistakes - bad realloc()/free() pointers, double
@@ -496,7 +495,7 @@ debug_mem_realloc(unsigned char *file, int line, void *ptr, size_t size)
 	 * and change nothing, this conforms to usual realloc() behavior. */
 	if (ah->size == size) {
 #ifdef CHECK_USELESS_REALLOC
-		fprintf(stderr, "[%s:%d] mem_realloc() oldsize = newsize = %d\n", file, line, size);
+		fprintf(stderr, "[%s:%d] mem_realloc() oldsize = newsize = %zu\n", file, line, size);
 #endif
 		return (void *) ptr;
 	}
@@ -552,7 +551,8 @@ set_mem_comment(void *ptr, unsigned char *str, int len)
 
 	ah->comment = malloc(len + 1);
 	if (ah->comment) {
-		safe_strncpy(ah->comment, str, len + 1);
+		memcpy(ah->comment, str, len);
+		ah->comment[len] = 0;
 		mem_stats.true_amount += len + 1;
 	}
 }
