@@ -1,5 +1,4 @@
 /* Secure file saving handling */
-/* $Id: secsave.h,v 1.8 2004/07/28 02:20:01 jonas Exp $ */
 
 #ifndef EL__UTIL_SECSAVE_H
 #define EL__UTIL_SECSAVE_H
@@ -32,7 +31,8 @@ struct secure_save_info {
 	int secure_save; /* use secure save for this file */
 };
 
-struct secure_save_info *secure_open(unsigned char *, mode_t);
+struct secure_save_info *secure_open(unsigned char *);
+
 int secure_close(struct secure_save_info *);
 
 int secure_fputs(struct secure_save_info *, const char *);
