@@ -1,5 +1,4 @@
 /* Text mode drawing functions */
-/* $Id: draw.c,v 1.21.4.4 2005/09/14 15:10:52 jonas Exp $ */
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
@@ -26,7 +25,8 @@
 #include "intl/charsets.h"
 #include "intl/gettext/libintl.h"
 #include "protocol/uri.h"
-#include "sched/session.h"
+#include "session/location.h"
+#include "session/session.h"
 #include "terminal/draw.h"
 #include "terminal/tab.h"
 #include "terminal/terminal.h"
@@ -291,7 +291,7 @@ draw_frames(struct session *ses)
 {
 	struct document_view *doc_view, *current_doc_view;
 	int *l;
-	int n, d, more;
+	int n, d;
 
 	assert(ses && ses->doc_view && ses->doc_view->document);
 	if_assert_failed return;
@@ -308,16 +308,19 @@ draw_frames(struct session *ses)
 
 	current_doc_view = current_frame(ses);
 	d = 0;
-	do {
-		more = 0;
+	while (1) {
+		int more = 0;
+
 		foreach (doc_view, ses->scrn_frames) {
 			if (doc_view->depth == d)
 				draw_doc(ses, doc_view, doc_view == current_doc_view);
 			else if (doc_view->depth > d)
 				more = 1;
 		}
+
+		if (!more) break;
 		d++;
-	} while (more);
+	};
 }
 
 /* @rerender is ridiciously wound-up. */
