@@ -1,10 +1,10 @@
-/* $Id: form.h,v 1.48 2004/12/19 16:47:25 zas Exp $ */
 
 #ifndef EL__VIEWER_TEXT_FORM_H
 #define EL__VIEWER_TEXT_FORM_H
 
 #include "document/forms.h"
 #include "util/lists.h" /* LIST_HEAD */
+#include "viewer/action.h"
 
 struct document;
 struct document_view;
@@ -92,4 +92,6 @@ void submit_given_form(struct session *ses, struct document_view *doc_view, stru
 void auto_submit_form(struct session *ses);
 void do_reset_form(struct document_view *doc_view, struct form *form);
 
+void link_form_menu(struct session *ses);
+
 #endif
