@@ -1,5 +1,4 @@
 /* HTML viewer (and much more) */
-/* $Id: view.c,v 1.665.2.11 2005/09/14 15:16:25 jonas Exp $ */
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
@@ -29,14 +28,13 @@
 #include "document/view.h"
 #include "intl/charsets.h"
 #include "intl/gettext/libintl.h"
+#include "main/event.h"
 #include "osdep/osdep.h"
 #include "protocol/uri.h"
-#include "sched/action.h"
-#include "sched/download.h"
-#include "sched/event.h"
-#include "sched/location.h"
-#include "sched/session.h"
-#include "sched/task.h"
+#include "session/download.h"
+#include "session/location.h"
+#include "session/session.h"
+#include "session/task.h"
 #include "terminal/draw.h"
 #include "terminal/event.h"
 #include "terminal/kbd.h"
@@ -50,6 +48,7 @@
 #include "util/memory.h"
 #include "util/snprintf.h"
 #include "util/string.h"
+#include "viewer/action.h"
 #include "viewer/dump/dump.h"
 #include "viewer/text/draw.h"
 #include "viewer/text/form.h"
@@ -111,10 +110,8 @@ enum frame_event_status
 move_page_down(struct session *ses, struct document_view *doc_view)
 {
 	enum frame_event_status status;
-	int count;
+	int count = eat_kbd_repeat_count(ses);
 
-	count = ses->kbdprefix.repeat_count;
-	ses->kbdprefix.repeat_count = 0;
 	ses->navigate_mode = NAVIGATE_LINKWISE;
 
 	do {
@@ -155,10 +152,8 @@ enum frame_event_status
 move_page_up(struct session *ses, struct document_view *doc_view)
 {
 	enum frame_event_status status;
-	int count;
+	int count = eat_kbd_repeat_count(ses);
 
-	count = ses->kbdprefix.repeat_count;
-	ses->kbdprefix.repeat_count = 0;
 	ses->navigate_mode = NAVIGATE_LINKWISE;
 
 	do {
@@ -190,8 +185,7 @@ move_link(struct session *ses, struct document_view *doc_view, int direction,
 		wraparound = get_opt_bool("document.browse.links.wraparound");
 	}
 
-	count = ses->kbdprefix.repeat_count;
-	ses->kbdprefix.repeat_count = 0;
+	count = eat_kbd_repeat_count(ses);
 
 	do {
 		int current_link = doc_view->vs->current_link;
@@ -206,8 +200,8 @@ move_link(struct session *ses, struct document_view *doc_view, int direction,
 			}
 
 		} else {
-			if (next_link_in_view(doc_view, current_link + direction,
-					      direction, link_in_view_y, set_pos_x))
+			if (next_link_in_view_y(doc_view, current_link + direction,
+					        direction))
 				continue;
 		}
 
@@ -248,8 +242,7 @@ move_link_dir(struct session *ses, struct document_view *doc_view, int dir_x, in
 
 	ses->navigate_mode = NAVIGATE_LINKWISE;
 
-	count = ses->kbdprefix.repeat_count;
-	ses->kbdprefix.repeat_count = 0;
+	count = eat_kbd_repeat_count(ses);
 
 	do {
 		int current_link = doc_view->vs->current_link;
@@ -340,9 +333,7 @@ horizontal_scroll(struct session *ses, struct document_view *doc_view, int steps
 enum frame_event_status
 scroll_up(struct session *ses, struct document_view *doc_view)
 {
-	int steps = ses->kbdprefix.repeat_count;
-
-	ses->kbdprefix.repeat_count = 0;
+	int steps = eat_kbd_repeat_count(ses);
 
 	if (!steps)
 		steps = get_opt_int("document.browse.scrolling.vertical_step");
@@ -353,9 +344,7 @@ scroll_up(struct session *ses, struct document_view *doc_view)
 enum frame_event_status
 scroll_down(struct session *ses, struct document_view *doc_view)
 {
-	int steps = ses->kbdprefix.repeat_count;
-
-	ses->kbdprefix.repeat_count = 0;
+	int steps = eat_kbd_repeat_count(ses);
 
 	if (!steps)
 		steps = get_opt_int("document.browse.scrolling.vertical_step");
@@ -366,9 +355,7 @@ scroll_down(struct session *ses, struct document_view *doc_view)
 enum frame_event_status
 scroll_left(struct session *ses, struct document_view *doc_view)
 {
-	int steps = ses->kbdprefix.repeat_count;
-
-	ses->kbdprefix.repeat_count = 0;
+	int steps = eat_kbd_repeat_count(ses);
 
 	if (!steps)
 		steps = get_opt_int("document.browse.scrolling.horizontal_step");
@@ -379,9 +366,7 @@ scroll_left(struct session *ses, struct document_view *doc_view)
 enum frame_event_status
 scroll_right(struct session *ses, struct document_view *doc_view)
 {
-	int steps = ses->kbdprefix.repeat_count;
-
-	ses->kbdprefix.repeat_count = 0;
+	int steps = eat_kbd_repeat_count(ses);
 
 	if (!steps)
 		steps = get_opt_int("document.browse.scrolling.horizontal_step");
@@ -574,11 +559,9 @@ enum frame_event_status
 move_cursor_rel(struct session *ses, struct document_view *view,
 	        int rx, int ry)
 {
-	int count = ses->kbdprefix.repeat_count;
+	int count = eat_kbd_repeat_count(ses);
 	int x, y;
 
-	ses->kbdprefix.repeat_count = 0;
-
 	int_lower_bound(&count, 1);
 
 	x = ses->tab->x + rx*count;
@@ -641,12 +624,10 @@ copy_current_link_to_clipboard(struct session *ses,
 int
 try_jump_to_link_number(struct session *ses, struct document_view *doc_view)
 {
-	int link_number = ses->kbdprefix.repeat_count - 1;
+	int link_number = eat_kbd_repeat_count(ses) - 1;
 
 	if (link_number < 0) return 1;
 
-	ses->kbdprefix.repeat_count = 0;
-
 	if (!doc_view) return 0;
 
 	if (link_number >= doc_view->document->nlinks)
@@ -702,7 +683,7 @@ try_prefix_key(struct session *ses, struct document_view *doc_view,
 	    || ses->kbdprefix.repeat_count /* The user has already begun
 	                                    * entering a prefix. */
 	    || !doc_opts->num_links_key
-	    || (doc_opts->num_links_key == 1 && !doc_opts->num_links_display)) {
+	    || (doc_opts->num_links_key == 1 && !doc_opts->links_numbering)) {
 		/* Repeat count.
 		 * ses->kbdprefix.repeat_count is initialized to zero
 		 * the first time by init_session() calloc() call.
@@ -742,13 +723,37 @@ try_prefix_key(struct session *ses, struct document_view *doc_view,
 	return FRAME_EVENT_IGNORED;
 }
 
+static enum frame_event_status
+try_form_insert_mode(struct session *ses, struct document_view *doc_view,
+		     struct link *link, struct term_event *ev)
+{
+	enum frame_event_status status = FRAME_EVENT_IGNORED;
+	enum edit_action action_id;
+
+	if (!link_is_textinput(link))
+		return FRAME_EVENT_IGNORED;
+
+	action_id = kbd_action(KEYMAP_EDIT, ev, NULL);
+
+	if (ses->insert_mode == INSERT_MODE_OFF) {
+		if (action_id == ACT_EDIT_ENTER) {
+			ses->insert_mode = INSERT_MODE_ON;
+			status = FRAME_EVENT_REFRESH;
+		}
+	}
+
+	return status;
+}
+
 static enum frame_event_status
 try_form_action(struct session *ses, struct document_view *doc_view,
 		struct link *link, struct term_event *ev)
 {
 	enum frame_event_status status;
 
-	if (!link || !link_is_textinput(link))
+	assert(link);
+
+	if (!link_is_textinput(link))
 		return FRAME_EVENT_IGNORED;
 
 	status = field_op(ses, doc_view, link, ev);
@@ -766,6 +771,17 @@ frame_ev_kbd(struct session *ses, struct document_view *doc_view, struct term_ev
 {
 	enum frame_event_status status = FRAME_EVENT_IGNORED;
 	int accesskey_priority;
+	struct link *link = get_current_link(doc_view);
+
+	if (link) {
+		status = try_form_insert_mode(ses, doc_view, link, ev);
+		if (status != FRAME_EVENT_IGNORED)
+			return status;
+
+		status = try_form_action(ses, doc_view, link, ev);
+		if (status != FRAME_EVENT_IGNORED)
+			return status;
+	}
 
 #ifdef CONFIG_MARKS
 	status = try_mark_key(ses, doc_view, ev);
@@ -879,9 +895,6 @@ frame_ev_mouse(struct session *ses, struct document_view *doc_view, struct term_
 static enum frame_event_status
 frame_ev(struct session *ses, struct document_view *doc_view, struct term_event *ev)
 {
-	struct link *link;
-	enum frame_event_status status;
-
 	assertm(doc_view && doc_view->document, "document not formatted");
 	if_assert_failed return FRAME_EVENT_IGNORED;
 
@@ -891,34 +904,16 @@ frame_ev(struct session *ses, struct document_view *doc_view, struct term_event
 	/* When changing frame, vs may be NULL. See bug 525. */
 	if (!doc_view->vs) return FRAME_EVENT_IGNORED;
 
-	link = get_current_link(doc_view);
-
-	status = try_form_action(ses, doc_view, link, ev);
-	if (status != FRAME_EVENT_IGNORED)
-		return status;
-
-	if (ev->ev == EVENT_KBD) {
-		status = frame_ev_kbd(ses, doc_view, ev);
-
+	switch (ev->ev) {
+	case EVENT_KBD:
+		return frame_ev_kbd(ses, doc_view, ev);
 #ifdef CONFIG_MOUSE
-	} else if (ev->ev == EVENT_MOUSE) {
-		status = frame_ev_mouse(ses, doc_view, ev);
+	case EVENT_MOUSE:
+		return frame_ev_mouse(ses, doc_view, ev);
 #endif /* CONFIG_MOUSE */
-
-	} else {
-		status = FRAME_EVENT_IGNORED;
+	default:
+		return FRAME_EVENT_IGNORED;
 	}
-
-	/* Is this assertion correct ? It is triggered when
-	 * pressing a checkbox or submit in elinks bugzilla.
-	 * XXX: I disabled it for now, please check.
-	 * --Zas
-	if (ses->insert_mode == INSERT_MODE_ON) {
-		assert(link == get_current_link(doc_view));
-	}
-	*/
-
-	return status;
 }
 
 struct document_view *
@@ -1004,9 +999,11 @@ do_mouse_event(struct session *ses, struct term_event *ev,
 
 	if (doc_view != first) draw_formatted(ses, 0);
 
-	copy_struct(&evv, ev);
-	evv.info.mouse.x -= doc_view->box.x;
-	evv.info.mouse.y -= doc_view->box.y;
+	set_mouse_term_event(&evv,
+			     ev->info.mouse.x - doc_view->box.x,
+			     ev->info.mouse.y - doc_view->box.y,
+			     ev->info.mouse.button);
+
 	return send_to_frame(ses, doc_view, &evv);
 }
 
@@ -1087,27 +1084,51 @@ send_mouse_event(struct session *ses, struct document_view *doc_view,
 }
 #endif /* CONFIG_MOUSE */
 
+static void
+try_typeahead(struct session *ses, struct document_view *doc_view,
+              struct term_event *ev, enum main_action action_id)
+{
+	switch (get_opt_int("document.browse.search.typeahead")) {
+		case 0:
+			return;
+		case 1:
+			action_id = ACT_MAIN_SEARCH_TYPEAHEAD_LINK;
+			break;
+		case 2:
+			action_id = ACT_MAIN_SEARCH_TYPEAHEAD_TEXT;
+			break;
+		default:
+			INTERNAL("invalid value for document.browse.search.typeahead");
+	}
+
+	search_typeahead(ses, doc_view, action_id);
+
+	/* Cross your fingers -- I'm just asking
+	 * for an infinite loop! -- Miciah */
+	term_send_event(ses->tab->term, ev);
+}
+
 /* Returns the session if event cleanup should be done or NULL if no cleanup is
  * needed. */
 static struct session *
 send_kbd_event(struct session *ses, struct document_view *doc_view,
 	       struct term_event *ev)
 {
-	int func_ref;
-	enum main_action action;
+	int event;
+	enum main_action action_id;
 
 	if (doc_view && send_to_frame(ses, doc_view, ev) != FRAME_EVENT_IGNORED)
 		return NULL;
 
-	action = kbd_action(KEYMAP_MAIN, ev, &func_ref);
+	action_id = kbd_action(KEYMAP_MAIN, ev, &event);
 
-	if (action == ACT_MAIN_QUIT) {
+	if (action_id == ACT_MAIN_QUIT) {
 		if (check_kbd_key(ev, KBD_CTRL_C))
 quit:
-			action = ACT_MAIN_REALLY_QUIT;
+			action_id = ACT_MAIN_REALLY_QUIT;
 	}
 
-	switch (do_action(ses, action, 0)) {
+	switch (do_action(ses, action_id, 0)) {
 		case FRAME_EVENT_SESSION_DESTROYED:
 			return NULL;
 		case FRAME_EVENT_IGNORED:
@@ -1117,25 +1138,25 @@ quit:
 			return ses;
 	}
 
-	if (action == ACT_MAIN_SCRIPTING_FUNCTION) {
+	if (action_id == ACT_MAIN_SCRIPTING_FUNCTION) {
 #ifdef CONFIG_SCRIPTING
-		trigger_event(func_ref, ses);
+		trigger_event(event, ses);
 #endif
 		return NULL;
 	}
 
 	if (check_kbd_key(ev, KBD_CTRL_C)) goto quit;
 
-	if (get_kbd_modifier(ev) & KBD_ALT) {
+	if (get_kbd_modifier(ev) & KBD_MOD_ALT) {
 		struct window *win;
 
-		get_kbd_modifier(ev) &= ~KBD_ALT;
+		get_kbd_modifier(ev) &= ~KBD_MOD_ALT;
 		activate_bfu_technology(ses, -1);
 		win = ses->tab->term->windows.next;
 		win->handler(win, ev);
 		if (ses->tab->term->windows.next == win) {
 			delete_window(win);
-			get_kbd_modifier(ev) |= KBD_ALT;
+			get_kbd_modifier(ev) |= KBD_MOD_ALT;
 
 			return NULL;
 		}
@@ -1154,27 +1175,8 @@ quit:
 		return ses;
 	}
 
-	if (!(get_kbd_modifier(ev) & KBD_CTRL)) {
-
-		switch (get_opt_int("document.browse.search.typeahead")) {
-			case 0:
-				return NULL;
-			case 1:
-				action = ACT_MAIN_SEARCH_TYPEAHEAD_LINK;
-				break;
-			case 2:
-				action = ACT_MAIN_SEARCH_TYPEAHEAD_TEXT;
-				break;
-			default:
-				INTERNAL("invalid value for document.browse.search.typeahead");
-		}
-
-		/* FIXME: what if !doc_view ? --Zas */
-		search_typeahead(ses, doc_view, action);
-
-		/* Cross your fingers -- I'm just asking
-		 * for an infinite loop! -- Miciah */
-		term_send_event(ses->tab->term, ev);
+	if (!(get_kbd_modifier(ev) & KBD_MOD_CTRL)) {
+		if (doc_view) try_typeahead(ses, doc_view, ev, action_id);
 	}
 
 	return NULL;
@@ -1183,27 +1185,29 @@ quit:
 void
 send_event(struct session *ses, struct term_event *ev)
 {
-	struct document_view *doc_view;
-
 	assert(ses && ev);
 	if_assert_failed return;
-	doc_view = current_frame(ses);
 
 	if (ev->ev == EVENT_KBD) {
+		struct document_view *doc_view = current_frame(ses);
+
 		ses = send_kbd_event(ses, doc_view, ev);
 	}
 #ifdef CONFIG_MOUSE
-	if (ev->ev == EVENT_MOUSE) {
+	else if (ev->ev == EVENT_MOUSE) {
+		struct document_view *doc_view = current_frame(ses);
+
 		ses = send_mouse_event(ses, doc_view, ev);
 	}
 #endif /* CONFIG_MOUSE */
 
-	/* ses may disappear ie. in close_tab() */
+	/* @ses may disappear ie. in close_tab() */
 	if (ses) ses->kbdprefix.repeat_count = 0;
 }
 
 enum frame_event_status
-download_link(struct session *ses, struct document_view *doc_view, int action)
+download_link(struct session *ses, struct document_view *doc_view,
+	      action_id_T action_id)
 {
 	struct link *link = get_current_link(doc_view);
 	void (*download)(void *ses, unsigned char *file) = start_download;
@@ -1215,7 +1219,7 @@ download_link(struct session *ses, struct document_view *doc_view, int action)
 		ses->download_uri = NULL;
 	}
 
-	switch (action) {
+	switch (action_id) {
 		case ACT_MAIN_LINK_DOWNLOAD_RESUME:
 			download = resume_download;
 		case ACT_MAIN_LINK_DOWNLOAD:
